/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool;

import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

public class Parser
{
    DocumentBuilder builder = null;

    public final static String Tag_Application = "SiTool";
    public final static String Tag_Account = "Account";
    public final static String Tag_Module_SaveCalc = "SaveCalc";
    public final static String Tag_Module_Technology = "Technology";
    public final static String Tag_DriveTech = "DriveTech";
    public final static String Tag_EcoTech = "EcoTech";
    public final static String Tag_OtherTech = "OtherTech";
    public final static String Tag_WeaponTech = "WeaponTech";
    public final static String Tag_Module_Planets = "MyPlanets";
    public final static String Tag_Planet = "Planet";
    public final static String Tag_Resources = "Resources";
    public final static String Tag_Use = "Use";
    public final static String Tag_Utilization = "Utilization";
    public final static String Tag_Buildings = "Buildings";
    public final static String Tag_AstroBuildings = "AstroBuildings";
    public final static String Tag_Fleet = "Fleet";
    public final static String Tag_FlightResources = "inFlightResources";
    public final static String Tag_FlightUse = "inFlightUse";
    public final static String Tag_AstroFleet = "AstroFleet";
    public final static String Tag_TotalFleet = "TotalFleet";
    public final static String Tag_Defense = "Defense";
    public final static String Tag_AstroDefense = "AstroDefense";
    public final static String Tag_BuildlistItem = "BuildlistItem";
    public final static String Tag_TechlistItem = "TechlistItem";
    public final static String Tag_SaveTarget = "SaveTarget";

    public final static String Attribute_Name = "name";
    public final static String Attribute_LaF = "laf";
    public final static String Attribute_Width = "width";
    public final static String Attribute_Height = "height";
    public final static String Attribute_Default = "default";
    public final static String Attribute_Language = "language";
    public final static String Attribute_Galaxy = "galaxy";
    public final static String Attribute_System = "system";
    public final static String Attribute_Planet = "planet";
    public final static String Attribute_Size = "size";
    public final static String Attribute_AstroSize = "astroSize";
    public final static String Attribute_Mine = "mine";
    public final static String Attribute_Ship = "ship";
    public final static String Attribute_Speed = "speed";
    public final static String Attribute_FleetSpeed = "fleetspeed";
    public final static String Attribute_SpeedUp = "speedup";
    public final static String Attribute_GalaxyTime = "galaxytime";
    public final static String Attribute_Time = "time";
    public final static String Attribute_Orbit = "orbit";
    public final static String Attribute_Start = "start";
    public final static String Attribute_Sync = "sync";
    public final static String Attribute_Position = "position";
    public final static String Attribute_Building = "building";
    public final static String Attribute_Tech = "tech";
    public final static String Attribute_Level = "level";
    public final static String Attribute_Version = "version";
    public final static String Attribute_Origin = "origin";
    public final static String Attribute_Target = "target";
    public final static String Attribute_Duration = "duration";
    public final static String Attribute_Units = "units";
    public final static String Attribute_Account = "account";
    public final static String Attribute_Alien = "alien";
    public final static String Attribute_Specialization_Fleeter = "fleeter";
    public final static String Attribute_Specialization_Miner = "miner";
    public final static String Attribute_Specialization_Researcher = "researcher";
    public final static String Attribute_Rocket = "rocket";
    public final static String Attribute_Diffusion = "diffusion";
    public final static String Attribute_Warp = "warp";
    public final static String Attribute_SpectralMode = "spectral";
    public final static String Attribute_ImportPlanets = "importPlanets";
    public final static String Attribute_ImportSpyReports = "importSpyReports";
    public final static String Attribute_Outpost = "outpost";
    public final static String Attribute_Distance = "distance";

    public static final String Attribute_Productionbonus_Iron = "bonusIron";
    public static final String Attribute_Productionbonus_Metal = "bonusMetal";
    public static final String Attribute_Productionbonus_Kryptonite = "bonusKryptonite";
    public static final String Attribute_Productionbonus_Spice = "bonusSpice";
    public static final String Attribute_Productionbonus_Energy = "bonusEnergy";

    /****************************************
     * Constructor for the Parser object
     *
     * @throws ParserConfigurationException  Description of the Exception
     ***************************************/
    public Parser() throws ParserConfigurationException
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating( false );
        factory.setIgnoringElementContentWhitespace( true );
        builder = factory.newDocumentBuilder();
    }


    /****************************************
     * Gets the document attribute of the Parser object
     *
     * @param is  Description of the Parameter
     * @return    The document value
     ***************************************/
    public Document getDocument( InputStream is )
    {
        Document doc = null;
        try
        {
            doc = builder.parse( is );
        }
        catch ( Exception ex )
        {
            ex.printStackTrace();
        }

        return doc;
    }


    /****************************************
     * Gets the document attribute of the Parser object
     *
     * @return   The document value
     ***************************************/
    public Document getDocument()
    {
        Document doc = null;
        doc = builder.newDocument();
        return doc;
    }

    public void writeDocument( OutputStream os, Document doc )
    {
        try
        {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer;
            transformer = tFactory.newTransformer();
            transformer.setOutputProperty( "indent", "yes" );
            transformer.setOutputProperty( "method", "xml" );

            DOMSource source = new DOMSource( doc );
            StreamResult result = new StreamResult( os );
            transformer.transform( source, result );
        }
		catch (TransformerConfigurationException e)
		{
			//e.printStackTrace();
		}
		catch (TransformerException e)
		{
			//e.printStackTrace();
		}
    }
}
