/****************************************
 *  COPYRIGHT (C) 2020
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

import sitool.resources.ImportExportPane;

/**
 * @author graf
 *
 */
public class ButtonTabbedPane extends JTabbedPane {

    private static final long serialVersionUID = -1342750622729698378L;
    private final Icon closeIcon;
    private final Icon saveIcon;

    public ButtonTabbedPane(final Icon closeIcon) {
        this(closeIcon, null);
    }

    public ButtonTabbedPane(final Icon closeIcon, final Icon saveIcon) {
        this.closeIcon = closeIcon;
        this.saveIcon = saveIcon;
    }

    @Override
    public void addTab(String title, Component component) {
        int index = getTabCount();
        super.addTab( null, component );
        super.setTabComponentAt(index, new TabHeader(title, component));
        setSelectedIndex( index );
    }


    @Override
    public String getTitleAt( int index ) {
        TabHeader header = (TabHeader)getTabComponentAt(index);
        return (header != null) ? header.getTitle() : "";
    }

    @Override
    public void setTitleAt(int index, String title) {
        TabHeader header = (TabHeader)getTabComponentAt(index);
        header.setTitle(title);
    }

    private class TabHeader extends JPanel
    {
        private static final long serialVersionUID = 3265944962645128235L;
        private final JLabel titleComponent;

        public TabHeader(final String title, final Component component) {
            setBorder(new EmptyBorder(2, 0, 0, 0));
            setLayout(new FlowLayout(LEFT, 3, 0));
            setOpaque(false);
            titleComponent = new JLabel(title);
            add(titleComponent);
            add(new JLabel("  "));
            JButton button;
            if(saveIcon != null) {
                button = new JButton( saveIcon );
                button.setBorder(new EmptyBorder(2, 2, 1, 1));
                add(button);
                button.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed( ActionEvent e ) {
                        if( component instanceof ImportExportPane ) {
                            ((ImportExportPane)component).getMainFrame().getConfigData().saveData((ImportExportPane)component);
                        }
                    }
                });
            }
            button = new JButton( closeIcon );
            button.setBorder(new EmptyBorder(2, 2, 1, 1));
            add(button);
            button.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed( ActionEvent e ) {
                    if( component != null ) {
                        ButtonTabbedPane.this.remove(component );
                    }
                }
            });
        }

        public String getTitle() {
            return titleComponent.getText();
        }

        public void setTitle(String title) {
            String oldTitle = titleComponent.getText();
            titleComponent.setText(title);
            if (title == null || oldTitle == null || !title.equals(oldTitle)) {
                revalidate();
                repaint();
            }
        }
    }
}
