/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;

import javax.swing.BorderFactory;
import javax.swing.JLabel;

/**
 * @author graf
 *
 */
public class GradientLabel extends JLabel
{

    private static final long serialVersionUID = 6429143077875410033L;

    /**
     * 
     */
    public GradientLabel()
    {
        super();
        // we draw our own background, so this must not be opaque
        setOpaque( false );
        setBorder( BorderFactory.createEmptyBorder( 0, 8, 0, 8 ) );
    }

    /**
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    @Override
    protected void paintComponent( Graphics g )
    {
        Graphics2D g2 = ( Graphics2D )g;
        Rectangle rect;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        rect = new Rectangle(0, 0, getWidth() - 1, getHeight() - 1 );
        Color bg = getBackground();
        Color color1 = darker( bg );
        Color color2 = lighter( bg );
        GradientPaint paint;
        int split = rect.y + rect.height / 3;
        paint = new GradientPaint( rect.x, rect.y, color1,  rect.x, split, color2, false );
        g2.setPaint( paint );
        g2.fillRect(rect.x, rect.y, rect.width, split);
        paint = new GradientPaint( rect.x, split, color2,  rect.x, rect.y + rect.height, color1, false );
        g2.setPaint( paint );
        g2.fillRect(rect.x, split, rect.width, rect.height - split);
        g2.setPaint( color1 );
        g2.drawRect( rect.x, rect.y, rect.width, rect.height  );
        super.paintComponent( g );
    }
    
    public Color darker( Color c ) 
    {
        double FACTOR = 0.8;
        return new Color(
                 (int)( c.getRed()   * FACTOR ), 
                 (int)( c.getGreen() * FACTOR ),
                 (int)( c.getBlue()  * FACTOR ),
                 c.getAlpha()
            );
    }

    public Color lighter( Color c ) 
    {
        double FACTOR = 1.9;
        return new Color(
                 Math.min((int)( c.getRed()   * FACTOR ), 255 ), 
                 Math.min((int)( c.getGreen() * FACTOR ), 255 ),
                 Math.min((int)( c.getBlue()  * FACTOR ), 255 ),
                 c.getAlpha()
            );
    }
    
}
