/****************************************
 *  COPYRIGHT (C) 2006, 2022
 *  Holger Graf
 ****************************************/
package sitool.components;

/**
 * @author graf
 *
 */
public class ListItem<T extends Enum<T>>
{
    final private Planet planet;
    final private boolean isAstro;
    final private T item;
    private int level;

    public ListItem( Planet planet, T item, int level ) {
        this(planet, false, item, level);
    }

    public ListItem( Planet planet, boolean isAstro, T item, int level ) {
        super();
        this.planet = planet;
        this.isAstro = isAstro;
        this.item = item;
        this.level = level;
    }

    @Override
    public boolean equals( Object obj )
    {
        boolean rv = false;
        if( obj instanceof ListItem )
        {
            @SuppressWarnings("unchecked")
            ListItem<T> that = ( ListItem<T> )obj;
            rv = this.planet.equals( that.planet );
            rv &= ( this.isAstro == that.isAstro );
            rv &= ( this.item == that.item );
            rv &= ( this.level == that.level );
        }
        return rv;
    }

    @Override
    public int hashCode() {
        return ( item.ordinal() << 16 ) + level;
    }

    public Long getKey()
    {
        long key;

        Position position = planet.getPosition();
        key = position.getGalaxy();
        key *= 1000;
        key += position.getSystem();
        key *= 100;
        key += position.getPlanet();
        key *= 10;
        key += ( isAstro ) ? 1 : 0;

        return Long.valueOf( key );
    }

    public Planet getPlanet() {
        return planet;
    }

    public T getItem() {
        return item;
    }

    public int getLevel() {
        return level;
    }

    public boolean isAstro() {
        return isAstro;
    }

    public void increaseLevel() {
        level++;
    }

}
