package sitool.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.FocusListener;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;


public class PositionField extends JComponent
{
	private static final long serialVersionUID = 7486663925998614807L;
	private JTextField galaxyField = new JTextField( 4 );
    private JTextField systemField = new JTextField( 4 );
    private JTextField planetField = new JTextField( 4 );

	public PositionField()
	{
		super();
		super.setLayout( new GridBagLayout() );

        galaxyField.setHorizontalAlignment( SwingConstants.RIGHT );
        systemField.setHorizontalAlignment( SwingConstants.RIGHT );
        planetField.setHorizontalAlignment( SwingConstants.RIGHT );

        // initialize constraints
		GridBagConstraints fieldConstraints = new GridBagConstraints();
        fieldConstraints.gridheight = 1;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.fill = GridBagConstraints.BOTH;
        fieldConstraints.anchor = GridBagConstraints.CENTER;
        fieldConstraints.weighty = 1.0;

        fieldConstraints.weightx = 1.0;
        add( galaxyField, fieldConstraints );
        fieldConstraints.weightx = 0.0;
        add( new JLabel( ":" ), fieldConstraints );
        fieldConstraints.weightx = 1.0;
        add( systemField, fieldConstraints );
        fieldConstraints.weightx = 0.0;
        add( new JLabel( ":" ), fieldConstraints );
        fieldConstraints.weightx = 1.0;
        add( planetField, fieldConstraints );
        
        FocusListener l = new TextFieldFocusListener();
        galaxyField.addFocusListener( l );
        systemField.addFocusListener( l );
        planetField.addFocusListener( l );
	}

	public Position getPosition()
	{
        int galaxy = getFieldvalue(galaxyField);
        int system = getFieldvalue(systemField);
        int planet = getFieldvalue(planetField);
		
        return new Position( galaxy, system, planet );
	}

	public void setPosition( Position position )
	{
		if( position.isRealPosition() )
		{
			galaxyField.setText( Integer.toString( position.getGalaxy() ) );
			systemField.setText( Integer.toString( position.getSystem() ) );
			planetField.setText( Integer.toString( position.getPlanet() ) );
		}
		else
		{
			galaxyField.setText( "" );
			systemField.setText( "" );
			planetField.setText( "" );
		}
	}
	
    public void addFocusListener( FocusListener l )
    {
        galaxyField.addFocusListener( l );
        systemField.addFocusListener( l );
        planetField.addFocusListener( l );
    }
    
    public void removeFocusListener( FocusListener l )
    {
        galaxyField.removeFocusListener( l );
        systemField.removeFocusListener( l );
        planetField.removeFocusListener( l );
    }
    
	private int getFieldvalue( JTextField field )
	{
		int rv = 0;
		try
		{
			rv = Integer.parseInt( field.getText() );
		}
		catch( NumberFormatException ex )
		{}
		return rv;
	}
}
