/****************************************
 *  COPYRIGHT (C) 2022
 *  Holger Graf
 ****************************************/
package sitool.components;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;

import sitool.ConfigData;

/**
 * @author graf
 *
 */
public class ProductionBonusBox extends JDialog {

    private static final long serialVersionUID = -8399215445703695460L;

    private final JLabel[] labels = new JLabel[Resource.values().length + 1];
    private final JSpinner[] spinner = new JSpinner[Resource.values().length + 1];

    private JButton okButton = new JButton("wwwwww");
    private JButton cancelButton = new JButton("wwwwwwwww");

    private TableCellEditor editor = null;

    private final ProductionBonus productionBonus = new ProductionBonus();
    private final ConfigData configData;

    public ProductionBonusBox( ConfigData configData, Dialog owner, boolean modal ) {
        super(owner, modal);
        this.configData = configData;
        setIconImage(new ImageIcon(getClass().getResource( "/symbol.png" )).getImage());
        setDefaultCloseOperation(HIDE_ON_CLOSE);
        for( int i = 0; i < labels.length; i++) {
            labels[i] = new JLabel();
            spinner[i] = new JSpinner(new SpinnerNumberModel(0, 0, 15, 3));
        }
        getContentPane().setLayout(new BorderLayout(5, 5));
        ((JPanel)getContentPane()).setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        JPanel centerPane = new JPanel(new GridLayout(labels.length, 2, 5, 5));
        centerPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        for( int i = 0; i < labels.length; i++) {
            labels[i] = new JLabel();
            spinner[i] = new JSpinner(new SpinnerNumberModel(0, 0, 15, 3));
            centerPane.add(labels[i]);
            centerPane.add(spinner[i]);
        }
        getContentPane().add(centerPane, BorderLayout.CENTER);
        JPanel buttonPane = new JPanel(new GridLayout(1, 2, 5, 5));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 15, 5, 15));
        okButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed( ActionEvent e ) {
                ok();
            }
        });
        cancelButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed( ActionEvent e ) {
                cancel();
            }
        });
        buttonPane.add(okButton);
        buttonPane.add(cancelButton);
        getContentPane().add( buttonPane, BorderLayout.SOUTH );
        pack();
    }

    public void show(TableCellEditor editor, ProductionBonus bonus, Point point) {
        this.editor = editor;
        spinner[0].setValue(bonus.getIron());
        spinner[1].setValue(bonus.getMetal());
        spinner[2].setValue(bonus.getKryptonite());
        spinner[3].setValue(bonus.getSpice());
        spinner[4].setValue(bonus.getEnergy());
        transferValues();
        updateLabel();
        setLocation(point);
        setVisible(true);
    }

    public void cancel() {
        setVisible(false);
        if(editor != null) {
            editor.cancelCellEditing();
        }
        editor = null;
    }

    public ProductionBonus getProductionBonus() {
        return productionBonus;
    }

    private void transferValues() {
        productionBonus.setIron(((Number)spinner[0].getValue()).intValue());
        productionBonus.setMetal(((Number)spinner[1].getValue()).intValue());
        productionBonus.setKryptonite(((Number)spinner[2].getValue()).intValue());
        productionBonus.setSpice(((Number)spinner[3].getValue()).intValue());
        productionBonus.setEnergy(((Number)spinner[4].getValue()).intValue());
    }

    private void ok() {
        transferValues();
        setVisible(false);
        if(editor != null) {
            editor.stopCellEditing();
        }
        editor = null;
    }

    private void updateLabel() {
        setTitle(configData.getI18nText("Account.productionBonus"));
        for(int i = 0; i < Resource.values().length; i++) {
            labels[i].setText(configData.getI18nText(Resource.values()[i].name()));
        }
        labels[labels.length - 1].setText(configData.getI18nText("Energy"));
        okButton.setText(configData.getI18nText("OK"));
        cancelButton.setText(configData.getI18nText("Cancel"));
    }
}
