/****************************************
 *  COPYRIGHT (C) 2020
 *  Holger Graf
 ****************************************/
package sitool.components;

/**
 * @author graf
 *
 */
public class Specialization {

    private int fleeter = 0;
    private int miner = 0;
    private int researcher = 0;

    private final int[] speedbonus = { 0, 10, 20, 30, 40 };
    private final int[] recyclerbonus = { 0, 5, 10, 15, 20 };
    private final int[] farmking = { 0, 5, 10, 20, 30 };
    private final int[] productionbonus = { 0, 3, 6, 10, 15 };
    private final int[] capacitybonus = { 0, 5, 10, 20, 30 };
    private final int[] defensebonus = { 0, 3, 7, 15, 20 };

    public Specialization() {
    }

    public Specialization(Specialization that) {
        this.fleeter = that.getFleeter();
        this.miner = that.getMiner();
        this.researcher = that.getResearcher();
    }

    /**
     * @return the fleeter
     */
    public int getFleeter() {
        return fleeter;
    }

    /**
     * @param fleeter the fleeter to set
     */
    public void setFleeter( int fleeter ) {
        this.fleeter = normalize(fleeter);
    }

    /**
     * @return the miner
     */
    public int getMiner() {
        return miner;
    }

    /**
     * @param miner the miner to set
     */
    public void setMiner( int miner ) {
        this.miner = normalize(miner);
    }

    /**
     * @return the researcher
     */
    public int getResearcher() {
        return researcher;
    }

    /**
     * @param researcher the researcher to set
     */
    public void setResearcher( int researcher ) {
        this.researcher = normalize(researcher);
    }

    public int getSpeedbonus(Ship ship) {
        return speedbonus[fleeter] + ((ship == Ship.recycler) ? recyclerbonus[fleeter] : 0);
    }

    public int getFarmking() {
        return farmking[fleeter];
    }

    public int getProductionbonus() {
        return productionbonus[miner];
    }

    public int getCapacitybonus() {
        return capacitybonus[miner];
    }

    public int getDefensebonus() {
        return defensebonus[miner];
    }

    public void setValue(Specialization value) {
        setMiner(value.getMiner());
        setFleeter(value.getFleeter());
        setResearcher(value.getResearcher());
    }

    private int normalize(int value) {
        if(value < 0) {
            value = 0;
        }
        else if(value > 4) {
            value = 4;
        }
        return value;
    }

    private boolean isSet() {
        return getFleeter() + getMiner() + getResearcher() != 0;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if(isSet()) {
            sb.append("[ ")
            .append(getMiner())
            .append(" | ")
            .append(getFleeter())
            .append(" | ")
            .append(getResearcher())
              .append(" ]");
        }
        return sb.toString();
    }

}
