/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.components;

import sitool.resources.ResourceCalculator;

/**
 * @author graf
 *
 */
public class Stage
{
    private int level;
    final private Building building;

    public Stage(Building building, int level)
    {
        this.level = level;
        this.building = building;
    }

    public Stage(Stage other)
    {
        this(other.building, other.level);
    }


    /**
     * @return the level
     */
    public int getLevel()
    {
        return level;
    }
    /**
     * @param level the level to set
     */
    public void setLevel( int level )
    {
        this.level = level;
    }
    /**
     * @return the production
     */
    public double getProduction(int temperature, Version version)
    {
        return ResourceCalculator.calculateProduction( building, level, temperature, version );
    }

    /**
     * @return the capacity
     */
    public long getCapacity()
    {
        return ResourceCalculator.calculateCapacity( building, level );
    }

    /**
     * @return the consumption
     */
    public double getConsumption()
    {
        return ResourceCalculator.calculateConsumption( building, level );
    }

    /**
     * @return the cost
     */
    public Cost getCost()
    {
        return ResourceCalculator.calculateCost( building, level );
    }

    @Override
    public String toString() {
        return new StringBuilder()
                        .append(building.name())
                        .append(": ")
                        .append(level)
                        .toString();
    }

}
