/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.fleet;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.CostReduction;
import sitool.components.EnumWrapper;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Stage;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.BuildingTableCellRenderer;
import sitool.table.FleetBuilderTableModel;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.SpinnerCellEditor;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class FleetBuilderPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -8045686423181593884L;

    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JComboBox<Planet> myPlanets = new JComboBox<>();
    private JCheckBox astro = new JCheckBox();
    private JCheckBox engineer = new JCheckBox();
    private JSpinner weaponsFactory = new JSpinner();
    private JSpinner msa = new JSpinner();
    private JComboBox<EnumWrapper<CostReduction>> costReduction = new JComboBox<>();
    private JPanel center;
    private JTable table;
    private FleetBuilderTableModel model;

    private JLabel planetLabel = new JLabel();
    private JLabel astroLabel = new JLabel();
    private JLabel weaponsLabel = new JLabel();
    private JLabel msaLabel = new JLabel();
    private JLabel engineerLabel = new JLabel();
    //private JLabel costReductionLable = new JLabel();

    public FleetBuilderPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateLabel();
        updateMyPlanets();
        updateTable();
        adjustColumns();
    }

    private void updateLabel()
    {
        planetLabel.setText( configData.getI18nText( "Planet" ) );
        astroLabel.setText( configData.getI18nText( "Asteroid" ) );
        weaponsLabel.setText( configData.getI18nText( "WeaponsFact.level", configData.getI18nText( Building.weaponsFactory.name() ) ) );
        msaLabel.setText( configData.getI18nText( "Msa.level" ) );
        engineerLabel.setText( configData.getI18nText( "fleet.engineer" ) );
        model.fireTableStructureChanged();
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        weaponsFactory.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );
        msa.setModel( new SpinnerNumberModel( 0, 0, 150, 1 ) );

        JPanel west = new JPanel( new GridBagLayout() );
        center.add( west, BorderLayout.WEST );
        west.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.anchor = GridBagConstraints.NORTH;
        constraints.weightx = 1.0;
        constraints.weighty = 0.0;
        constraints.insets = new Insets( 2, 2, 2, 2 );

        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( planetLabel, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astroLabel, constraints );
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( myPlanets, constraints );
        constraints.gridx = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( astro, constraints );

        constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( weaponsLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( weaponsFactory, constraints );
        constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( msaLabel, constraints );
        constraints.gridx = 1;
        constraints.gridwidth = 3;
        constraints.weightx = 1.0;
        west.add( msa, constraints );
        constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 1;
        constraints.weightx = 0.0;
        west.add( engineerLabel, constraints );
        constraints.gridx = 2;
        constraints.gridwidth = 2;
        constraints.weightx = 0.0;
        west.add( engineer, constraints );
        constraints.gridx = 0;
        constraints.gridy = 5;
        constraints.gridwidth = 4;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        west.add( costReduction, constraints );
        constraints.gridx = 0;
        constraints.gridy = 6;
        constraints.gridwidth = 4;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        constraints.gridheight = GridBagConstraints.REMAINDER;
        west.add( new JLabel(), constraints );

        model = new FleetBuilderTableModel( mainFrame );
        final TableCellRenderer headerRenderer = new BlindTableCellRenderer();
        final BuildingTableCellRenderer rowRenderer = new BuildingTableCellRenderer();
        final NumericSpinnerTableCellRenderer columnrenderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) );
        final TableCellEditor editor = new SpinnerCellEditor( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) );
        editor.addCellEditorListener( model );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = 7149497989459592130L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 )
                {
                    rv = headerRenderer;
                }
                else if( row >= FleetBuilderTableModel.rows )
                {
                    rv = rowRenderer;
                }
                else if ( modelColumn == 1 )
                {
                    rv = columnrenderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }

            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 1 )
                {
                    rv = editor;
                }
                else
                {
                    rv = super.getCellEditor( row, column );
                }
                return rv;
            }
        };
        AlternateLineTableCellRenderer renderer = new AlternateLineTableCellRenderer();
        renderer.setHorizontalAlignment( SwingConstants.RIGHT );
        table.setDefaultRenderer( Object.class, renderer );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        myPlanets.addItemListener( new ItemListener() {

            public void itemStateChanged( ItemEvent e )
            {
                actionPlanetChanged( e );
            }

        });
        weaponsFactory.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setWeaponsfactoryLevel( level );
                adjustColumns();
            }

        });
        msa.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                JSpinner spinner = ( JSpinner )e.getSource();
                int level = ((Number)spinner.getValue()).intValue();
                model.setMsaLevel( level );
                adjustColumns();
            }

        });
        astro.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isAstro = ( e.getStateChange() == ItemEvent.SELECTED );
                Planet planet = ( Planet )myPlanets.getSelectedItem();
                int level;
                Stage stage;
                stage = planet.getStage( Building.weaponsFactory, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                weaponsFactory.setValue( level );
                stage = planet.getStage( Building.msa, isAstro );
                level = ( stage != null ) ? stage.getLevel() : 0;
                msa.setValue( level );
            }
        });
        engineer.addItemListener( new ItemListener()
        {
            public void itemStateChanged( ItemEvent e )
            {
                boolean isEngineer = ( e.getStateChange() == ItemEvent.SELECTED );
                model.setEngineer( isEngineer );
                costReduction.setEnabled(isEngineer);
                if( !isEngineer ) {
                    costReduction.setSelectedIndex(0);
                }
            }
        });

        for(CostReduction cr : CostReduction.values()) {
            costReduction.addItem(new EnumWrapper<CostReduction>(configData, cr));
        }

        costReduction.setEnabled(false);
        costReduction.addItemListener(new ItemListener() {
            @SuppressWarnings("unchecked")
            @Override
            public void itemStateChanged( ItemEvent e ) {
                if( e.getStateChange() == ItemEvent.SELECTED ) {
                    model.setCostReduction(((EnumWrapper<CostReduction>)costReduction.getSelectedItem()).getBase());
                }
            }
        });
        return center;
    }

    private void updateTable()
    {
        ItemEvent event;
        event = new ItemEvent( myPlanets, 0, myPlanets.getSelectedItem(), ItemEvent.SELECTED );
        actionPlanetChanged( event );
    }

    private void updateMyPlanets()
    {
        Object selection = myPlanets.getSelectedItem();
        myPlanets.removeAllItems();
        myPlanets.addItem( new Planet( "-- -- --", new Position( 0, 0, 0) ) );
        List<Planet> ownPlanets = new ArrayList<Planet>( configData.getMyPlanets() );
        for( Planet planet : ownPlanets )
        {
            myPlanets.addItem( planet );
        }
        // restore selection
        if( selection != null )
        {
            myPlanets.setSelectedItem( selection );
        }
        else
        {
            myPlanets.setSelectedIndex( 0 );
        }
        myPlanets.setMaximumRowCount( myPlanets.getItemCount() );
    }

    protected void actionPlanetChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Planet planet = ( Planet )e.getItem();
            astro.setSelected( false );
            int level;
            Stage stage;
            stage = planet.getStage( Building.weaponsFactory, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            weaponsFactory.setValue( level );
            stage = planet.getStage( Building.msa, false );
            level = ( stage != null ) ? stage.getLevel() : 0;
            msa.setValue( level );
        }
    }

    private void adjustColumns()
    {
        int width = 165;
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column = columnModel.getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        width = 55;
        column = columnModel.getColumn( 1 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
//        column.setMaxWidth( width );
        width = 88;
        column = columnModel.getColumn( 2 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        width = 45;
        column = columnModel.getColumn( columnModel.getColumnCount() - 2 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
        width = 60;
        column = columnModel.getColumn( columnModel.getColumnCount() - 1 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
    }
}
