/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.fleet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.GradientLabel;
import sitool.components.ModulePane;
import sitool.components.TotalPointSource;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.BuildingTableCellRenderer;
import sitool.table.GradientTableCellRenderer;
import sitool.table.MultiLineTableCellHeaderRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.PlanetTableHeaderMouseListener;
import sitool.table.ShipTableModel;
import sitool.table.SpinnerCellEditor;
import sitool.table.TooltipTable;
import sitool.table.TotalShipTableModel;

/**
 *
 * @author graf
 *
 */
public class ShipPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private MainFrame mainFrame;
    private JPanel center;
    private JTable table;
    private JPanel totalShips;
    private JLabel totalShipCaption;
    private ShipTableModel model;
    private TotalShipTableModel totalModel;
    private boolean isAsteroid;

	public ShipPane( MainFrame parent, boolean isAsteroid )
    {
        mainFrame = parent;
        this.isAsteroid = isAsteroid;
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        ConfigData configData = mainFrame.getConfigData();
        totalShipCaption.setText(configData.getI18nText("fleet.all.ships"));
        if( configData.getActiveAccount() != null )
        {
            if( !isAsteroid || configData.getMyAsteroids().size() > 0 )
            {
                center.setVisible( true );
                updateTable();
            }
            else
            {
                center.setVisible( false );
            }
            totalShips.setVisible(configData.getActiveAccount().hasTotalShips());
            totalModel.fireTableDataChanged();
        }
        else
        {
            center.setVisible( false );
        }
    }

    private void updateTable()
    {
        model.fireTableStructureChanged();

        // define column width and row height
        int rowHeight = new JComboBox<>().getPreferredSize().height - 3;
        table.setRowHeight( rowHeight );
        // fix the first column
        int width = 155;
        TableColumn column = table.getColumnModel().getColumn( 0 );
        column.setPreferredWidth( width );
        column.setMinWidth( width );
        column.setMaxWidth( width );
//        width = 60;
//        column = table.getColumnModel().getColumn( table.convertColumnIndexToView( model.getColumnCount() - 1 ) );
//        column.setPreferredWidth( width );
//        column.setMinWidth( width );
//        column.setMaxWidth( width );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        final TableCellRenderer headerRenderer = new BlindTableCellRenderer();
        final TableCellRenderer totalRowRenderer = new BuildingTableCellRenderer();
        final GradientTableCellRenderer capacityRowRenderer = new GradientTableCellRenderer();
        final AlternateLineTableCellRenderer columnRenderer = new AlternateLineTableCellRenderer();
        capacityRowRenderer.setHorizontalAlignment( SwingConstants.CENTER );
        capacityRowRenderer.setBackground( Color.white );
        columnRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        JScrollPane scrollPane;
        model = new ShipTableModel( mainFrame, isAsteroid );
        TotalPointSource.Type type = ( isAsteroid ) ? TotalPointSource.Type.AstroShips : TotalPointSource.Type.Ships;
        mainFrame.registerTotalPointSource( type, model );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = 7149497989459592130L;
            @Override
            public Dimension getPreferredScrollableViewportSize() {
                Dimension size = this.getPreferredSize();
                size.height += 28;
                return size;
            }

            @Override
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 )
                {
                    rv = headerRenderer;
                }
                else if( row == model.getRowCount() - 2  )
                {
                    rv = capacityRowRenderer;
                }
                else if( row > model.getRowCount() - 2  )
                {
                    rv = totalRowRenderer;
                }
                else if ( modelColumn == model.getColumnCount() - 1 )
                {
                    rv = columnRenderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }
        };
        JTableHeader header = table.getTableHeader();
        header.addMouseListener( new PlanetTableHeaderMouseListener( PlanetTableHeaderMouseListener.Type.Ship ) );
        TableCellRenderer renderer = new MultiLineTableCellHeaderRenderer( header.getDefaultRenderer() );
        header.setDefaultRenderer( renderer );
        table.setDefaultRenderer( Number.class, new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) ) );
        TableCellEditor editor = new SpinnerCellEditor( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) );
        editor.addCellEditorListener( model );
        table.setDefaultEditor( Number.class, editor );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_NEVER,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.NORTH );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 0 ) );
        totalShips = new JPanel(new BorderLayout());
        //totalShips.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        totalModel = new TotalShipTableModel(mainFrame);
        mainFrame.registerTotalPointSource( TotalPointSource.Type.Tactical, totalModel );
        JTable totalTable = new TooltipTable(totalModel) {
            private static final long serialVersionUID = -5582226176724877642L;

            @Override
            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer rv;
                int modelRow = convertRowIndexToModel( row );
                if( modelRow % 2 == 0) {
                    rv = totalRowRenderer;
                }
                else {
                    rv = capacityRowRenderer;
                }
                return rv;
            }

        };
        totalShipCaption = new GradientLabel() {
            private static final long serialVersionUID = -1542667594895321814L;

            @Override
            public Dimension getPreferredSize() {
                return new JComboBox<>().getPreferredSize();
            }

        };
        totalShips.add(totalShipCaption, BorderLayout.NORTH);
        totalShips.add(totalTable, BorderLayout.CENTER);
        center.add( totalShips, BorderLayout.CENTER );

        return center;
    }

}
