/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.fleet;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.BuildingTableCellRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.SpinnerCellEditor;
import sitool.table.TacticalFleetStrengthTableModel;
import sitool.table.TooltipTable;

/**
 * @author graf
 *
 */
public class TacticalStrengthPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -8045686423181593884L;

    final private MainFrame mainFrame;
    private JPanel center;
    private JTable table;
    private JButton sync = new JButton();
    private JButton reset = new JButton();
    private TacticalFleetStrengthTableModel model;


    public TacticalStrengthPane( MainFrame parent )
    {
        mainFrame = parent;
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateLabel();
        updateTable();
    }

    private void updateLabel()
    {
        ConfigData configData = mainFrame.getConfigData();
        sync.setText(configData.getI18nText("Tactical.sync"));
        reset.setText(configData.getI18nText("Tactical.reset"));
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );

        JPanel north = new JPanel( new BorderLayout() );
        north.setBorder(new EmptyBorder(10, 10, 10, 10));
        JPanel buttons = new JPanel(new GridLayout(1, 2, 10, 5));
        buttons.add(sync);
        buttons.add(reset);
        sync.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed( ActionEvent e ) {
                actionFillValues();
            }
        });
        reset.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed( ActionEvent e ) {
                actionResetValues();
            }
        });
        north.add(buttons, BorderLayout.EAST);
        center.add(north, BorderLayout.NORTH);
        model = new TacticalFleetStrengthTableModel( mainFrame );
        final TableCellRenderer headerRenderer = new BlindTableCellRenderer();
        final BuildingTableCellRenderer rowRenderer = new BuildingTableCellRenderer();
        final NumericSpinnerTableCellRenderer columnrenderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) );
        final TableCellEditor editor = new SpinnerCellEditor( new SpinnerNumberModel( Long.valueOf(0), Long.valueOf(0), Long.valueOf(Long.MAX_VALUE), Long.valueOf(1) ) );
        editor.addCellEditorListener( model );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = 7149497989459592130L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 )
                {
                    rv = headerRenderer;
                }
                else if( row >= TacticalFleetStrengthTableModel.rows )
                {
                    rv = rowRenderer;
                }
                else if ( modelColumn == 2 )
                {
                    rv = columnrenderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }

            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 2 )
                {
                    rv = editor;
                }
                else
                {
                    rv = super.getCellEditor( row, column );
                }
                return rv;
            }
        };
        AlternateLineTableCellRenderer renderer = new AlternateLineTableCellRenderer();
        renderer.setHorizontalAlignment( SwingConstants.RIGHT );
        table.setDefaultRenderer( Object.class, renderer );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        table.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "resetValues");
        table.getActionMap().put("resetValues", new AbstractAction() {
            private static final long serialVersionUID = 5999113634471333070L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                actionResetValues();
            }
        });
        table.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_DOWN_MASK), "fillValues");
        table.getActionMap().put("fillValues", new AbstractAction() {
            private static final long serialVersionUID = 6340761181404378574L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                actionFillValues();
            }
        });


        JScrollPane scrollPane = new JScrollPane( table,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        center.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        return center;
    }

    protected void actionFillValues() {
        model.fillValues();
    }

    protected void actionResetValues() {
        model.resetValues();
    }

    private void updateTable()
    {
        model.update();
        model.fireTableStructureChanged();
        table.requestFocus();
    }

}
