/****************************************
 *  COPYRIGHT (C) 2018
 *  Holger Graf
 ****************************************/
package sitool.json;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.jr.ob.JSON;
import com.fasterxml.jackson.jr.ob.JSONObjectException;

/**
 * @author graf
 *
 */
public class PlanetBean {

    private long id;
    private String coords;
    private PlanetType type;
    private String name;
    private int fields;
    private List<Long> resources;
    private Object energy;
    private List<Long> storage;
    // buildings, ships, defenses and research switch types
    private Map<String, LevelBean> buildings;
    private Map<String, LevelBean> ships;
    private Map<String, LevelBean> defense;
    private Map<String, LevelBean> researches;
    private SuperComputerBean scomputer;
    private TeleporterBean teleporter;
    private PlanetBean astro;


    public PlanetBean() {}


    public long getId() {
        return id;
    }


    public void setId( long id ) {
        this.id = id;
    }


    public String getCoords() {
        return coords;
    }


    public void setCoords( String coords ) {
        this.coords = coords;
    }


    public PlanetType getType() {
        return type;
    }


    public void setType( Object type ) {
        PlanetType planetType;
        try {
            planetType = PlanetType.valueOf(type.toString());
        }
        catch(IllegalArgumentException | NullPointerException ex) {
            System.err.println(ex.getMessage());
            planetType = PlanetType.normal;
        }
        this.type = planetType;
    }


    public String getName() {
        return name;
    }


    public void setName( String name ) {
        this.name = name;
    }


    public int getFields() {
        return fields;
    }


    public void setFields( int fields ) {
        this.fields = fields;
    }


    public List<Long> getResources() {
        return resources;
    }


    public void setResources( List<Long> resources ) {
        this.resources = resources;
    }


    public Object getEnergy() {
        return energy;
    }


    public void setEnergy( Object energy ) {
        this.energy = energy;
    }


    public List<Long> getStorage() {
        return storage;
    }


    public void setStorage( List<Long> storage ) {
        this.storage = storage;
    }


    public Map<String, LevelBean> getBuildings() {
        return buildings;
    }


    public void setBuildings( Object buildings ) throws JSONObjectException, IOException {
        this.buildings = null;
        // there is a difference between not visible (which is a Boolean) and empty (which is a List)
        if(buildings instanceof List) {
            this.buildings = new LinkedHashMap<>();
        }
        else if(buildings instanceof Map) {
            this.buildings = JSON.std.beanFrom(LevelMap.class, JSON.std.asString(buildings));
        }
    }


    public Map<String, LevelBean> getShips() {
        return ships;
    }


    public void setShips( Object ships ) throws JSONObjectException, IOException {
        this.ships = null;
     // there is a difference between not visible (which is a Boolean) and empty (which is a List)
        if(ships instanceof List) {
            this.ships = new LinkedHashMap<>();
        }
        else if(ships instanceof Map) {
            this.ships = JSON.std.beanFrom(LevelMap.class, JSON.std.asString(ships));
        }
    }


    public Map<String, LevelBean> getDefense() {
        return defense;
    }


    public void setDefense( Object defense ) throws JSONObjectException, IOException {
        this.defense = null;
        // there is a difference between not visible (which is a Boolean) and empty (which is a List)
        if(defense instanceof List) {
            this.defense = new LinkedHashMap<>();
        }
        else if(defense instanceof Map) {
            this.defense = JSON.std.beanFrom(LevelMap.class, JSON.std.asString(defense));
        }
    }


    public Map<String, LevelBean> getResearches() {
        return researches;
    }


    public void setResearches( Object researches ) throws JSONObjectException, IOException {
        this.researches = null;
        if(researches instanceof Map) {
            this.researches = JSON.std.beanFrom(LevelMap.class, JSON.std.asString(researches));
        }
    }


    public TeleporterBean getTeleporter() {
        return teleporter;
    }


    public void setTeleporter( TeleporterBean teleporter ) {
        this.teleporter = teleporter;
    }


    public PlanetBean getAstro() {
        return astro;
    }


    public void setAstro( Object astro ) throws JSONObjectException, IOException {
        this.astro = null;
        if(astro instanceof Map) {
            this.astro = JSON.std.beanFrom(PlanetBean.class, JSON.std.asString(astro));
        }
    }


    public SuperComputerBean getScomputer() {
        return scomputer;
    }


    public void setScomputer( SuperComputerBean scomputer ) {
        this.scomputer = scomputer;
    }

}
