/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.planets;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import sitool.Account;
import sitool.ConfigData;
import sitool.components.Coordinates;
import sitool.components.GradientLabel;
import sitool.components.Orbits;
import sitool.components.Position;


/**
 *
 * @author graf
 *
 */
public class CoordinatorBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    private static Position[] positions = new Position[] {
        new Position( "01:01:01" ), new Position( "02:02:02" ), new Position( "03:03:03" ),
        new Position( "04:04:04" ) /*, new Position( "05:05:05" ), new Position( "06:06:06" ) */
    };
    protected static final Color enabled  = new Color( 0x44108810, true);
    protected static final Color disabled = new Color( 0x44881010, true);

    private ConfigData configData;
    private CoordinatorPane parent;

    private boolean read[];
    private GradientLabel[] targetCoordinates;
    private Coordinates[] scanCoordinates;
    private Coordinates[] referenceCoordinates;
    private JButton syncButton;

    private JTextField galaxyTickField = new JTextField( 5 );
    private JTextField tickField = new JTextField( 5 );
    private int currentGalaxyTicks;
    private int currentTicks;
    private int longest;

    private Account current;

    public CoordinatorBox( CoordinatorPane parent )
    {
	    super( parent.getMainFrame() );
	    this.parent = parent;
        configData = parent.getMainFrame().getConfigData();

        current = configData.getActiveAccount();
        currentGalaxyTicks = current.getGalaxyTicks();
        currentTicks = current.getTicks();

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "CoordinatorBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel upper = new JPanel( new FlowLayout( FlowLayout.CENTER, 15, 5 ) );
        JLabel galaxyTickLabel = new JLabel( configData.getI18nText( "Coordinator.galaxyTicks" ) );
        JLabel tickLabel = new JLabel( configData.getI18nText( "Coordinator.ticks" ) );

        upper.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        upper.add( galaxyTickLabel );
        galaxyTickField.setEditable( false );
        galaxyTickField.setHorizontalAlignment( SwingConstants.RIGHT );
        upper.add( galaxyTickField );
        galaxyTickField.setText( Integer.toString( currentGalaxyTicks ) );
        upper.add( tickLabel );
        tickField.setEditable( false );
        tickField.setText( Integer.toString( currentTicks ) );
        tickField.setHorizontalAlignment( SwingConstants.RIGHT );
        upper.add( tickField );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 660, 220 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "Close" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( upper, BorderLayout.NORTH );
        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();

    }

    protected JPanel createCenterPanel()
    {
        JPanel outer = new JPanel( new BorderLayout() );
        JPanel center = new JPanel( new GridLayout( positions.length, 4, 15, 10 ) );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {

            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    ((JTextField)e.getSource()).setText( input );
                }
            }

        };

        read = new boolean[positions.length];
        targetCoordinates = new GradientLabel[positions.length];
        referenceCoordinates = new Coordinates[positions.length];
        scanCoordinates = new Coordinates[positions.length];
        Map<Position, Coordinates> map = configData.getCoordinates();

        int[] orbits = new int[positions.length];
        for( int i = 0; i < positions.length; i++ )
        {
            JLabel pos = new JLabel( positions[i].toString() );
            pos.setHorizontalAlignment( SwingConstants.RIGHT );
            center.add( pos );
            JTextField inputField = new JTextField();
            inputField.addMouseListener( l );
            center.add( inputField );
            JLabel target = new GradientLabel();
            center.add( target );
            targetCoordinates[i] = new GradientLabel();
            Coordinates coordinates = map.get( positions[i] );
            referenceCoordinates[i] = (coordinates != null ) ? coordinates : new Coordinates( "" );
            targetCoordinates[i].setText( configData.translate( referenceCoordinates[i], currentGalaxyTicks, currentTicks, false ).toString() );
            targetCoordinates[i].setBackground( disabled );
            center.add( targetCoordinates[i] );
            addDocumentListener( inputField, target, i );
            if( referenceCoordinates[i].isValid() )
            {
                orbits[i] = Orbits.getOrbit( referenceCoordinates[i].getOrbit() ).length;
            }
        }

        //TODO limit to some sane value
        longest = kgV( orbits );
        //longest = 2000;

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        syncButton = new JButton( configData.getI18nText( "CoordinatorBox.title" ) );
        syncButton.setEnabled( false );
        syncButton.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                parent.setSync( currentGalaxyTicks, currentTicks );
            }
        } );
        lower.add( syncButton );
        outer.add( center, BorderLayout.NORTH );
        outer.add( lower, BorderLayout.SOUTH );
        return outer;
    }

    private void addDocumentListener( final JTextField textField, final JLabel target, final int index )
    {
        textField.getDocument().addDocumentListener( new DocumentListener()
        {

            @Override
            public void removeUpdate( DocumentEvent e )
            {
                updateText( textField, target, index );
            }

            @Override
            public void insertUpdate( DocumentEvent e )
            {
                updateText( textField, target, index );
            }

            @Override
            public void changedUpdate( DocumentEvent e )
            {
                updateText( textField, target, index );
            }
        } );
    }

    protected void updateText( JTextField textField, JLabel target, int index )
    {
        boolean isEnabled = false;
        String text = textField.getText();
        scanCoordinates[index] = new Coordinates( text );
        target.setText( scanCoordinates[index].toString() );

        targetCoordinates[index].setBackground( disabled );
        targetCoordinates[index].setText( configData.translate( referenceCoordinates[index], currentGalaxyTicks, currentTicks, false ).toString() );
        read[index] = false;

        if( scanCoordinates[index].isValid() && referenceCoordinates[index].isValid()
         && referenceCoordinates[index].getPosition().equals( scanCoordinates[index].getPosition() )
         && referenceCoordinates[index].getOrbit() == scanCoordinates[index].getOrbit()
          )
        {
            int offset = 0;
            Point end = scanCoordinates[index].getGalaxy();
            while( !Coordinates.getGalaxyPoint( positions[index], offset ).equals( end ) )
                offset++;
            currentGalaxyTicks = offset;

            offset = 0;
            while( offset < longest )
            {
                while( (offset < longest) && !configData.translate( referenceCoordinates[index], currentGalaxyTicks, offset, false ).equals( scanCoordinates[index] ) )
                    offset++;
                if( offset < longest )
                    read[index] = check( offset );
                if( read[index] )
                    break;
                offset++;
            }

            if( read[index] ) currentTicks = offset;
            isEnabled = true;
            for( int i = 0; i < read.length; i++ )
            {
                isEnabled &= read[i];
                targetCoordinates[i].setBackground( read[i] ? enabled : disabled );
                targetCoordinates[i].setText( configData.translate( referenceCoordinates[i], currentGalaxyTicks, currentTicks, false ).toString() );
            }

            galaxyTickField.setText( Integer.toString( currentGalaxyTicks ) );
            tickField.setText( Integer.toString( currentTicks ) );
        }
        syncButton.setEnabled( isEnabled );
    }

    private boolean check( int offset )
    {
        boolean rv = false;
        boolean test = true;
        for( int i = 0; i < read.length; i++ )
        {
            if( read[i] )
            {
                test &= configData.translate( referenceCoordinates[i], currentGalaxyTicks, offset, false ).equals( scanCoordinates[i] );
            }
        }
        if( test )
            rv = test;
        return rv;
    }

    private int kgV(int a, int b)
    {
        int max = a < b ? b : a ;
        int min = a < b ? a : b ;
        int i;

        for( i = 1 ; (i * max) % min != 0 ; i++ )
            ;

        return i * max;
    }

    private int kgV(int[] is)
    {
        int rv = 0;
        if( is.length > 0)
        {
            rv = is[0] ;
            if (is.length > 1)
            {
                for (int i = 1; i < is.length; i++)
                {
                    rv = kgV (is[i], rv) ;
                }
            }
        }
        return rv;
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        dispose();
    }

}
