/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.planets;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import com.fasterxml.jackson.jr.ob.JSON;

import sitool.ConfigData;
import sitool.components.Building;
import sitool.components.Defense;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.Research;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.Stage;
import sitool.components.Technology;
import sitool.components.Version;
import sitool.datatransfer.DataImport;
import sitool.json.IntergalacticOverviewBean;
import sitool.json.JSONTags;
import sitool.json.SpyReportBean;


/**
 *
 * @author graf
 *
 */
public class PlanetBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    private JTextArea textArea = new JTextArea();
    private ConfigData configData;
    private Version version;

    private PlanetPane parent = null;

    // these Scanboxes are all pretty much the same, they just need a different scanner...
    public PlanetBox( PlanetPane parent )
    {
	    super( parent.getMainFrame() );
        this.parent = parent;
        configData = parent.getMainFrame().getConfigData();
        version = configData.getVersion();

        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "PlanetBox.title" ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 640, 520 ) );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );

        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();

        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {

            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    textArea.setText( input );
                }
            }

        };

        textArea.addMouseListener( l );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JScrollPane scrollPane = new JScrollPane( textArea,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        center.add( scrollPane, BorderLayout.CENTER );

        return center;
    }

    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        String input = textArea.getText();
        Version version = configData.getVersion();

        if( version != Version.v20 )
        {
            if( input.indexOf(JSONTags.reportType_SpyReport) > -1 ) {
                scanSpyReportJSON(input);
            }
            else if( input.indexOf(JSONTags.reportType_PlanetOverview) > -1 ) {
                scanIntergalacticPlanetOverviewJSON(input);
            }
            else if( input.indexOf( configData.getI18nText( "PlanetBox.spyMarker" ) ) > -1 )
            {
                scanSpyReport( input );
            }
            else
            {
                scanShort( input );
            }
        }
        else
            if( input.indexOf( configData.getI18nText( "PlanetBox.spyMarker2" ) ) > -1 )
            {
                scanSpyReport( input );
            }
            else
            {
                scanShort2( input );
            }
        dispose();
    }

    private void scanShort( String input )
    {
        String marker = configData.getI18nText( "Planet.marker" ) + " ";
        BufferedReader br = new BufferedReader( new StringReader( input ) );
        String line;
        String name = null;
        Pattern pattern = Pattern.compile( ".*°C.*\\s+\\(\\d+/(\\d+).*\\s+(\\d+):(\\d+):(\\d+)\\s+\\(x:.*y:.*z:.*" );
        Pattern shortPattern = Pattern.compile( "\\s*\\[(\\d+):(\\d+):(\\d+)\\s*\\]\\s+\\-\\s+(\\S+.*)" );
        try
        {
            while( (line = br.readLine() ) != null )
            {
                Matcher m = shortPattern.matcher( line );
                if( m.matches() )
                {
                    // don't fear any NumberFormatExceptions, the regular expression takes care of that
                    int gal = Integer.parseInt( m.group( 1 ) );
                    int sys = Integer.parseInt( m.group( 2 ) );
                    int pl = Integer.parseInt( m.group( 3 ) );
                    name = m.group( 4 ).trim();
                    Position position = new Position( gal, sys, pl );
                    Planet planet = new Planet( name, position );
                    planet.setSize( 0 );
                    parent.fillFields( planet, false );
                    break;
                }
                int pos = line.indexOf( marker );
                if( pos >= 0 )
                {
                    name = line.substring( pos + marker.length() );
                }
                else
                {
                    m = pattern.matcher( line );
                    if( m.matches() )
                    {
                        // don't fear any NumberFormatExceptions, the regular expression takes care of that
                        int size = Integer.parseInt( m.group( 1 ) );
                        int gal = Integer.parseInt( m.group( 2 ) );
                        int sys = Integer.parseInt( m.group( 3 ) );
                        int pl = Integer.parseInt( m.group( 4 ) );
                        Position position = new Position( gal, sys, pl );
                        Planet planet = new Planet( name, position );
                        planet.setSize( size );
                        parent.fillFields( planet, true );
                        break;
                    }
                }
            }
            br.close();
        }
        catch( IOException e1 )
        {}
    }

    private void scanShort2( String input )
    {
        Pattern pattern = Pattern.compile( "(:\\s+(.+)\\s+\\[)?(\\d+):(\\d+):(\\d+)\\]?" );

        BufferedReader br = new BufferedReader( new StringReader( input ) );
        String line;
        try
        {
            while( (line = br.readLine() ) != null )
            {
                Matcher m = pattern.matcher( line );
                if( m.find() )
                {
                    // don't fear any NumberFormatExceptions, the regular expression takes care of that
                    int gal = Integer.parseInt( m.group( 3 ) );
                    int sys = Integer.parseInt( m.group( 4 ) );
                    int pl = Integer.parseInt( m.group( 5 ) );

                    Position position = new Position( gal, sys, pl );
                    String name = ( m.group(2) != null ) ? m.group( 2 ) : "";
                    Planet planet = new Planet( name, position );
                    planet.setSize( 0 );
                    parent.fillFields( planet, false );
                    break;
                }
            }
            br.close();
        }
        catch( IOException e1 )
        {}
    }


    private void scanSpyReport( String input )
    {
        // reset
        parent.fillFields( null, false, null );
        Pattern basePattern = Pattern.compile( ".*" + configData.getI18nText( "PlanetBox.spyMarker" ) + "\\s+(\\S+.*)\\[(\\d+):(\\d+):(\\d+)\\s*\\].*", Pattern.DOTALL );
        Matcher m = basePattern.matcher( input );
        Version version = configData.getVersion();
        if( m.matches() )
        {
            // get planet
            String name = m.group( 1 ).trim();
            int gal = Integer.parseInt( m.group( 2 ) );
            int sys = Integer.parseInt( m.group( 3 ) );
            int pl = Integer.parseInt( m.group( 4 ) );
            Position position = new Position( gal, sys, pl );
            Planet planet = new Planet( name, position );
            planet.setSize( 1000 );
            boolean astro = configData.getI18nText( "Asteroid" ).equals( name );
            planet.setAstroSize( astro ? 100 : 0 );
            position.setAsteroidPosition( astro );

            // check buildings
            for( Building building : Building.values() )
            {
                Pattern buildingPattern = Pattern.compile( ".*" + configData.getI18nText( building.name() ) + "\\s+(\\d+)\\s+.*", Pattern.DOTALL );
                m = buildingPattern.matcher( input );
                if( m.matches() )
                {
                    int level = Integer.parseInt( m.group( 1 ) );
                    Stage stage = new Stage(building, level);
                    planet.addBuilding( building, stage, astro );
                }
            }
            Fleet fleet = planet.getFleet( astro );
            // check defense
            for( Defense defense : Defense.defenses( version ) )
            {
                Pattern defensePattern = Pattern.compile( ".*" + configData.getI18nText( defense.name() ) + "\\s+(\\d+)\\s+.*", Pattern.DOTALL );
                m = defensePattern.matcher( input );
                if( m.matches() )
                {
                    fleet.setCount( defense, Long.parseLong( m.group( 1 ) ) );
                }
            }
            // and ships
            for( Ship ship : Ship.ships( version ) )
            {
                Pattern shipPattern = Pattern.compile( ".*" + configData.getI18nText( ship.name() ) + "\\s+(\\d+)\\s+.*", Pattern.DOTALL );
                m = shipPattern.matcher( input );
                if( m.matches() )
                {
                    fleet.setCount( ship, Long.parseLong( m.group( 1 ) ) );
                }
            }
            // finally technology
            Technology newTech = new Technology();
            for( Research research : Research.values() )
            {
                Pattern techPattern = Pattern.compile( ".*" + configData.getI18nText( research.name() ) + "\\s+(\\d+)\\s+.*", Pattern.DOTALL );
                m = techPattern.matcher( input );
                if( m.matches() )
                {
                    newTech.setTech( research, Integer.parseInt( m.group( 1 ) ) );
                }
            }

            parent.fillFields( planet, false, newTech );
        }

    }

    private void scanSpyReportJSON( String input ) {
        try {
            SpyReportBean reportBean = JSON.std.beanFrom(SpyReportBean.class, input);
            final Planet planet = DataImport.createPlanetFromJSON(version, reportBean, true, true);

            Technology newTech = DataImport.createTechnologyFromJSON(reportBean.getPlanet().getResearches());
            parent.fillFields( planet, false, newTech );
        }
        catch( Exception e1 ){
            e1.printStackTrace();
        }
    }

    private void scanIntergalacticPlanetOverviewJSON( String input ) {
        try {
            IntergalacticOverviewBean overview = JSON.std.beanFrom(IntergalacticOverviewBean.class, input);
            final List<Planet> planets = DataImport.createPlanetsFromJSON(version, overview, configData.isOutpost(), true, true);
            Technology newTech = DataImport.createTechnologyFromJSON(overview.getResearches());
            Specialization specialization = DataImport.createSpecializationFromJSON(overview.getSpecs());
            Fleet totalShips = DataImport.importTotalShipsFromJSON(overview.getShips(), configData.isOutpost());
            parent.updateConfiguration(planets, newTech, specialization, totalShips);
        }
        catch( Exception e1 ){
            e1.printStackTrace();
        }
    }
}
