/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.planets;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Building;
import sitool.components.Defense;
import sitool.components.Fleet;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionField;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.Stage;
import sitool.components.Technology;
import sitool.components.TextFieldFocusListener;
import sitool.components.Version;
import sitool.datatransfer.DataImport;
import sitool.resources.BuildingBox;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.ImageTableCellRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.PlanetTableModel;
import sitool.table.SpinnerCellEditor;
import sitool.table.TemperatureTableCellRenderer;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class PlanetPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private MainFrame mainFrame = null;
    private ConfigData configData;

    private JTextField planetName = new JTextField();
    private PositionField planetPosition = new PositionField();
    private JTextField astroSize = new JTextField();
    private JTextField planetSize = new JTextField();
    private JCheckBox ownPlanet = new JCheckBox();
    private JButton importButton = new JButton();

    private PlanetTableModel planets;
    private JTable planetTable;
    JPanel center;

    private JLabel labelName = new JLabel();
    private JLabel labelPosition = new JLabel();
    private JLabel labelSize = new JLabel();
    private JLabel labelOwn = new JLabel();
    private JLabel labelAstroSize = new JLabel();
    private JButton addButton = new JButton();
    private JButton removePlanet = new JButton();

    private Planet newPlanet;
    private Technology newTechnology;

	public PlanetPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        planets = new PlanetTableModel( mainFrame );
        center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        mainFrame.getRootPane().setDefaultButton( addButton );
        if( mainFrame.getConfigData().getActiveAccount() != null )
        {
            center.setVisible( true );
            updateTable();
            updateLabel();
        }
        else
        {
            center.setVisible( false );
        }
    }

    private void updateLabel()
    {
        labelName.setText( configData.getI18nText( "Planet.name" ) );
        labelPosition.setText( configData.getI18nText( "Planet.position" ) );
        labelSize.setText( configData.getI18nText( "Planet.size" ) );
        labelOwn.setText( configData.getI18nText( "Planet.own" ) );
        labelAstroSize.setText( configData.getI18nText( "Planet.astrosize" ) );
        importButton.setText( configData.getI18nText( "Planet.import" ) );
        addButton.setText( configData.getI18nText( "add" ) );
        removePlanet.setText( configData.getI18nText( "delete" ) );
    }

    private void updateTable()
    {
        planets.fireTableStructureChanged();

        TableColumn col;
        TableColumnModel columnModel = planetTable.getColumnModel();
        int width = 16;
        col = columnModel.getColumn(PlanetTableModel.DOWN);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = columnModel.getColumn(PlanetTableModel.UP);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 70;
        col = columnModel.getColumn(PlanetTableModel.SIZE);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        col = columnModel.getColumn(PlanetTableModel.ASTROSIZE);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 75;
        col = columnModel.getColumn(PlanetTableModel.TEMPERATURE);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        width = 65;
        col = columnModel.getColumn(PlanetTableModel.OWN);
        col.setMinWidth( width );
        col.setMaxWidth( width );
        col.setPreferredWidth( width );
        Version version = configData.getVersion();
        if( version != Version.v20 )
        {
            TableColumn column = columnModel.getColumn( PlanetTableModel.COORDINATES );
            columnModel.removeColumn( column );
        }
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel northPanel = new JPanel( new GridBagLayout() );
        FocusListener l = new TextFieldFocusListener();
        planetName.addFocusListener( l );
        planetSize.addFocusListener( l );
        astroSize.addFocusListener( l );

        GridBagConstraints fieldConstraints = new GridBagConstraints();
        fieldConstraints.gridheight = 1;
        fieldConstraints.gridwidth = 1;
        fieldConstraints.fill = GridBagConstraints.BOTH;
        fieldConstraints.anchor = GridBagConstraints.CENTER;
        fieldConstraints.weighty = 1.0;
        fieldConstraints.weightx = 1.0;
        fieldConstraints.insets = new Insets( 5, 5, 5, 5 );

        fieldConstraints.gridx = 0;
        northPanel.add( labelName, fieldConstraints );
        fieldConstraints.gridx = 2;
        northPanel.add( labelPosition, fieldConstraints );
        fieldConstraints.gridx = 3;
        northPanel.add( labelSize, fieldConstraints );
        fieldConstraints.gridx = 4;
        northPanel.add( labelOwn, fieldConstraints );
        fieldConstraints.gridx = 5;
        northPanel.add( labelAstroSize, fieldConstraints );
        fieldConstraints.gridy = 1;
        fieldConstraints.gridx = 0;
        northPanel.add( planetName, fieldConstraints );
        fieldConstraints.gridx = 2;
        northPanel.add( planetPosition, fieldConstraints );
        fieldConstraints.gridx = 3;
        northPanel.add( planetSize, fieldConstraints );
        ownPlanet.setSelected( true );
        fieldConstraints.gridx = 4;
        northPanel.add( ownPlanet, fieldConstraints );
        fieldConstraints.gridx = 5;
        northPanel.add( astroSize, fieldConstraints );

        importButton.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                actionImportPlanet();
            }

        });
        fieldConstraints.gridx = 6;
        northPanel.add( importButton, fieldConstraints );

        addButton.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                actionAddPlanet();
            }

        });
        fieldConstraints.gridy = 2;
        fieldConstraints.gridx = 2;
        fieldConstraints.gridwidth = 2;
        northPanel.add( addButton, fieldConstraints );
        center.add(northPanel, BorderLayout.NORTH );

        final DefaultCellEditor col0Editor = new DefaultCellEditor( new JTextField() );
        col0Editor.setClickCountToStart( 1 );
        final TableCellRenderer col2Renderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        final TableCellEditor col2Editor = new SpinnerCellEditor( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        JTextField editor = new JTextField();
        editor.setBorder( new EmptyBorder( 0, 4, 0, 14 ) );
        editor.setHorizontalAlignment( SwingConstants.RIGHT );
        final TableCellRenderer col3Renderer = new TemperatureTableCellRenderer();
        final TableCellEditor col3Editor = new DefaultCellEditor( editor );
        final TableCellRenderer col4Renderer =  new CheckBoxTableCellRenderer();
        final TableCellEditor col4Editor = new DefaultCellEditor( new JCheckBox() );
        final TableCellRenderer col5Renderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        final TableCellEditor col5Editor = new SpinnerCellEditor( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        final ImageTableCellRenderer ic = new ImageTableCellRenderer();

        planetTable = new TooltipTable( planets ) {
            private static final long serialVersionUID = 2885181790454689168L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case PlanetTableModel.SIZE:
                        rv = col2Renderer;
                        break;
                    case PlanetTableModel.TEMPERATURE:
                        rv = col3Renderer;
                        break;
                    case PlanetTableModel.OWN:
                        rv = col4Renderer;
                        break;
                    case PlanetTableModel.ASTROSIZE:
                        rv = col5Renderer;
                        break;
                    case PlanetTableModel.UP:
                    case PlanetTableModel.DOWN:
                        rv = ic;
                        break;
                    default:
                        rv = super.getCellRenderer( row, column );
                        break;
                }
                return rv;
            }
            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn =  convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case 0:
                        rv = col0Editor;
                        break;
                    case 2:
                        rv = col2Editor;
                        break;
                    case 3:
                        rv = col3Editor;
                        break;
                    case 4:
                        rv = col4Editor;
                        break;
                    case 5:
                        rv = col5Editor;
                        break;
                    default:
                        rv = super.getCellEditor( row, column );
                        break;
                }
                return rv;
            }
        };
        planetTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        planetTable.addMouseListener( planets );
        planetTable.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "deletePlanets");
        planetTable.getActionMap().put("deletePlanets", new AbstractAction() {
            private static final long serialVersionUID = -5030298157979186506L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                if( removePlanet.isEnabled() ) {
                    actionDeletePlanets();
                }
            }
        });

        TableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        planetTable.setDefaultRenderer( Object.class, defaultCellRenderer );

        JScrollPane scrollPane = new JScrollPane( planetTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        center.add( scrollPane, BorderLayout.CENTER );
        removePlanet.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionDeletePlanets();
            }
        });
        removePlanet.setEnabled( false );
        JPanel actionPanel = new JPanel( new FlowLayout( FlowLayout.CENTER, 5, 5 ) );
        JPanel buttonPanel = new JPanel( new GridLayout( 1, 1) );
        buttonPanel.add(  removePlanet );
        actionPanel.add( buttonPanel );
        center.add( actionPanel, BorderLayout.EAST );

        planetTable.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    // enable the button when selection is active
                    removePlanet.setEnabled( e.getFirstIndex() >= 0 );
                }
            }
        });

        return center;
    }

    protected void actionDeletePlanets() {
        int[] rows = planetTable.getSelectedRows();
        planets.removeRows( rows );
        mainFrame.getDisplay().update();
    }

    protected void actionImportPlanet()
    {
        JDialog dlg = new PlanetBox( this );
        mainFrame.showModalBox( dlg );
    }

    protected void fillFields( Planet planet, boolean isMine )
    {
        planetName.setText( planet.getName() );
        planetPosition.setPosition( planet.getPosition() );
        planetSize.setText( Integer.toString( planet.getSize() ) );
        ownPlanet.setSelected( isMine );
        astroSize.setText( "" );
    }

    protected void fillFields( Planet planet, boolean isMine, Technology newTech )
    {
        newPlanet = planet;
        newTechnology = newTech;
        if( planet != null )
        {
            fillFields( planet, isMine );
            astroSize.setText( Integer.toString( planet.getAstroSize() ) );
        }
    }

    public void addPlanet( Planet planet, Technology newTech ) {
        fillFields(planet, true, newTech);
        actionAddPlanet();
    }

    public void updateConfiguration( List<Planet> planets, Technology newTech, Specialization specs, Fleet totalShips )
    {
        Account account = configData.getActiveAccount();
        if(account != null) {
            DataImport.updateConfiguration(account, planets, newTech, specs, totalShips);
            // update table
            this.planets.update();
            // update points
            mainFrame.getDisplay().update();
        }
    }

    protected void actionAddPlanet()
    {
        String name = planetName.getText();
        Position position = planetPosition.getPosition();
        if( name.trim().length() > 0 && position.isRealPosition() )
        {
            Version version = configData.getVersion();
            Planet planet = new Planet( name, position );
            planet.setMine( ownPlanet.isSelected() );
            if( planet.isMine() )
            {
                int size = 0;
                try
                {
                    size = Integer.parseInt( planetSize.getText() );
                }
                catch( NumberFormatException ex )
                {}
                planet.setSize( size );
                size = 0;
                try
                {
                    size = Integer.parseInt( astroSize.getText() );
                }
                catch( NumberFormatException ex )
                {}
                planet.setAstroSize( size );
            }
            planet = planets.addOrUpdateRow( planet );
            if( planet != null && planet.isMine() )
            {
                if( newPlanet != null )
                {
                    Stage stage;
                    boolean isAstro = newPlanet.getPosition().isAsteroidPosition();
                    // update buildings if they exist
                    Map<Building, Stage> newBuildings = newPlanet.getBuildings(isAstro);
                    if( newBuildings != null) {
                        // buildings
                        Building[] buildings = (isAstro) ? Building.astroBuildings() : Building.planetBuildings(configData.getVersion());
                        for( Building building : buildings )
                        {
                            stage = newBuildings.get( building );
                            if( stage != null )
                            {
                            	BuildingBox.setValue( planet, building, stage.getLevel(), isAstro );
                            }
                        }
                    }
                    // fleet
                    Fleet fleet = planet.getFleet( isAstro );
                    Fleet newFleet = newPlanet.getFleet( isAstro );
                    if(newFleet != null) {
                        // ships
                        if(newFleet.hasShips()) {
                            for( Ship ship : Ship.ships( version ) )
                            {
                                fleet.setCount( ship, newFleet.getCount( ship ) );
                            }
                        }
                        // defense
                        if(newFleet.hasDefenses()) {
                            for( Defense defense : Defense.defenses( version ) )
                            {
                                fleet.setCount( defense, newFleet.getCount( defense ) );
                            }
                        }
                    }
                }
                // tech
                if( newTechnology != null && newTechnology.isValid() )
                {
                    configData.setMyTech( newTechnology );
                }
                // update points
                mainFrame.getDisplay().update();
            }
            // reset
            newPlanet = null;
            newTechnology = null;
            planetName.setText( "" );
            planetPosition.setPosition( new Position( 0, 0, 0 ) );
            astroSize.setText( "" );
            planetSize.setText( "" );
            ownPlanet.setSelected( true );
        }
    }

}
