/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Resource;
import sitool.table.StockpileTableModel;

/**
 * 
 * @author graf
 *
 */
public class AstroResourceBox extends JDialog implements ActionListener
{
    private static final long serialVersionUID = -4586875011135823902L;

    private JTextArea textArea = new JTextArea();

    private MainFrame mainFrame;
    private ConfigData configData;
    private StockpileTableModel stockPileModel;
    private int row;
    private String sPattern;

    // these Scanboxes are all pretty much the same, they just need a different scanner... 
    public AstroResourceBox( MainFrame parent, StockpileTableModel stockPileModel, int row )
    {
	    super( parent );
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        this.stockPileModel = stockPileModel;
        this.row = row;

        StringBuilder sb = new StringBuilder();
        sb.append( ".*" );
        sb.append( configData.getI18nText( Resource.iron.name() ).toLowerCase() );
        sb.append( "\\s+(\\d+.*)\\s+" );
        sb.append( configData.getI18nText( Resource.metal.name() ).toLowerCase() );
        sb.append( "\\s+(\\d+\\S*\\d*)\\s+" );
        sb.append( configData.getI18nText( Resource.kryptonite.name() ).toLowerCase() );
        sb.append( "\\s+(\\d+\\S*\\d*)\\s+" );
        sb.append( configData.getI18nText( Resource.spice.name() ).toLowerCase() );
        sb.append( "\\s+(\\d+\\S*\\d*)\\s+" );
        sb.append( configData.getI18nText( "Energy" ).toLowerCase() );
        sb.append( ".*" );
        sPattern = sb.toString();
        
        enableEvents( AWTEvent.WINDOW_EVENT_MASK );
        this.setTitle( configData.getI18nText( "AstroResourceBox.title", stockPileModel.getValueAt( row, 0 ) ) );
        JPanel contentPane = ( JPanel )getContentPane();
        contentPane.setLayout( new BorderLayout() );
        contentPane.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        
        JPanel center = createCenterPanel();
        center.setPreferredSize( new Dimension( 640, 520 ) );
        
        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER, 4, 4 ) );
        JButton button = new JButton( configData.getI18nText( "OK" ) );
        button.addActionListener( this );
        lower.add( button );
        getRootPane().setDefaultButton( button );
        
        contentPane.add( center, BorderLayout.CENTER );
        contentPane.add( lower, BorderLayout.SOUTH );
        pack();
        
        // by popular demand, add paste clipboard to RMB
        MouseAdapter l = new MouseAdapter()
        {
            @Override
            public void mouseClicked( MouseEvent e )
            {
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    String input = "";
                    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    Transferable contents = clipboard.getContents( null );
                    boolean hasTransferableText = ( ( contents != null )
                                    && contents.isDataFlavorSupported( DataFlavor.stringFlavor ) );
                    if( hasTransferableText )
                    {
                        try
                        {
                            input = ( String )contents.getTransferData( DataFlavor.stringFlavor );
                        }
                        catch( UnsupportedFlavorException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                        catch( IOException ex )
                        {
                            System.out.println( ex );
                            ex.printStackTrace();
                        }
                    }
                    textArea.setText( input );
                }
            }

        };
        
        textArea.addMouseListener( l );
    }

    protected JPanel createCenterPanel()
    {
        JPanel center = new JPanel( new BorderLayout() );
        center.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10) );

        JScrollPane scrollPane = new JScrollPane( textArea,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        center.add( scrollPane, BorderLayout.CENTER );
        
        return center;
    }
    
    // Overridden so we can exit when window is closed
    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    protected void processWindowEvent( WindowEvent e )
    {
        if( e.getID() == WindowEvent.WINDOW_CLOSING )
        {
            cancel();
        }
        super.processWindowEvent( e );
    }

    // Close the dialog

    /***************************************************************************
     * Description of the Method
     **************************************************************************/
    void cancel()
    {
        dispose();
    }

    // Close the dialog on a button event

    /***************************************************************************
     * Description of the Method
     * @param e Description of the Parameter
     **************************************************************************/
    public void actionPerformed( ActionEvent e )
    {
        String input = textArea.getText(); 
        BufferedReader br = new BufferedReader( new StringReader( input ) );
        String line;
        Pattern pattern = Pattern.compile( sPattern );
        try
        {
            boolean doContinue = true;
            line = br.readLine();
            StringBuilder sb = new StringBuilder();
            while( line != null && doContinue )
            {
            	sb.append( line );
            	sb.append( ' ' );
            	doContinue = !line.toLowerCase().contains( configData.getI18nText( "Energy" ).toLowerCase() );
                line = br.readLine();
            }
            br.close();
            Matcher m = pattern.matcher( sb.toString().toLowerCase() );
            if( m.matches() && m.groupCount() > 3 )
            {
            	for( int index = 1; index <= 4; index++ )
            	{
            		stockPileModel.setValueAt( m.group( index ), row, 2 * index - 1 );
            	}
            }
        }
        catch( IOException e1 )
        {}
        dispose();
    }
    
}
