/****************************************
 *  COPYRIGHT (C) 2013-2021
 *  Holger Graf
 ****************************************/
package sitool.resources;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Coordinates;
import sitool.components.ModulePane;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionField;
import sitool.components.Version;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.TeleporterTableModel;
import sitool.table.TooltipTable;

public class TeleporterPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -4655043025777882546L;
    protected JComboBox startPlanets = new JComboBox();
    protected JComboBox targetPlanets = new JComboBox();
    protected PositionField launchPosition = new PositionField();
    protected PositionField targetPosition = new PositionField();
    private static final String spacer = "   ";

    private MainFrame mainFrame;
    private ConfigData configData;

    private JButton calculateButton = new JButton();
    private JLabel labelCoordsStart = new JLabel();
    private JLabel labelCoordsTarget = new JLabel();
    private JButton deleteSelected = new JButton();
    private JButton deleteAll = new JButton();
    private Account current;
    private JTable resultTable;
    private TeleporterTableModel resultModel;

    public TeleporterPane(MainFrame parent)
    {
        this.mainFrame = parent;
        this.configData = mainFrame.getConfigData();
        this.current = configData.getActiveAccount();

        setLayout( new BorderLayout() );
        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );
        Border border = BorderFactory.createEmptyBorder( 0, 6, 0, 6 );
        panel.setBorder( new EtchedBorder( EtchedBorder.LOWERED ) );

        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        ItemListener l = new ItemListener() {
            public void itemStateChanged( ItemEvent e )
            {
                actionComboBoxChanged( e );
            }
        };
        startPlanets.addItemListener( l );
        targetPlanets.addItemListener( l );

        JPanel northPanel = new JPanel( new BorderLayout() );

        GridBagConstraints gridConstraints = new GridBagConstraints();
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 0;
        gridConstraints.gridheight = 1;
        gridConstraints.gridwidth = 2;
        gridConstraints.fill = GridBagConstraints.NONE;
        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.weightx = 0.0;
        gridConstraints.weighty = 0.0;
        gridConstraints.insets = new Insets( 2, 2, 2, 2 );

        JPanel businessPanel = new JPanel( new GridBagLayout() );
        businessPanel.add( labelCoordsStart, gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( labelCoordsTarget, gridConstraints );
        gridConstraints.gridwidth = 1;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.weightx = 1.0;
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 1;
        businessPanel.add( launchPosition, gridConstraints );
        gridConstraints.gridx = 1;
        businessPanel.add( startPlanets, gridConstraints );
        gridConstraints.gridx = 2;
        businessPanel.add( new JLabel( spacer ), gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( targetPosition, gridConstraints );
        gridConstraints.gridx = 4;
        businessPanel.add( targetPlanets, gridConstraints );
        FocusListener f = new FocusAdapter() {
            public void focusLost( FocusEvent e )
            {
                JComponent src = ( JComponent )e.getSource();
                Object o = src.getParent();
                if( o.equals( launchPosition ) && launchPosition.getPosition().isRealPosition() )
                {
                    Planet p = new Planet( "", launchPosition.getPosition() );
                    int pos = boxContains( p, startPlanets );
                    startPlanets.setSelectedIndex( pos );
                }
                else if( o.equals( targetPosition ) )
                {
                    Planet p = new Planet( "", targetPosition.getPosition() );
                    int pos = boxContains( p, targetPlanets );
                    targetPlanets.setSelectedIndex( pos );
                }
            }
        };
        launchPosition.addFocusListener( f );
        targetPosition.addFocusListener( f );

        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.gridx = 1;
        gridConstraints.gridy = 2;
        gridConstraints.gridwidth = 3;
        gridConstraints.insets.top = 6;
        calculateButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionCalculate();
            }
        } );
        businessPanel.add( calculateButton, gridConstraints );
        northPanel.add( businessPanel, BorderLayout.WEST );
        panel.add( northPanel, BorderLayout.NORTH );

        JPanel removePanel = new JPanel( new GridLayout( 2, 1, 10, 4 ) );
        int bHeight = deleteAll.getPreferredSize().height;
        removePanel.setBorder( BorderFactory.createEmptyBorder( bHeight, 4, bHeight, 4 ) );
        deleteSelected.setEnabled( false );
        removePanel.add( deleteSelected );
        removePanel.add( deleteAll );
        northPanel.add( removePanel, BorderLayout.EAST );

        deleteSelected.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                actionDelete();
            }
        } );

        deleteAll.addActionListener( new ActionListener()
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
                resultModel.removeAll();
            }
        } );

        JPanel southPanel = new JPanel( new BorderLayout() );
        resultModel = new TeleporterTableModel( configData );
        resultTable = new TooltipTable( resultModel ) {
            private static final long serialVersionUID = 4353244704522148723L;

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = this.getPreferredSize();
                return size;
            }
        };
        resultTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        DefaultTableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        DefaultTableCellRenderer planetCellRenderer = new AlternateLineTableCellRenderer();
        defaultCellRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        planetCellRenderer.setHorizontalAlignment( SwingConstants.LEFT );
        resultTable.setDefaultRenderer( Object.class, defaultCellRenderer );
        resultTable.setDefaultRenderer( Planet.class, planetCellRenderer );

        JScrollPane scrollPane = new JScrollPane( resultTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( border );
        southPanel.add( scrollPane, BorderLayout.CENTER );

        panel.add( southPanel, BorderLayout.CENTER );

        resultTable.getSelectionModel().addListSelectionListener( new ListSelectionListener()
        {
            public void valueChanged( ListSelectionEvent e )
            {
                if( !e.getValueIsAdjusting() )
                {
                    int rows[] = resultTable.getSelectedRows();
                    deleteSelected.setEnabled( rows.length > 0 );
                }
            }
        });
        resultTable.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "delete");
        resultTable.getActionMap().put("delete", new AbstractAction() {
            private static final long serialVersionUID = 5999113634471333070L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                if(deleteSelected.isEnabled()) {
                    actionDelete();
                }
            }
        });

        return panel;
    }

    protected void actionCalculate()
    {
        Position startPos = launchPosition.getPosition();
        Position targetPos = targetPosition.getPosition();
        if( startPos.isRealPosition() && targetPos.isRealPosition() )
        {
            if( configData.getVersion() == Version.v20 )
            {
                // since version is not the default, current is guaranteed not null
                // check coordinates availability to continue
                if( !current.isSync() )
                {
                    resultModel.fireTableDataChanged();
                    return;
                }
                Map<Position, Coordinates> map = configData.getCoordinates();
                if( map.get( startPos ) == null ||  map.get( targetPos ) == null )
                {
                    resultModel.fireTableDataChanged();
                    return;
                }
            }
            Planet origin = configData.getPlanetOnPosition( startPos );
            Planet target = configData.getPlanetOnPosition( targetPos );
            Entry<Planet, Planet> data = new AbstractMap.SimpleEntry<Planet, Planet>( origin, target );
            resultModel.addData( data );
        }
        resultModel.fireTableDataChanged();

    }

    protected void actionComboBoxChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Object source = e.getSource();
            if( source == startPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    launchPosition.setPosition( position );
            }
            else if( source == targetPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    targetPosition.setPosition( position );
            }
        }
    }

    protected int boxContains( Object o, JComboBox box )
    {
        int rv = 0;
        int itemCount = box.getItemCount();
        for( int i = 0; i < itemCount; i++ )
        {
            if( o.equals( box.getItemAt( i ) ) )
            {
                rv = i;
                break;
            }
        }
        return rv;
    }

    protected void actionDelete()
    {
        int[] rows = resultTable.getSelectedRows();
        if( rows.length > 0 )
        {
            int[] modelRows = new int[rows.length];
            for( int i = 0; i < rows.length; i++)
            {
                modelRows[i] = resultTable.convertRowIndexToModel( rows[i] );
            }
            resultModel.removeData( modelRows );
        }

    }

    @Override
    public void update()
    {
        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        updateLabel();
        updateTable();
    }

    private void updateTable()
    {
        Account account = configData.getActiveAccount();
        if( current != account )
        {
            resultModel.removeAll();
            launchPosition.setPosition( new Position( 0, 0, 0 ) );
            targetPosition.setPosition( new Position( 0, 0, 0 ) );
            current = account;
        }
        resultModel.fireTableStructureChanged();
    }

    private void updateMyPlanets( JComboBox planets )
    {
        Object selection = planets.getSelectedItem();
        planets.removeAllItems();
        List<Planet> myPlanets;
        int size = 0;
        myPlanets = new ArrayList<Planet>( mainFrame.getConfigData().getPlanets() );
        planets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.planets" ) + "  --", new Position( 0, 0, 0) ) );
        size++;
        for( Planet planet : myPlanets )
        {
            planets.addItem( planet );
            size++;
        }
        // restore selection
        if( selection != null )
        {
            planets.setSelectedItem( selection );
        }
        else
        {
            planets.setSelectedIndex( 0 );
        }
        planets.setMaximumRowCount( size );
    }

    private void updateLabel()
    {
        labelCoordsStart.setText( configData.getI18nText( "Flighttime.start" ) );
        labelCoordsTarget.setText( configData.getI18nText( "Flighttime.target" ) );
        calculateButton.setText( configData.getI18nText( "Flighttime.calculate" ) );
        deleteSelected.setText( configData.getI18nText( "delete" ) );
        deleteAll.setText( configData.getI18nText( "deleteAll" ) );
    }
}
