/****************************************
 *  COPYRIGHT (C) 2013
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import javax.swing.JPanel;

import sitool.Account;
import sitool.ConfigData;
import sitool.components.Coordinates;
import sitool.components.Distance;
import sitool.components.FleetSpeed;
import sitool.components.Position;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class AbstractDurationPanel extends JPanel
{
    private static final long serialVersionUID = 4808555238851084902L;
    protected ConfigData configData;

    /**
     * @param configData
     */
    public AbstractDurationPanel( ConfigData configData )
    {
        super();
        this.configData = configData;
    }

    protected long getDuration( FleetSpeed lowestSpeed, long distance, int percent, double speedFactor, Version version )
    {
        long rv = 0;
        // see the comment in Calculator about spectral mode and speedFactor
        int spectral = configData.isSpectralMode() ? 10 : 1;
        if( version != Version.v20 )
        {
            rv = 100 * Calculator.calculateDuration( lowestSpeed.getVelocity(), distance, speedFactor ) / percent;
        }
        else
        {
            double speed = percent * (double)lowestSpeed.getVelocity() / 100.0;
            double acceleration = percent * (double)lowestSpeed.getAcceleration() / 100.0;
            rv = Calculator.calculateDuration2( speed, acceleration, distance );
            rv += lowestSpeed.getPreptime();
            // adjust time
            rv = Math.round( rv / speedFactor );
        }
        return spectral * rv;
    }

    protected long[] getDistance( Position startPos, Position targetPos, Version version, Account account )
    {
        long[] rv;
        if( version != Version.v20 )
        {
            rv = Distance.getDistance( version, startPos, targetPos );
        }
        else
        {
            Coordinates startCoordinates = configData.getCoordinates( startPos, account );
            Coordinates targetCoordinates = configData.getCoordinates( targetPos, account );
            rv = new long[] { targetCoordinates.distance( startCoordinates ) };
        }
        return rv;
    }

}
