/****************************************
 *  COPYRIGHT (C) 2008-2020
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Coordinates;
import sitool.components.Fleet;
import sitool.components.FleetSpeed;
import sitool.components.ModulePane;
import sitool.components.Percent;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionField;
import sitool.components.Ship;
import sitool.components.Technology;
import sitool.components.TimePanel;
import sitool.components.TotalPointDisplay;
import sitool.components.Version;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.ResourceTableCellRenderer;
import sitool.table.StatisticsTableModel;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class FlightDurationPane extends AbstractDurationPanel implements ModulePane
{
	private static final long serialVersionUID = -1878874776754387489L;

    protected JComboBox<Planet> startPlanets = new JComboBox<>();
    protected JComboBox<Planet> targetPlanets = new JComboBox<>();
    protected PositionField launchPosition = new PositionField();
    protected PositionField targetPosition = new PositionField();
    private static final String spacer = "   ";

    private MainFrame mainFrame;
    private Calculator calculator;

    private JButton calculateButton = new JButton();
    private JLabel labelSpeed = new JLabel();
    private JComboBox<Percent> boxSpeed = new JComboBox<>();
    private JLabel labelFarmKing = new JLabel();
    private JCheckBox boxFarmKing = new JCheckBox();
    private JLabel labelCoordsStart = new JLabel();
    private JLabel labelCoordsTarget = new JLabel();
    private JLabel labelShips = new JLabel();
    protected JCheckBox useAvailable = new JCheckBox();
    private JButton selectAll = new JButton();
    private JButton deselectAll = new JButton();
    private AvailableShips availableShipsModel;
    private ResultModel resultModel;
    private JTable availableShipsTable;
    private JTable resultTable;
    private JLabel labelAllianceSpecialization = new JLabel();
    private JSpinner spinnerAllianceSpecialization;

    private FleetSpeed lowestSpeed = new FleetSpeed();
    private long[] distance = { 0 };
    private long[] time = { 0 };

    /**
     * @throws HeadlessException
     */
    public FlightDurationPane( MainFrame parent ) throws HeadlessException
    {
        super(parent.getConfigData());

        this.mainFrame = parent;
        calculator = new Calculator( mainFrame.getConfigData() );

        setLayout( new BorderLayout() );
        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    /**
     *
     */
    public void update()
    {
        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        updateLabel();
        updateTable();
    }

    private void updateTable()
    {
        availableShipsModel.update();
        resultModel.fireTableStructureChanged();

        Version version = configData.getVersion();
        if( version != Version.v20 )
        {
            TableColumnModel columnModel = availableShipsTable.getColumnModel();
            TableColumn column = columnModel.getColumn( 3 );
            columnModel.removeColumn( column );
            column = columnModel.getColumn( 2 );
            columnModel.removeColumn( column );
            columnModel = resultTable.getColumnModel();
            column = columnModel.getColumn( 2 );
            columnModel.removeColumn( column );
            column = columnModel.getColumn( 1 );
            columnModel.removeColumn( column );
        }
        else
        {
            int width;
            TableColumnModel columnModel = resultTable.getColumnModel();
            width = 140;
            TableColumn column = columnModel.getColumn( 0 );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
            width = 100;
            column = columnModel.getColumn( 2 );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
            column = columnModel.getColumn( 1 );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );

            column = columnModel.getColumn( 4 );
            width = 110;
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
            column = columnModel.getColumn( 5 );
            column.setPreferredWidth( width );
            column.setMinWidth( width );
            column.setMaxWidth( width );
        }
    }

    private void updateLabel()
    {
        labelCoordsStart.setText( configData.getI18nText( "Flighttime.start" ) );
        labelCoordsTarget.setText( configData.getI18nText( "Flighttime.target" ) );
        labelShips.setText( configData.getI18nText( "fleet.ships" ) );
        labelSpeed.setText( configData.getI18nText( "Savecalc.table.speed" ) );
        labelFarmKing.setText( configData.getI18nText( "Flighttime.farmKing" ) );
        useAvailable.setText( configData.getI18nText( "Flighttime.useAvailable" ) );
        selectAll.setText( configData.getI18nText( "Flighttime.selectAll" ) );
        deselectAll.setText( configData.getI18nText( "Flighttime.deselectAll" ) );
        calculateButton.setText( configData.getI18nText( "Flighttime.calculate" ) );
        labelAllianceSpecialization.setText( configData.getI18nText( "Flighttime.allianceSpecialization" ) );
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel();
        Border border = BorderFactory.createEmptyBorder( 0, 6, 0, 6 );
        panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
        panel.setBorder( new EtchedBorder( EtchedBorder.LOWERED ) );

        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        ItemListener l = new ItemListener() {
            public void itemStateChanged( ItemEvent e )
            {
                actionComboBoxChanged( e );
            }
        };
        startPlanets.addItemListener( l );
        targetPlanets.addItemListener( l );

        JPanel northPanel = new JPanel( new FlowLayout( FlowLayout.LEFT, 6, 5 ) );

        GridBagConstraints gridConstraints = new GridBagConstraints();
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 0;
        gridConstraints.gridheight = 1;
        gridConstraints.gridwidth = 2;
        gridConstraints.fill = GridBagConstraints.NONE;
        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.weightx = 0.0;
        gridConstraints.weighty = 0.0;
        gridConstraints.insets = new Insets( 2, 2, 2, 2 );

        JPanel businessPanel = new JPanel( new GridBagLayout() );
        businessPanel.add( labelCoordsStart, gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( labelCoordsTarget, gridConstraints );
        gridConstraints.gridwidth = 1;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.weightx = 1.0;
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 1;
        businessPanel.add( launchPosition, gridConstraints );
        gridConstraints.gridx = 1;
        businessPanel.add( startPlanets, gridConstraints );
        gridConstraints.gridx = 2;
        businessPanel.add( new JLabel( spacer ), gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( targetPosition, gridConstraints );
        gridConstraints.gridx = 4;
        businessPanel.add( targetPlanets, gridConstraints );
        gridConstraints.gridx = 5;
        businessPanel.add(new JLabel(spacer), gridConstraints);
        gridConstraints.gridx = 6;
        businessPanel.add( labelAllianceSpecialization, gridConstraints );
        spinnerAllianceSpecialization = new JSpinner(new SpinnerNumberModel( 0, 0, 100, 1 ));
        spinnerAllianceSpecialization.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                availableShipsModel.fireTableDataChanged();
                actionCalculate();
            }
        });
        gridConstraints.gridx = 7;
        businessPanel.add(spinnerAllianceSpecialization, gridConstraints);
        northPanel.add( businessPanel );

        panel.add( northPanel );
        FocusListener f = new FocusAdapter() {
            public void focusLost( FocusEvent e )
            {
                JComponent src = ( JComponent )e.getSource();
                Object o = src.getParent();
                if( o.equals( launchPosition ) && launchPosition.getPosition().isRealPosition() )
                {
                    Planet p = new Planet( "", launchPosition.getPosition() );
                    int pos = boxContains( p, startPlanets );
                    startPlanets.setSelectedIndex( pos );
                }
                else if( o.equals( targetPosition ) )
                {
                    Planet p = new Planet( "", targetPosition.getPosition() );
                    int pos = boxContains( p, targetPlanets );
                    targetPlanets.setSelectedIndex( pos );
                }
                actionCalculate();
            }
        };
        launchPosition.addFocusListener( f );
        targetPosition.addFocusListener( f );

        businessPanel = new JPanel();
        businessPanel.setLayout( new BoxLayout( businessPanel, BoxLayout.X_AXIS ) );
        businessPanel.setBorder( border );
        businessPanel.add( labelShips );
        businessPanel.add( Box.createHorizontalGlue() );
        businessPanel.add( useAvailable );
        businessPanel.add( Box.createHorizontalStrut( 25 ) );
        businessPanel.add( selectAll );
        businessPanel.add( Box.createHorizontalStrut( 5 ) );
        businessPanel.add( deselectAll );
        businessPanel.add( Box.createHorizontalStrut( 5 ) );
        panel.add( businessPanel );
        useAvailable.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                availableShipsModel.fireTableDataChanged();
            }
        });
        selectAll.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                Fleet fleet = availableShipsModel.getFleet();
                for( Ship ship : Ship.movableShips( configData.getVersion() ) )
                {
                    // we unconditionally select all and let the table model
                    // sort it out and generate the update events
                    fleet.setCount( ship, 1l );
                }
                availableShipsModel.fireTableDataChanged();
            }
        });
        deselectAll.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                availableShipsModel.getFleet().clearShips();
                availableShipsModel.fireTableDataChanged();
            }
        });

        final DefaultTableCellRenderer col1Renderer = new AlternateLineTableCellRenderer();
        col1Renderer.setHorizontalAlignment( SwingConstants.RIGHT );
        final ResourceTableCellRenderer col24Renderer = new ResourceTableCellRenderer();
        final CheckBoxTableCellRenderer col5Renderer = new CheckBoxTableCellRenderer();
        col5Renderer.setHorizontalAlignment( SwingConstants.CENTER );
        JCheckBox checkBox = new JCheckBox();
        checkBox.setHorizontalAlignment( SwingConstants.CENTER );
        final TableCellEditor col5Editor = new DefaultCellEditor( checkBox );
        availableShipsModel = new AvailableShips( mainFrame );
        TableModelListener t = new TableModelListener() {
            public void tableChanged( TableModelEvent e )
            {
                actionCalculate();
            }
        };
        availableShipsModel.addTableModelListener( t );

        availableShipsTable = new TooltipTable( availableShipsModel ) {
            private static final long serialVersionUID = 4185065063401225124L;

            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case 1:
                        rv = col1Renderer;
                        break;
                    case 2:
                    case 3:
                    case 4:
                        rv = col24Renderer;
                        break;
                    case 5:
                        rv = col5Renderer;
                        break;
                    default:
                        rv = super.getCellRenderer( row, column );
                        break;
                }
                return rv;
            }

            public TableCellEditor getCellEditor(int row, int column)
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn =  convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case 5:
                        rv = col5Editor;
                        break;
                    default:
                        rv = super.getCellEditor( row, column );
                        break;
                }
                return rv;
            }

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = this.getPreferredSize();
                return size;
            }
        };
        availableShipsTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        DefaultTableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        availableShipsTable.setDefaultRenderer( Object.class, defaultCellRenderer );

        JScrollPane scrollPane = new JScrollPane( availableShipsTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( border );
        panel.add( scrollPane );

        for( int i = 10; i > 0; i-- )
        {
            boxSpeed.addItem( new Percent( 10 * i ) );
        }
        boxSpeed.setSelectedIndex( 0 );
        boxSpeed.setMaximumRowCount( 10 );

        panel.add( Box.createVerticalStrut( 10 ) );
        businessPanel = new JPanel( new FlowLayout( FlowLayout.LEFT, 4, 5 ) );
        businessPanel.add( labelSpeed );
        businessPanel.add( boxSpeed );
        businessPanel.add( labelFarmKing );
        businessPanel.add( boxFarmKing );
        panel.add( businessPanel );
        boxSpeed.addItemListener( l );
        boxFarmKing.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                actionCalculate();
            }
        });


        businessPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        calculateButton.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                actionCalculate();
            }

        });
        businessPanel.add( calculateButton );
        panel.add( businessPanel );

        panel.add( Box.createVerticalStrut( 10 ) );
        resultModel = new ResultModel();
        resultTable = new TooltipTable( resultModel ) {
            private static final long serialVersionUID = 4185065063401225124L;

            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case 0:
                        rv = col1Renderer;
                        break;
                    default:
                        rv = super.getCellRenderer( row, column );
                        break;
                }
                return rv;
            }

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = this.getPreferredSize();
                return size;
            }

            @Override
            public String getToolTipText(MouseEvent event)
            {
                String tooltip = null;
                int viewRow = rowAtPoint( event.getPoint() );
                int viewColumn = columnAtPoint( event.getPoint() );
                int modelRow = convertRowIndexToModel( viewRow );
                int modelColumn = convertColumnIndexToModel( viewColumn );
                if( modelRow >= 0 && modelColumn >= 0 )
                {
                    Object value = getModel().getValueAt( modelRow, modelColumn );
                    if( value instanceof String )
                    {
                        tooltip = value.toString();
                        if( modelColumn == 4 || modelColumn == 5 )
                        {
                            Date date = new Date( System.currentTimeMillis() + 2000 * time[modelRow] / (6 - modelColumn) );
                            tooltip += " [" + TimePanel.getDateFormat().format( date ) + "]";
                        }
                        if( tooltip.trim().length() == 0 )
                        {
                            tooltip = null;
                        }
                    }
                    else if( value instanceof Number )
                    {
                        if( ((Number)value).longValue() != 0 )
                        {
                            tooltip = value.toString();
                        }
                    }
                }
                return tooltip;
            }
        };
        resultTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        defaultCellRenderer = new AlternateLineTableCellRenderer();
        defaultCellRenderer.setHorizontalAlignment( SwingConstants.CENTER );
        resultTable.setDefaultRenderer( Object.class, defaultCellRenderer );
        scrollPane = new JScrollPane( resultTable,
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( border );
        panel.add( scrollPane );

        for( int i = 0; i < 25; i++ )
        {
            businessPanel = new JPanel( new GridLayout( 1, 1 ) );
            businessPanel.add( Box.createVerticalGlue() );
            panel.add( businessPanel );
        }

        return panel;
    }


    private void updateMyPlanets( JComboBox<Planet> planets )
	{
        Object selection = planets.getSelectedItem();
        planets.removeAllItems();
        List<Planet> myPlanets;
        int size = 0;
        if( planets == startPlanets )
        {
            myPlanets = new ArrayList<Planet>( mainFrame.getConfigData().getMyPlanets() );
            planets.addItem( new Planet( "-- " + configData.getI18nText( "Savecalc.ownPlanets" ) + "  --", new Position( 0, 0, 0) ) );
            size++;
        }
        else
        {
            myPlanets = new ArrayList<Planet>( mainFrame.getConfigData().getPlanets() );
            planets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.planets" ) + "  --", new Position( 0, 0, 0) ) );
            size++;
        }
        for( Planet planet : myPlanets )
        {
            planets.addItem( planet );
            size++;
        }
        if( planets == startPlanets )
        {
            List<Planet> asteroids = new ArrayList<Planet>( configData.getMyPlanets( false, true ) );
            if( asteroids.size() > 0 )
            {
                planets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.asteroids" ) + "  --", new Position( 0, 0, 0) ) );
                size++;
                for( Planet asteroid : asteroids )
                {
                    planets.addItem( asteroid );
                    size++;
                }
            }
        }
        // restore selection
        if( selection != null )
        {
            planets.setSelectedItem( selection );
        }
        else
        {
            planets.setSelectedIndex( 0 );
        }
        planets.setMaximumRowCount( size );
	}

	/**
     * @param e
     */
    protected void actionComboBoxChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Object source = e.getSource();
            if( source == startPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    launchPosition.setPosition( position );
                availableShipsModel.fireTableDataChanged();
            }
            else if( source == targetPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    targetPosition.setPosition( position );
            }
            actionCalculate();
        }
    }

    protected int boxContains( Object o, JComboBox<? extends Object> box )
    {
        int rv = 0;
        int itemCount = box.getItemCount();
        for( int i = 0; i < itemCount; i++ )
        {
            if( o.equals( box.getItemAt( i ) ) )
            {
                rv = i;
                break;
            }
        }
        return rv;
    }

    protected void actionCalculate()
    {
        distance = new long[0];
        lowestSpeed.setPreptime( 0 );
        lowestSpeed.setAcceleration( 0 );
        lowestSpeed.setVelocity( 0 );
        time = new long[0];

        Version version = configData.getVersion();
        Position startPos = launchPosition.getPosition();
        Position targetPos = targetPosition.getPosition();
        if( startPos.isRealPosition() && targetPos.isRealPosition() )
        {
            double speedFactor = 1.0;
            Account current = configData.getActiveAccount();
            if( current != null )
            {
                speedFactor = current.getFleetSpeed();
                if( current.getVersion() == Version.v20 )
                {
                    if( !current.isSync() )
                    {
                        resultModel.fireTableDataChanged();
                        return;
                    }
                    Map<Position, Coordinates> map = configData.getCoordinates();
                    if( map.get( startPos ) == null ||  map.get( targetPos ) == null )
                    {
                        resultModel.fireTableDataChanged();
                        return;
                    }
                }
            }
            distance = getDistance( startPos, targetPos, version, current );
            int bonus = ((Number)spinnerAllianceSpecialization.getValue()).intValue();
            lowestSpeed = availableShipsModel.getFleet().getFleetVelocity(version, configData.getMyTech(), configData.getSpecialization(), bonus);
            if( boxFarmKing.isSelected()) {
                lowestSpeed.setVelocity(lowestSpeed.getVelocity() * (100 + configData.getSpecialization().getFarmking()) / 100);
            }
            int percent = ((Percent)boxSpeed.getSelectedItem()).getValue();
            if( lowestSpeed.getVelocity() > 0 )
            {
                time = new long[distance.length];
                for(int i = 0; i < distance.length; i++)
                {
                    time[i] = getDuration( lowestSpeed, distance[i], percent, speedFactor, version );
                }
            }
        }
        resultModel.fireTableDataChanged();
    }

    private class AvailableShips extends StatisticsTableModel
    {
        private static final long serialVersionUID = -6053748172198257547L;
        private Fleet fleet = new Fleet();

        public AvailableShips( MainFrame mainFrame )
        {
            super( mainFrame );
        }

        public void update()
        {
            fireTableStructureChanged();
        }

        public int getColumnCount()
        {
            return 6;
        }

        public int getRowCount()
        {
            Ship[] ships = Ship.movableShips( mainFrame.getConfigData().getVersion() );
            return ships.length;
        }

        public String getColumnName( int columnIndex )
        {
            String rv = "";
            switch( columnIndex )
            {
                case 1:
                    rv = configData.getI18nText( "Flighttime.available" );
                    break;
                case 2:
                    rv = configData.getI18nText( "Preptime" ).replace( "- ", "" );
                    break;
                case 3:
                    rv = configData.getI18nText( "Acceleration" ).replace( "- ", "" );
                    break;
                case 4:
                    rv = configData.getI18nText( "Savecalc.table.speed" );
                    break;
                case 5:
                    rv = configData.getI18nText( "Flighttime.number" );
                    break;
            }
            return rv;
        }

        public boolean isCellEditable( int rowIndex, int columnIndex )
        {
            boolean rv = ( columnIndex == 5 );
            if( rv && useAvailable.isSelected() )
            {
                Ship[] ships = Ship.movableShips( mainFrame.getConfigData().getVersion() );
                Planet planet = (Planet)startPlanets.getSelectedItem();
                rv &= planet.getFleet().getCount( ships[rowIndex] ) > 0;
            }
            return rv;
        }

        public Object getValueAt( int rowIndex, int columnIndex )
        {
            Object rv = null;
            Ship[] ships = Ship.movableShips( mainFrame.getConfigData().getVersion() );
            Ship ship = ships[rowIndex];
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( ships[rowIndex].name() );
                    break;
                case 1:
                    Planet planet = (Planet)startPlanets.getSelectedItem();
                    if( planet != null && launchPosition.getPosition().equals( planet.getPosition() ) )
                    {
                        long i = planet.getFleet().getCount( ships[rowIndex] );
                        if( i > 0 )
                        {
                            rv = Long.valueOf( i );
                        }
                        else
                        {
                            rv = "";
                        }
                    }
                    break;
                case 2:
                    rv = ConfigData.formatTime( ship.getPreparationTime() );
                    break;
                case 3:
                    int acceleration = (10 + configData.getMyTech().getTech( ship.getType() ) ) * ship.getAcceleration() / 10;
                    rv = format( acceleration );
                    break;
                case 4:
                    Fleet f = new Fleet();
                    f.setCount(ship, 1l);
                    int bonus = ((Number)spinnerAllianceSpecialization.getValue()).intValue();
                    long velocity = f.getFleetVelocity(configData.getVersion(), configData.getMyTech(), configData.getSpecialization(), bonus).getVelocity();
                    rv = format( velocity );
                    break;
                case 5:
                    if( useAvailable.isSelected() )
                    {
                        Planet p = (Planet)startPlanets.getSelectedItem();
                        if( ( p.getFleet().getCount( ships[rowIndex] ) == 0 ) && ( fleet.getCount( ships[rowIndex] ) > 0 ) )
                        {
                            // this is needed to get the update
                            setValueAt( Boolean.FALSE, rowIndex, columnIndex );
                        }
                    }
                    rv = Boolean.valueOf( fleet.getCount( ships[rowIndex] ) > 0 );
                    break;
            }
            return rv;
        }

        public void setValueAt(Object value, int rowIndex, int columnIndex)
        {
            switch( columnIndex )
            {
                case 5:
                    long count = 0;
                    Ship[] ships = Ship.movableShips( mainFrame.getConfigData().getVersion() );
                    if( ((Boolean)value).booleanValue() )
                    {
                        count = 1;
                    }
                    fleet.setCount( ships[rowIndex], Long.valueOf( count ) );
                    fireTableDataChanged();
                    break;
            }
        }

        public Fleet getFleet()
        {
            return fleet;
        }

        public boolean markResource( int row, int column )
        {
            boolean rv = false;
            Ship[] ships = Ship.movableShips( mainFrame.getConfigData().getVersion() );
            Ship ship = ships[row];
            switch( column )
            {
                case 2:
                    if( fleet.getCount( ship ) > 0 )
                    {
                        rv = ( ship.getPreparationTime() == fleet.getFleetVelocity( configData.getVersion(), configData.getMyTech(), configData.getSpecialization(), 0 ).getPreptime() );
                    }
                    break;
                case 3:
                    if( fleet.getCount( ship ) > 0 )
                    {
                        Technology tech = configData.getMyTech();
                        int acceleration;
                        acceleration = (10 + tech.getTech( ship.getType() ) ) * ship.getAcceleration() / 10;
                        rv = ( acceleration == fleet.getFleetVelocity( configData.getVersion(), tech, configData.getSpecialization(), 0 ).getAcceleration() );
                    }
                    break;
                case 4:
                    if( fleet.getCount( ship ) > 0 )
                    {
                        Version version = configData.getVersion();
                        Technology tech = configData.getMyTech();
                        long velocity;
                        if( version != Version.v20 )
                        {
                            velocity = (100 + configData.getSpecialization().getSpeedbonus(ship) + 10 * tech.getTech( ship.getType() ) ) * ship.getVelocity() / 100;
                        }
                        else
                        {
                            velocity = (20 + tech.getTech( ship.getType() ) ) * ship.getVelocity2() / 20;
                        }
                        rv = ( velocity == fleet.getFleetVelocity( configData.getVersion(), tech, configData.getSpecialization(), 0 ).getVelocity() );
                    }
                    break;
            }
            return rv;
        }
    }

    private class ResultModel extends AbstractTableModel
    {
        /**
         *
         */
        private static final long serialVersionUID = -1756696081829178787L;

        public String getColumnName( int columnIndex )
        {
            String rv = "";
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( "Flighttime.distance" );
                    break;
                case 1:
                    rv = configData.getI18nText( "Preptime" ).replace( "- ", "" );
                    break;
                case 2:
                    rv = configData.getI18nText( "Acceleration" ).replace( "- ", "" );
                    break;
                case 3:
                    rv = configData.getI18nText( "Savecalc.table.speed" );
                    break;
                case 4:
                    rv = configData.getI18nText( "Savecalc.table.oneway" );
                    break;
                case 5:
                    rv = configData.getI18nText( "Savecalc.table.total" );
                    break;
            }
            return rv;
        }


        public int getColumnCount()
        {
            return 6;
        }

        public int getRowCount()
        {
            return distance.length;
        }

        public Object getValueAt( int rowIndex, int columnIndex )
        {
            Object rv = "";

            switch( columnIndex )
            {
                case 0:
                    if( distance[rowIndex] > 0 )
                    {
                        rv = TotalPointDisplay.format( distance[rowIndex] );
                    }
                    break;
                case 1:
                    if( lowestSpeed.getVelocity() > 0 )
                    {
                        rv = ConfigData.formatTime( lowestSpeed.getPreptime() );
                    }
                    break;
                case 2:
                    if( lowestSpeed.getVelocity() > 0 )
                    {
                        rv = TotalPointDisplay.format( lowestSpeed.getAcceleration() );
                    }
                    break;
                case 3:
                    if( lowestSpeed.getVelocity() > 0 )
                    {
                        StringBuilder sb = new StringBuilder();
                        sb.append( TotalPointDisplay.format( lowestSpeed.getVelocity() ) );
                        sb.append( " x " );
                        sb.append( boxSpeed.getSelectedItem() );
                        sb.append( " (" );
                        sb.append( configData.getUnitFromMaxSpeed( lowestSpeed ) );
                        sb.append( ")" );
                        rv = sb.toString();
                    }
                    break;
                case 4:
                    if( time.length > rowIndex && time[rowIndex] > 0 )
                    {
                        rv = calculator.formatTime( time[rowIndex] / 2.0  );
                    }
                    break;
                case 5:
                    if( time.length > rowIndex && time[rowIndex] > 0 )
                    {
                        rv = calculator.formatTime( time[rowIndex] );
                    }
                    break;
            }
            return rv;
        }

    }
}
