/****************************************
 *  COPYRIGHT (C) 2008-2021
 *  Holger Graf
 ****************************************/
package sitool.savecalc;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Distance;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionField;
import sitool.components.Ship;
import sitool.components.Specialization;
import sitool.components.Technology;
import sitool.components.TimeField;
import sitool.components.TimePanel;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.SaveTargetComparator;
import sitool.table.SaveTargetComparatorMode;
import sitool.table.SaveTargetTableModel;
import sitool.table.SortableTableCellHeaderRenderer;
import sitool.table.SortableTableHeaderMouseListener;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class FlightVelocityPane extends JPanel implements SaveCalcPane
{
	private static final long serialVersionUID = -1878874776754387489L;

    protected JComboBox<Planet> startPlanets = new JComboBox<>();
    protected JComboBox<Planet> targetPlanets = new JComboBox<>();
    protected PositionField launchPosition = new PositionField();
    protected PositionField targetPosition = new PositionField();
    protected TimeField duration;

    private static final String spacer = "   ";

    private MainFrame mainFrame;
    protected ConfigData configData;

    private TimePanel timePanel;
    private JButton calculateButton = new JButton();
    private JLabel labelCoordsStart = new JLabel();
    private JLabel labelDuration = new JLabel();
    private JButton timeButton = new JButton();
    private JLabel labelCoordsTarget = new JLabel();
    private JLabel labelShips = new JLabel();
    private JLabel labelAllianceSpecialization = new JLabel();
    private JSpinner spinnerAllianceSpecialization;
    private JCheckBox useAvailable = new JCheckBox();
    private JCheckBox useTransporter = new JCheckBox();
    private JButton selectButton = new JButton();
    private Fleet fullFleet;
    private SaveTargetTableModel resultModel;
    private JTable resultTable;
    private SaveTargetComparator comparator = new SaveTargetComparator( SaveTargetComparatorMode.SORT_BY_DURATION );

    /**
     * @throws HeadlessException
     */
    public FlightVelocityPane( MainFrame parent ) throws HeadlessException
    {
        super();

        this.mainFrame = parent;
        configData = mainFrame.getConfigData();
        duration = new TimeField( configData );

        setLayout( new BorderLayout() );
        JPanel centerPanel = createCenterPanel();
        add( centerPanel, BorderLayout.CENTER );
    }

    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    /**
     *
     */
    public void update()
    {
        fullFleet = new Fleet();
        Long L = Long.valueOf( 1 );
        for( Ship ship : Ship.movableShips( configData.getVersion() ) )
        {
            fullFleet.setCount( ship, L );
        }
        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        Position position = launchPosition.getPosition();
        if( position.isRealPosition() )
        {
            updateBoxSelection( startPlanets, position );
        }
        position = targetPosition.getPosition();
        if( position.isRealPosition() )
        {
            updateBoxSelection( targetPlanets, position );
        }
        updateLabel();
        updateTable();
    }

    public void setDuration( long duration )
    {
        long readBack = 2 * duration;
        this.duration.setDay( Long.toString( readBack / Calculator.secondsPerDay ) );
        readBack %= Calculator.secondsPerDay;
        this.duration.setHour( Long.toString( readBack / Calculator.secondsPerHour ) );
        readBack %= Calculator.secondsPerHour;
        this.duration.setMinute( Long.toString( readBack / Calculator.secondsPerMinute ) );
        actionCalculate();
    }

    private void updateTable()
    {
        resultModel.fireTableStructureChanged();
        adjustTableColumns();
    }

    private void updateLabel()
    {
        labelCoordsStart.setText( configData.getI18nText( "Flighttime.start" ) );
        labelCoordsTarget.setText( configData.getI18nText( "Flighttime.target" ) );
        StringBuilder sb = new StringBuilder();
        sb.append( configData.getI18nText( "Flighttime.duration" ) );
        sb.append( " (" );
        sb.append( configData.getI18nText( "Savecalc.table.oneway" ) );
        sb.append( ")" );
        labelDuration.setText( sb.toString() );
        timeButton.setText( configData.getI18nText( "Savecalc.calculate" ) );
        labelShips.setText( configData.getI18nText( "fleet.ships" ) );
        labelAllianceSpecialization.setText( configData.getI18nText( "Flighttime.allianceSpecialization" ) );
        useAvailable.setText( configData.getI18nText( "Flighttime.useAvailable" ) );
        useTransporter.setText( configData.getI18nText( "Flighttime.useTransporter" ) );
        calculateButton.setText( configData.getI18nText( "Flightvelocity.calculate" ) );
        selectButton.setText( configData.getI18nText( "Flightvelocity.select" ) );
        timePanel.updateLabel();
    }

    protected void actionSaveTime()
    {
        JDialog dlg = new DateBox( this );
        mainFrame.showModalBox( dlg );
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel();
        Border border = BorderFactory.createEmptyBorder( 0, 6, 0, 6 );
        panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
        panel.setBorder( new EtchedBorder( EtchedBorder.LOWERED ) );

        updateMyPlanets( startPlanets );
        updateMyPlanets( targetPlanets );
        ItemListener l = new ItemListener() {
            public void itemStateChanged( ItemEvent e )
            {
                actionComboBoxChanged( e );
            }
        };
        startPlanets.addItemListener( l );
        targetPlanets.addItemListener( l );

        JPanel northPanel = new JPanel( new FlowLayout( FlowLayout.LEFT, 6, 5 ) );

        GridBagConstraints gridConstraints = new GridBagConstraints();
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 0;
        gridConstraints.gridheight = 1;
        gridConstraints.gridwidth = 2;
        gridConstraints.fill = GridBagConstraints.NONE;
        gridConstraints.anchor = GridBagConstraints.WEST;
        gridConstraints.weightx = 0.0;
        gridConstraints.weighty = 0.0;
        gridConstraints.insets = new Insets( 2, 2, 2, 2 );

        JPanel businessPanel = new JPanel( new GridBagLayout() );
        businessPanel.add( labelCoordsStart, gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( labelCoordsTarget, gridConstraints );
        gridConstraints.gridwidth = 1;
        gridConstraints.fill = GridBagConstraints.BOTH;
        gridConstraints.weightx = 1.0;
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 1;
        businessPanel.add( launchPosition, gridConstraints );
        gridConstraints.gridx = 1;
        businessPanel.add( startPlanets, gridConstraints );
        gridConstraints.gridx = 2;
        businessPanel.add( new JLabel( spacer ), gridConstraints );
        gridConstraints.gridx = 3;
        businessPanel.add( targetPosition, gridConstraints );
        gridConstraints.gridx = 4;
        businessPanel.add( targetPlanets, gridConstraints );
        gridConstraints.gridx = 5;
        businessPanel.add(new JLabel(spacer), gridConstraints);
        gridConstraints.gridx = 6;
        businessPanel.add( labelAllianceSpecialization, gridConstraints );
        spinnerAllianceSpecialization = new JSpinner(new SpinnerNumberModel( 0, 0, 100, 1 ));
        spinnerAllianceSpecialization.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                actionCalculate();
            }
        });
        gridConstraints.gridx = 7;
        businessPanel.add(spinnerAllianceSpecialization, gridConstraints);
        northPanel.add( businessPanel );
        panel.add( northPanel );
        FocusListener f = new FocusAdapter() {
            public void focusLost( FocusEvent e )
            {
                JComponent src = ( JComponent )e.getSource();
                Object o = src.getParent();
                if( o.equals( launchPosition ) && launchPosition.getPosition().isRealPosition() )
                {
                    updateBoxSelection( startPlanets, launchPosition.getPosition() );
                }
                else if( o.equals( targetPosition ) )
                {
                    updateBoxSelection( targetPlanets, targetPosition.getPosition() );
                }
                actionCalculate();
            }
        };
        launchPosition.addFocusListener( f );
        targetPosition.addFocusListener( f );

        gridConstraints.gridx = 0;
        gridConstraints.gridy = 2;
        businessPanel.add( labelDuration, gridConstraints );
        gridConstraints.gridx = 0;
        gridConstraints.gridy = 3;
        businessPanel.add( duration, gridConstraints );
        duration.addFocusListener( f );
        gridConstraints.gridx = 1;
        businessPanel.add( timeButton, gridConstraints );
        timeButton.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                actionSaveTime();
            }

        });

        businessPanel = new JPanel();
        businessPanel.setLayout( new BoxLayout( businessPanel, BoxLayout.X_AXIS ) );
        businessPanel.setBorder( border );
        businessPanel.add( labelShips );
        businessPanel.add( Box.createHorizontalStrut( 25 ) );
        businessPanel.add( useAvailable );
        businessPanel.add( Box.createHorizontalStrut( 25 ) );
        businessPanel.add( useTransporter );
        businessPanel.add( Box.createHorizontalGlue() );
        panel.add( businessPanel );
        ActionListener a = new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                actionCalculate();
            }
        };
        useAvailable.addActionListener( a );
        useTransporter.addActionListener( a );

        businessPanel = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        calculateButton.addActionListener( a );
        businessPanel.add( calculateButton );
        panel.add( businessPanel );

        panel.add( Box.createVerticalStrut( 10 ) );
        JPanel tablePanel = new JPanel( new BorderLayout() );
        tablePanel.setBorder( border );
        resultModel = new SaveTargetTableModel( mainFrame, false );
        final CheckBoxTableCellRenderer col2Renderer = new CheckBoxTableCellRenderer();
        resultTable = new TooltipTable( resultModel ) {
            private static final long serialVersionUID = -608088209614043437L;

            @Override
            public TableCellRenderer getCellRenderer( int row, int column ) {
                if(convertColumnIndexToModel(column) == 2 ) {
                    return col2Renderer;
                }
                return super.getCellRenderer(row, column);
            }

        };
        resultTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        resultTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        TableCellRenderer delegate = resultTable.getTableHeader().getDefaultRenderer();
        resultTable.getTableHeader().setDefaultRenderer( new SortableTableCellHeaderRenderer( delegate ) );
        resultTable.getTableHeader().addMouseListener( new SortableTableHeaderMouseListener() );
        DefaultTableCellRenderer defaultCellRenderer = new AlternateLineTableCellRenderer();
        defaultCellRenderer.setHorizontalAlignment( SwingConstants.CENTER );
        resultTable.setDefaultRenderer( Object.class, defaultCellRenderer );
        JScrollPane scrollPane = new JScrollPane( resultTable,
                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 6 ) );
        JTable rowHeader = resultModel.getRowHeaderTable();
        scrollPane.setRowHeaderView( rowHeader );
        scrollPane.setCorner( ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeader.getTableHeader() );
        tablePanel.add( scrollPane, BorderLayout.CENTER );
        Dimension size = startPlanets.getPreferredSize();
        size.width = SaveCalc.buttonWidth;
        size.height += 6;
        selectButton.setPreferredSize( size );
        selectButton.setHorizontalAlignment( SwingConstants.CENTER );
        selectButton.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionSelectClosest();
            }
        });
        JPanel buttonPanel = new JPanel( new BorderLayout() );
        buttonPanel.add( selectButton, BorderLayout.NORTH );
        // add some padding
        buttonPanel.add( Box.createVerticalStrut( 10 ), BorderLayout.CENTER );
        timePanel = new TimePanel( configData, resultTable );
        buttonPanel.add( timePanel, BorderLayout.SOUTH );
        tablePanel.add( buttonPanel, BorderLayout.EAST );
        panel.add( tablePanel );

        for( int i = 0; i < 25; i++ )
        {
            businessPanel = new JPanel( new GridLayout( 1, 1 ) );
            businessPanel.add( Box.createVerticalGlue() );
            panel.add( businessPanel );
        }

        return panel;
    }


    private void updateMyPlanets( JComboBox<Planet> planets )
    {
        Object selection = planets.getSelectedItem();
        planets.removeAllItems();
        List<Planet> myPlanets;
        int size = 0;
        if( planets == startPlanets )
        {
            myPlanets = new ArrayList<Planet>( mainFrame.getConfigData().getMyPlanets() );
            planets.addItem( new Planet( "-- " + configData.getI18nText( "Savecalc.ownPlanets" ) + "  --", new Position( 0, 0, 0) ) );
            size++;
        }
        else
        {
            myPlanets = new ArrayList<Planet>( mainFrame.getConfigData().getPlanets() );
            planets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.planets" ) + "  --", new Position( 0, 0, 0) ) );
            size++;
        }
        for( Planet planet : myPlanets )
        {
            planets.addItem( planet );
            size++;
        }
        if( planets == startPlanets )
        {
            List<Planet> asteroids = new ArrayList<Planet>( configData.getMyPlanets( false, true ) );
            if( asteroids.size() > 0 )
            {
                planets.addItem( new Planet( "-- " + configData.getI18nText( "MainFrame.asteroids" ) + "  --", new Position( 0, 0, 0) ) );
                size++;
                for( Planet asteroid : asteroids )
                {
                    planets.addItem( asteroid );
                    size++;
                }
            }
        }
        // restore selection
        if( selection != null )
        {
            planets.setSelectedItem( selection );
        }
        else
        {
            planets.setSelectedIndex( 0 );
        }
        planets.setMaximumRowCount( size );
    }

	/**
     * @param e
     */
    protected void actionComboBoxChanged( ItemEvent e )
    {
        if( e.getStateChange() == ItemEvent.SELECTED )
        {
            Object source = e.getSource();
            if( source == startPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    launchPosition.setPosition( position );
            }
            else if( source == targetPlanets )
            {
                Planet planet = ( Planet )e.getItem();
                Position position = planet.getPosition();
                if( position.isRealPosition() )
                    targetPosition.setPosition( position );
            }
            actionCalculate();
        }
    }

    /**
     * @param maxSpeed
     * @return
     */
    private double getUnitSpeed( Ship ship )
    {
        // we need rv to be long so it is properly truncated
        long rv;
        Technology myTech = configData.getMyTech();
        Specialization specialization = configData.getSpecialization();

        int boost = 0;
        switch( ship.getType() )
        {
            case Rocketdrive:
                boost = myTech.getRocket();
                break;
            case Diffusiondrive:
                boost = myTech.getDiffusion();
                break;
            case Warpdrive:
                boost = myTech.getWarp();
                break;
        }
        int bonus = ((Number)spinnerAllianceSpecialization.getValue()).intValue();
        rv = ( 100 + specialization.getSpeedbonus(ship) + 10 * boost + bonus ) * ship.getVelocity() / 100;
        return rv;
    }

    protected int boxContains( Object o, JComboBox<Planet> box )
    {
        int rv = 0;
        int itemCount = box.getItemCount();
        for( int i = 0; i < itemCount; i++ )
        {
            if( o.equals( box.getItemAt( i ) ) )
            {
                rv = i;
                break;
            }
        }
        return rv;
    }

    protected void actionCalculate()
    {
        long[] distance;
        long duration;
        Fleet fleet;

        Position startPos = launchPosition.getPosition();
        Position targetPos = targetPosition.getPosition();
        List<SaveTarget> data = new ArrayList<SaveTarget>();
        if( startPos.isRealPosition() && targetPos.isRealPosition() )
        {
            Planet origin = configData.getPlanetOnPosition( startPos );
            Planet target = configData.getPlanetOnPosition( targetPos );
            if( useAvailable.isSelected() )
            {
                Planet p = (Planet)startPlanets.getSelectedItem();
                fleet = p.getFleet();
            }
            else
            {
                fleet = fullFleet;
            }
            distance = Distance.getDistance( configData.getVersion(), startPos, targetPos );
            try
            {
                double speedFactor = 1.0;
                Account current = configData.getActiveAccount();
                if( current != null )
                {
                    speedFactor = current.getFleetSpeed();
                }
                duration = Calculator.parseTime( this.duration.getDay(), this.duration.getHour(), this.duration.getMinute() );
                if( duration > 0 )
                {
                    for(long actualDistance : distance)
                    {
                        double normal = Calculator.calculateSpeed( actualDistance, duration, speedFactor );
                        for( Ship ship : Ship.movableShips( configData.getVersion() ) )
                        {
                            if( fleet.getCount( ship ) > 0 )
                            {
                                if( useTransporter.isSelected() && ship.getLoad() < 6000 )
                                {
                                    continue;
                                }
                                double velocity = getUnitSpeed( ship );
                                // we need a linear relationship
                                int percent = (int)( 10 * Math.sqrt( normal / velocity ) ) * 10;
                                if( percent > 0 && percent <= 100 )
                                {
                                    String unit = configData.getI18nText( ship.name() );
                                    // apply speed modifier
                                    long actualDuration = 100 * Calculator.calculateDuration( velocity, actualDistance, speedFactor ) / percent;
                                    SaveTarget saveTarget = new SaveTarget( origin, target, Calculator.isLongDistance(origin, target, actualDistance), null, percent, unit, actualDuration, duration, current );
                                    data.add( saveTarget );
                                    percent += 10;
                                    if( percent <= 100 )
                                    {
                                        actualDuration = 100 * Calculator.calculateDuration( velocity, actualDistance, speedFactor ) / percent;
                                        saveTarget = new SaveTarget( origin, target, Calculator.isLongDistance(origin, target, actualDistance), null, percent, unit, actualDuration, duration, current );
                                        data.add( saveTarget );
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch( NumberFormatException ex )
            {}

            Collections.sort( data, comparator );
        }
        resultModel.setData( data );
    }

    protected void actionSelectClosest()
    {
        long delta = Long.MAX_VALUE;
        int rowIndex = -1;
        List<SaveTarget> data = resultModel.getData();
        for( int i = 0; i < data.size(); i++ )
        {
            SaveTarget target = data.get( i );
            long l = target.getDelta();
            if( l < delta )
            {
                rowIndex = i;
                delta = l;
            }
            else if ( ( l == delta )
                   && ( rowIndex >= 0 )
                   && ( target.getVelocity() < data.get( rowIndex ).getVelocity() ) )
            {
                rowIndex = i;
            }
        }

        if( rowIndex >= 0 )
        {
            resultTable.getSelectionModel().setSelectionInterval( rowIndex, rowIndex );
            // show newly selected row
            // since scrollRectToVisible() will just show the rect add some padding
            int lower = Math.max( rowIndex - 4, 0 );
            int upper = Math.min( rowIndex + 4, data.size() );
            Rectangle lRect = resultTable.getCellRect( lower, 0, true );
            Rectangle uRect = resultTable.getCellRect( upper, 0, true );
            Rectangle r = lRect.union( uRect );
            resultTable.scrollRectToVisible(r);
        }
    }

    protected void updateBoxSelection( JComboBox<Planet> box, Position position )
    {
        Planet p = new Planet( "", position );
        int pos = boxContains( p, box );
        box.setSelectedIndex( pos );
    }

    private void adjustTableColumns()
    {
        TableColumnModel model = resultTable.getColumnModel();
        int tableColumnWidths[] = new int[] { 25, 50, 96, 94, 94 };
        // lets lock columns 2-6
        for( int column = 2; column < model.getColumnCount(); column++ )
        {
            TableColumn tableColumn = model.getColumn( column );
            int width = tableColumnWidths[column-2];
            tableColumn.setPreferredWidth( width );
            tableColumn.setMinWidth( width );
            tableColumn.setMaxWidth( width );
        }
    }

}
