/****************************************
 *  COPYRIGHT (C) 2006
 *  Holger Graf
 ****************************************/
package sitool.statistic;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.TotalPointDisplay;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class AZPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -3083447829795125052L;

    private static long start[][] = new long[][] {
        {
            0, 50, 200, 400, 800, 1200, 2000, 5000, 10000, 20000, 40000, 90000, 200000, 500000, 2000000, 10000000,
            25_000_000, 50_000_000
        },
        {
            0, 86, 200, 400, 800, 1200, 2000, 5000, 10000, 20000, 40000, 90000, 200000, 500000, 1000000, 5000000, 10000000,
            50_000_000, 100_000_000, 500_000_000, 1_000_000_000, 2_000_000_000
        }
    };
    private static final Color strong = new Color( 165, 45, 20 );
    private static final Color hostile = new Color( 255, 50, 40 );
    private static final Color noob = new Color( 50, 110, 165 );

    private ConfigData configData;
    private MainFrame mainFrame;
    private Font normal;
    private Font bold;
    private Color unmarked;
    private JLabel[] zones = new JLabel[Math.max(start[0].length, start[1].length)];
    private JLabel[] points = new JLabel[zones.length];

    public AZPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();

        add( center, BorderLayout.CENTER );
    }

    private JPanel createCenterPanel()
    {
        JPanel center = new JPanel(new FlowLayout( FlowLayout.CENTER ) );
        JPanel inner = new JPanel( new GridLayout( zones.length, 2) );
        center.add( inner );

        for( int i = 0; i < zones.length; i++ )
        {
            zones[i] = new JLabel();
            zones[i].setHorizontalAlignment( SwingConstants.LEFT );
            inner.add( zones[i] );
            points[i] = new JLabel();
            points[i].setHorizontalAlignment( SwingConstants.RIGHT );
            inner.add( points[i] );
        }
        normal = zones[0].getFont();
        normal = normal.deriveFont( Font.PLAIN, normal.getSize2D() + 1.0f );
        bold = normal.deriveFont( Font.BOLD );
        unmarked = zones[0].getForeground();

        return center;
    }

    /**
     * @see sitool.components.ModulePane#update()
     */
    public void update()
    {
        long totalPoints = mainFrame.getDisplay().getTotalPoints();
        Version version = configData.getVersion();
        int myZone = 0;
        int select = (version == Version.v20) ? 1 : 0;
        for( int i = 0; i < start[select].length; i++ )
        {
            zones[i].setText( configData.getI18nText( "attack.zone.zone", i + 1 ) );
            String begin = TotalPointDisplay.format( start[select][i] );
            String end = ( i < start[select].length - 1 ) ? TotalPointDisplay.format( start[select][i+1] - 1 ) : "\u221E";
            points[i].setText( configData.getI18nText( "attack.zone.points", begin, end ) );

            //mark own zone
            boolean mark = ( totalPoints >= start[select][i] ) && configData.getActiveAccount() != null;
            if( i < start[select].length - 1 )
            {
                mark &= ( totalPoints < start[select][i+1] );
            }
            if( mark )
            {
                myZone = i;
                zones[i].setFont( bold );
                points[i].setFont( bold );
            }
            else
            {
                zones[i].setFont( normal );
                points[i].setFont( normal );
            }
        }

        for( int i = 0; i < zones.length; i++ )
        {
            Color fg;
            if( configData.getActiveAccount() != null )
            {
                fg = getMark(i, totalPoints, myZone);
            }
            else
            {
                fg = unmarked;
            }
            zones[i].setForeground( fg );
            points[i].setForeground( fg );

            zones[i].setVisible( i < start[select].length );
            points[i].setVisible( i < start[select].length );

        }
    }

    private Color getMark(int zone, long points, int myZone) {
        Version version = configData.getVersion();
        long zones[] = start[(version == Version.v20) ? 1 : 0];
        Color rv = unmarked;

        if( version == Version.sirius || version == Version.v20) {
            int lower = (zone < zones.length) ? zone : zones.length - 1;
            int upper = (zone + 1 < zones.length) ? zone + 1 : zones.length - 1;
            if( (zone < 8) || (zones[upper] < points / 4)  && (zone < zones.length - 1)) {
                rv = noob;
            }
            else if( zones[lower] > 4 * points) {
                rv = strong;
            } else
            {
                rv = hostile;
            }
        }
        else {
            if( (zone < 8) || (zone < myZone - 2) ) {
                rv = noob;
            }
            else if( zone > myZone + 2 ) {
                rv = strong;
            } else
            {
                rv = hostile;
            }
        }

        return rv;
    }
}
