/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

/**
 * @author graf
 */
public class CellCopyHandler extends MouseAdapter implements ActionListener
{
	private JPopupMenu popupMenu = new JPopupMenu();
	private String mostRecentValue;
	private Transferable contents = new Transferable()
	{
		public Object getTransferData(DataFlavor flavor)
						throws UnsupportedFlavorException, IOException
		{
			if( flavor != DataFlavor.stringFlavor )
			{
				throw new UnsupportedFlavorException( flavor );
			}
			
			return (mostRecentValue != null) ? mostRecentValue : "";
		}

		public DataFlavor[] getTransferDataFlavors()
		{
			return new DataFlavor[] { DataFlavor.stringFlavor };
		}

		public boolean isDataFlavorSupported(DataFlavor flavor)
		{
			return ( flavor == DataFlavor.stringFlavor );
		}
	};

	
	public CellCopyHandler()
	{
		JMenuItem menuItem = new JMenuItem( "Kopieren" );
		menuItem.addActionListener( this );
		popupMenu.add(menuItem);
	}

	public void mouseReleased(MouseEvent e)
	{
		showPopup(e);
	}

	private void showPopup(MouseEvent e)
	{
		if (e.isPopupTrigger())
		{
			if( e.getComponent() instanceof JTable )
			{
				JTable originator = (JTable) e.getComponent();
				Point p = e.getPoint();
				int rowIndex = originator.rowAtPoint(p);
		        int colIndex = originator.columnAtPoint(p);
		        mostRecentValue = numericValue( originator.getValueAt( rowIndex, colIndex ) );
		        popupMenu.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	public void actionPerformed(ActionEvent e)
	{
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents( contents, null );
	}
	
	private String numericValue( Object value )
	{
        StringBuffer sb = new StringBuffer();
        if( value != null )
        {
	        sb.append( value.toString() );
	        for( int i = sb.length() - 1; i >= 0; i-- )
	        {
	            char ch = sb.charAt( i );
	            if( !Character.isDigit( ch ) )
	            {
	                sb.deleteCharAt( i );
	            }
	        }
        }
        // if nothing is left, use '0'
        if( sb.length() == 0 )
        {
            sb.append( '0' );
        }
        return sb.toString();
	}
}
