/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.table;

import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.CostReduction;
import sitool.components.Defense;
import sitool.components.Resource;
import sitool.components.Ship;
import sitool.statistic.DefenseCalculator;

/**
 * @author graf
 *
 */
public class FleetBuilderTableModel extends StatisticsTableModel
{
    private static final long serialVersionUID = -4539721329053509477L;
    public static final int rows = Ship.values().length + Defense.values().length - 2;

    private int weaponsfactoryLevel = 0;
    private int msaLevel = 0;
    private boolean isEngineerEnabled = false;
    private CostReduction costReduction = CostReduction.normal;

    private long[] count = new long[rows];

    public FleetBuilderTableModel( MainFrame mainFrame )
    {
        super( mainFrame );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = 9;

        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        String[] parts;
        switch( column )
        {
            case 0:
                rv = formatHeader( "", "" );
                break;
            case 1:
                parts = configData.getI18nText( "Number" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case 2:
                parts = configData.getI18nText( "Buildtime" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case 3:
            case 4:
            case 5:
            case 6:
                rv = formatHeader( configData.getI18nText( Resource.values()[column-3].name() ), "" );
                break;
            case 7:
                parts = configData.getI18nText( "Transmitters" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case 8:
                rv =  formatHeader( configData.getI18nText( "Points" ).split( " ", 2 ) );
                break;

        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return rows + 2;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        Cost cost;
        if( rowIndex < rows )
        {
            switch( columnIndex )
            {
                case 0:
                    if( rowIndex < Ship.values().length - 1 )
                    {
                        Ship ship = Ship.values()[rowIndex + 1];
                        rv = configData.getI18nText( ship.name() );
                    }
                    else
                    {
                        Defense defense = Defense.values()[rowIndex - Ship.values().length + 1];
                        rv = configData.getI18nText( defense.name() );
                    }
                    break;
                case 1:
                    rv = Long.valueOf( count[rowIndex] );
                    break;
                case 2:
                    rv = formatTime( getTime( rowIndex ) );
                    break;
                case 3:
                case 4:
                case 5:
                case 6:
                    cost = getCost( rowIndex, true );
                    rv = format( cost.getResource( Resource.values()[columnIndex-3] ) * count[rowIndex] );
                    break;
                case 7:
                    cost = getCost( rowIndex, true );
                    long l = 0;
                    if( count[rowIndex] > 0 )
                    {
                    	l = cost.getIron() + cost.getMetal() + cost.getKryptonite() + cost.getSpice();
                    	l *= count[rowIndex];
                        int transmittercapacity = (100 + configData.getSpecialization().getCapacitybonus()) * Ship.transmitter.getLoad() / 100;
                        l /= transmittercapacity;
                        l++;
                    }
                    rv = format( l );
                    break;
                case 8:
                	cost = getCost( rowIndex, false ).multiply( count[rowIndex] );
                    rv = format( (long)cost.points() );
                    break;
            }
        }
        else if ( rowIndex == rows )
        {
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( "total" );
                    break;
                case 2:
                    rv = formatTime( getTime() );
                    break;
                case 3:
                case 4:
                case 5:
                case 6:
                    cost = getCost(true);
                    rv = format( cost.getResource( Resource.values()[columnIndex-3] ) );
                    break;
                case 7:
                    cost = getCost(true);
                    long l = cost.getIron() + cost.getMetal() + cost.getKryptonite() + cost.getSpice();
                    if( l > 0 )
                    {
                        int transmittercapacity = (100 + configData.getSpecialization().getCapacitybonus()) * Ship.transmitter.getLoad() / 100;
                        l /= transmittercapacity;
                        l++;
                    }
                    rv = format( l );
                    break;
                case 8:
                    cost = getCost(false);
                    rv = format( (long)cost.points() );
                    break;
            }
        }
        else
        {
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( "resourceequivalenz" );
                    break;
                case 2:
                case 7:
                case 8:
                    break;
                case 3:
                case 4:
                case 5:
                case 6:
                    double value = getWeightedResourceEquivalenz( getCost(true) );
                    int weight = Resource.values()[columnIndex-3].getWeight();
                    rv = format( Math.round( weight * value ) );
                    break;
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return ( columnIndex == 1 && rowIndex < rows );
    }

    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        switch( columnIndex )
        {
            case 1:
                long c = ((Number)value).longValue();
                count[rowIndex] = c;
                if( rowIndex >= rows - 2 )
                {
                    count[rowIndex] = ( c > 1 ) ? 1 : c;
                }
                break;
        }
    }

    public boolean markResource( int row, int column )
    {
        return false;
    }

    public void setWeaponsfactoryLevel( int weaponsfactoryLevel )
    {
        this.weaponsfactoryLevel = weaponsfactoryLevel;
        fireTableDataChanged();
    }

    public void setMsaLevel( int msaLevel )
    {
        this.msaLevel = msaLevel;
        fireTableDataChanged();
    }

    public void setEngineer( boolean isEngineerEnabled )
    {
        this.isEngineerEnabled = isEngineerEnabled;
        fireTableDataChanged();
    }

    protected Cost getCost( int rowIndex, boolean applybonus)
    {
        Cost cost = new Cost( 0, 0, 0, 0 );
        Ship ship = null;
        Defense defense = null;
        if( rowIndex < Ship.values().length - 1 )
        {
            ship = Ship.values()[rowIndex + 1];
            cost.add( ship.getCost() );
            if(applybonus) {
                cost.multiply(costReduction.getCost() / 100.0);
            }
        }
        else
        {
            defense = Defense.values()[rowIndex - Ship.values().length + 1];
            cost.add( defense.getCost() );
            if(applybonus) {
                double bonus = (100.0 - configData.getSpecialization().getDefensebonus()) / 100.0;
                cost.multiply(bonus);
            }
        }
        return cost;
    }

    protected Cost getCost(boolean applybonus)
    {
        Cost cost = new Cost( 0, 0, 0, 0 );
        for( int row = 0; row < rows; row++ )
        {
            cost.add( getCost( row, applybonus ).multiply( count[row] ) );
        }
        return cost;
    }

    protected long getTime( int rowIndex )
    {
        long rv = 0;
        if( count[rowIndex] > 0 )
        {
            double speedFactor = configData.getSpeedFactor(Ship.class);
            final boolean isShip = ( rowIndex < Ship.values().length - 1 );
            final boolean enableEngineer = (isShip) ? isEngineerEnabled : false;
            final Cost cost = getCost( rowIndex, false );
            rv = DefenseCalculator.buildTime( cost, enableEngineer, configData.getVersion() );
            rv = Math.round(rv /(((weaponsfactoryLevel > 0) ? weaponsfactoryLevel : 1 ) * Math.pow( 2, msaLevel ) * speedFactor));
            // it takes at least 1ms
            rv = (rv > 0) ? rv : 1;
            rv *= count[rowIndex];
            if( isShip ) {
                rv *= costReduction.getBuildtime();
            }
        }
        return rv;
    }

    protected long getTime()
    {
        long rv = 0;

        for( int row = 0; row < rows; row++ )
        {
            // each unit needs at least 1s to build, can't use total cost here
            rv += getTime( row );
        }

        return rv;
    }

    protected String formatTime( long buildTime )
    {
        if( buildTime == 0 )
        {
            return "";
        }

        return super.formatTime( buildTime, RoundMillis.UP );
    }

    public void setCostReduction( CostReduction costReduction ) {
        this.costReduction = costReduction;
        fireTableDataChanged();
    }
}
