/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.List;

import javax.swing.table.AbstractTableModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Planet;
import sitool.components.TotalPointDisplay;
import sitool.components.TotalPointSource;

/**
 * @author graf
 *
 */
public abstract class PlanetHeaderTableModel extends AbstractTableModel implements TotalPointSource
{
	private static final long serialVersionUID = -8854752303955966383L;

	protected MainFrame mainFrame;
    protected ConfigData configData;
    protected boolean isAsteroid;

    /**
     * @param mainFrame
     */
    public PlanetHeaderTableModel( MainFrame mainFrame, boolean isAsteroid )
    {
        this.mainFrame = mainFrame;
        configData = mainFrame.getConfigData();
        this.isAsteroid = isAsteroid;
    }

    protected String format( long value )
    {
        return TotalPointDisplay.format( value ); 
    }
    
    protected String formatHeader( String... lines )
    {
        StringBuilder sb = new StringBuilder();
        sb.append( "<html>" );
        sb.append( "<p align=\"left\">" );
        for( int i = 0; i < lines.length; i++ )
        {
            if( i > 0 )
            {
                sb.append( "<br>" );
            }
            if( lines[i].trim().length() == 0 )
            {
                lines[i] = "&nbsp;";
            }
            sb.append( lines[i] );
        }
        sb.append( "</p>" );
        sb.append( "</html>" );
        return sb.toString();
    }

    protected String getAllPoints()
    {
        long total = getTotalPoints();
        StringBuilder sb = new StringBuilder();
        sb.append( configData.getI18nText( "total.points" ) );
        sb.append( format( total ) );
        return sb.toString(); 
    }
    
    protected String getPoints( int columnIndex )
    {
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        Planet target = data.get( columnIndex - 1);

        Cost total = totalCost( target );
        return format( (long)total.points() ); 
    }
    
    public long getTotalPoints()
    {
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        Cost total = new Cost( 0, 0, 0, 0 );
        for( Planet target : data )
        {
            total.add( totalCost( target ) );
        }
        return (long)total.points();
    }

    abstract protected Cost totalCost( Planet planet );
    
    Planet getPlanet( int columnIndex )
    {
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        Planet planet = null;
        if( columnIndex > 0 && columnIndex <= data.size() )
        {
            planet = data.get( columnIndex - 1 );
        }
        return planet;
    }
    
    MainFrame getMainFrame()
    {
        return mainFrame;
    }
    
}
