/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;

import sitool.components.Distance;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.PositionSet;

/**
 * @author graf
 *
 */
public class PlanetTableCellEditor extends AbstractCellEditor implements TableCellEditor
{
    private static final long serialVersionUID = -841141806646316608L;

    private JPanel renderer;
    private JLabel canvasF = new JLabel( "[xx:xxx:");
    private JLabel canvasB = new JLabel( "]");
    private JSpinner editor = new JSpinner( new SpinnerNumberModel( 8, 1, Distance.getUpperPlanetBound(), 1 ) );

    private Planet planet;
    protected int clickCountToStart = 2;

    public PlanetTableCellEditor()
    {
        Dimension size;
        size = editor.getPreferredSize();
        size.height = canvasF.getPreferredSize().height;
        editor.setPreferredSize( size );
        Font font = canvasF.getFont().deriveFont( Font.PLAIN );
        canvasF.setFont( font );
        canvasB.setFont( font );
        renderer = new JPanel();
        renderer.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 0 ) );
        renderer.setBorder( new EmptyBorder( 0, 4, 0, 4 ) );
        renderer.add( canvasF );
        renderer.add( editor );
        renderer.add( canvasB );
    }


    public Component getTableCellEditorComponent( JTable table, Object value, boolean isSelected,
                    int row, int column )
    {
        planet = getPlanet(value);
        Position position = planet.getPosition();
        StringBuilder sb = new StringBuilder();
        sb.append( "[" );
        if( position.getGalaxy() < 10 )
        {
            sb.append( '0' );
        }
        sb.append( position.getGalaxy() );
        sb.append( ':' );
        if( position.getSystem() < 10 )
        {
            sb.append( '0' );
        }
        sb.append( position.getSystem() );
        sb.append( ':' );
        canvasF.setText( sb.toString() );
        editor.setValue( getDefault( table, row, column ) );
        return renderer;
    }


    private Planet getPlanet( Object value ) {
        Planet p;
        if(value instanceof PositionSet) {
            p = new Planet("", ((PositionSet)value).getPosition());
        }
        else {
            p = (Planet)value;
        }
        return p;
    }


    private Integer getDefault( JTable table, int row, int column )
    {
        Integer rv;
        TableModel model = table.getModel();
        int modelColumn = table.convertColumnIndexToModel( column );
        if( modelColumn == 1 )
        {
            Object o = model.getValueAt( row, modelColumn );
            Position p;
            if(o instanceof PositionSet) {
                p = ((PositionSet)o).getPosition();
            }
            else {
                p = ((Planet)model.getValueAt( row, 0 )).getPosition();
            }
            rv = Integer.valueOf( p.getPlanet() );
        }
        else
        {
            rv = Integer.valueOf( 8 );
        }
        return rv;
    }


    public Object getCellEditorValue()
    {
        Position position = planet.getPosition();
        int p = ((Number)editor.getValue()).intValue();
        position = new Position( position.getGalaxy(), position.getSystem(), p, true );
        planet.setPosition( position );
        return planet;
    }

    /**
     * Specifies the number of clicks needed to start editing.
     *
     * @param count  an int specifying the number of clicks needed to start editing
     * @see #getClickCountToStart
     */
    public void setClickCountToStart(int count)
    {
        clickCountToStart = count;
    }

    /**
     * Returns the number of clicks needed to start editing.
     * @return the number of clicks needed to start editing
     */
    public int getClickCountToStart()
    {
        return clickCountToStart;
    }

    public boolean isCellEditable(EventObject anEvent)
    {
        if (anEvent instanceof MouseEvent)
        {
            return ((MouseEvent)anEvent).getClickCount() >= clickCountToStart;
        }
        return true;
    }
}
