/****************************************
 *  COPYRIGHT (C) 2022
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.AbstractCellEditor;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellEditor;

import sitool.ConfigData;
import sitool.components.ProductionBonus;
import sitool.components.ProductionBonusBox;

/**
 * @author graf
 *
 */
public class ProductionBonusTableCellEditor extends AbstractCellEditor implements TableCellEditor {

    private static final long serialVersionUID = -4445507524364165070L;

    private final ProductionBonus productionBonus;
    private final JLabel component = new JLabel();
    private final ProductionBonusBox productionBonusDialog;
    private final Component owner;

    public ProductionBonusTableCellEditor(ConfigData configData, Component owner) {
        productionBonus = new ProductionBonus();
        this.owner = owner;
        component.setBorder( new EmptyBorder(1, 1, 1, 1) );
        productionBonusDialog = new ProductionBonusBox(configData, null, true);
        component.setHorizontalAlignment(SwingConstants.CENTER);
    }

    @Override
    public ProductionBonus getCellEditorValue() {
        return productionBonus;
    }

    @Override
    public Component getTableCellEditorComponent( JTable table, Object value,
                    boolean isSelected, int row, int column ) {
        ProductionBonus bonus = (value != null) ? (ProductionBonus)value : new ProductionBonus();
        productionBonus.setValue(bonus);
        component.setText(bonus.toString());
        final Point point = owner.getLocationOnScreen();
        final Rectangle rect = table.getCellRect(row, column, true);
        // rect is relative to the tables which is somewhat below the dialog
        point.translate(rect.x - rect.width, rect.y + 140 - rect.height / 2);
        // we need this function to continue immediately, show the dialog in the UI thread
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                productionBonusDialog.show(ProductionBonusTableCellEditor.this, productionBonus, point);
            }
        });
        return component;
    }

    @Override
    public boolean stopCellEditing() {
        productionBonus.setValue(productionBonusDialog.getProductionBonus());
        return super.stopCellEditing();
    }
}
