/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableModel;

public class SaveTargetTableCellRenderer extends AlternateLineTableCellRenderer {
    private static final long serialVersionUID = 6231853290242615424L;

    public Component getTableCellRendererComponent( JTable table, Object value,
                    boolean isSelected, boolean hasFocus, int row, int column ) {
        Component renderer = super.getTableCellRendererComponent(table, value, isSelected,
                        hasFocus, row, column);
        if(renderer instanceof JLabel) {
            JLabel label = (JLabel)renderer;
            label.setBorder(new EmptyBorder(0, 4, 0, 4));
            int modelRow = table.convertRowIndexToModel(row);
            int modelColumn = table.convertColumnIndexToModel(column);
            switch (modelColumn){
                case 0:
                case 1:
                case 4:
                    label.setHorizontalAlignment(SwingConstants.LEFT);
                    break;
                case 3:
                    label.setHorizontalAlignment(SwingConstants.RIGHT);
                    break;
                default:
                    label.setHorizontalAlignment(SwingConstants.CENTER);
                    break;
            }
            mark(renderer, table, isSelected, hasFocus, modelRow, modelColumn);
        }
        return renderer;
    }

    private void mark( Component renderer, JTable table, boolean isSelected,
                    boolean hasFocus, int modelRow, int modelColumn ) {
        TableModel model = table.getModel();
        if(model instanceof ResourceMarker) {
            Color fg = null;
            Color selected = null;
            Font font = null;
            Font italic = null;

            fg = table.getForeground();
            font = table.getFont();
            italic = font.deriveFont(Font.ITALIC);

            if(isSelected) {
                selected = UIManager.getColor("Table.selectionForeground");
            }

            boolean mark = false;
            mark = ((ResourceMarker)model).markResource(modelRow, modelColumn);
            if(mark) {
                renderer.setForeground(Color.DARK_GRAY);
                renderer.setFont(italic);
            }
            else {
                if(isSelected) {
                    renderer.setForeground(selected);
                }
                else {
                    renderer.setForeground(fg);
                }
                renderer.setFont(font);
            }
        }
    }

}
