/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Ship;
import sitool.components.Version;
import sitool.statistic.DefenseCalculator;

/**
 * @author graf
 *
 */
public class ShipStatisticsTableModel2 extends StatisticsTableModel
{
    private static final long serialVersionUID = 6478711188531663017L;

    public final static int SHIPTYPE = 0;
    public final static int STRUCTURE = 1;
    public final static int PREPTIME = 2;
    public final static int ACCELERATION = 3;
    public final static int VELOCITY = 4;
    public final static int SHIELD = 5;
    public final static int FIREPOWER = 6;
    public final static int HOLDS = 7;
    public final static int CONSUMPTION = 8;

    public ShipStatisticsTableModel2( MainFrame mainFrame )
    {
        super( mainFrame );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = CONSUMPTION + 1;

        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        String parts[];
        switch( column )
        {
            case SHIPTYPE:
                rv = formatHeader( "", "" );
                break;
            case STRUCTURE:
                parts = configData.getI18nText( "Structure" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case PREPTIME:
                parts = configData.getI18nText( "Preptime" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case ACCELERATION:
                parts = configData.getI18nText( "Acceleration" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case VELOCITY:
                parts = configData.getI18nText( "Velocity" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case SHIELD:
                parts = configData.getI18nText( "Shield" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case FIREPOWER:
                parts = configData.getI18nText( "Firepower" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case HOLDS:
                parts = configData.getI18nText( "Holds" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;
            case CONSUMPTION:
                parts = configData.getI18nText( "Consumption" ).split( " ", 2 );
                rv = formatHeader( parts );
                break;

        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return Ship.ships( configData.getVersion() ).length;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        Version version = configData.getVersion();
        Ship ship = Ship.ships( version )[rowIndex];
        switch( columnIndex )
        {
            case SHIPTYPE:
                rv = configData.getI18nText( ship.name() );
                break;
            case STRUCTURE:
                rv = format( DefenseCalculator.getStructure( ship.getCost() ) );
                break;
            case PREPTIME:
                rv = ConfigData.formatTime( ship.getPreparationTime() );
                break;
            case ACCELERATION:
                rv = format( ship.getAcceleration() );
                break;
            case VELOCITY:
                long velocity = (version == Version.v20) ? ship.getVelocity2() : ship.getVelocity();
                rv = format( velocity );
                break;
            case SHIELD:
                rv = format( ship.getShields() );
                break;
            case FIREPOWER:
                rv = format( ship.getFirePower() );
                break;
            case HOLDS:
                rv = format( ship.getLoad() );
                break;
            case CONSUMPTION:
                rv = format( ship.getConsumption() );
                break;

        }
        return rv;
    }

    public boolean markResource( int row, int column )
    {
        return false;
    }

    protected String format( long value )
    {
        String rv;
        if( value == 0 )
        {
            rv = " - ";
        }
        else
        {
            rv =  super.format( value );
        }
        return rv;
    }

}
