/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.List;

import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;

import sitool.Account;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Fleet;
import sitool.components.Planet;
import sitool.components.Ship;
import sitool.components.Version;

/**
 * @author graf
 *
 */
public class ShipTableModel extends PlanetHeaderTableModel implements CellEditorListener
{
    private static final long serialVersionUID = 9045672684082762031L;

    public ShipTableModel( MainFrame mainFrame, boolean isAsteroid )
    {
        super( mainFrame, isAsteroid );
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int columns = configData.getMyPlanets( !isAsteroid, isAsteroid ).size() + 2;
        return columns;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int columnIndex )
    {
        String rv;
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        if( columnIndex == 0 )
        {
            rv = "";
        }
        else  if( columnIndex > data.size() )
        {
            rv = configData.getI18nText( "total" );
        }

        else
        {
            Planet target = data.get( columnIndex - 1);
            //return a 2 line header, this is cheating, but, hey it works
            return formatHeader( target.getName(), target.getPosition().toString() );
        }
        return rv;
    }


    /**
     * @see javax.swing.table.AbstractTableModel#getColumnClass(int)
     */
    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        if( columnIndex == 0 )
            return String.class;
        else
            return Number.class;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        // provide summary line
        return Ship.ships( configData.getVersion() ).length + 2;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        Object rv = null;
        Version version = configData.getVersion();
        int rows = Ship.ships( version ).length;
        Ship ship = null;
        if( rowIndex < Ship.ships( configData.getVersion() ).length)
        {
            ship = Ship.ships( version )[rowIndex];
        }

        if( columnIndex == 0 )
        {
            if( rowIndex == rows + 1 )
            {
                rv = "";
            }
            else if( rowIndex == rows )
            {
                rv = configData.getI18nText( "fleet.LoadCapacity" );
            }
            else
            {
                rv = configData.getI18nText( ship.name() );
            }
        }
        else
        {
            if( columnIndex > data.size() )
            {
                if( rowIndex < rows )
                {
                    int count = 0;
                    for( int c = 0; c < data.size(); c++ )
                    {
                        Planet target = data.get( c );
                        count += target.getFleet( isAsteroid ).getCount( ship );
                    }
                    if( count > 0 )
                    {
                        rv = format( count );
                    }
                    else
                    {
                        rv = "";
                    }
                }
                else if( rowIndex == rows )
                {
                    long cargo = 0L;
                    for( Planet target : data )
                    {
                        cargo += getCargoCapacity( target );
                    }
                    rv = format( cargo );
                }
                else if( rowIndex == rows + 1 )
                {
                    rv = format( super.getTotalPoints() );
                }
            }
            else
            {
                Planet target = data.get( columnIndex - 1);
                if( rowIndex < rows )
                {
                    long count = target.getFleet( isAsteroid ).getCount( ship );
                    rv = Long.valueOf( count );
                }
                else if( rowIndex == rows )
                {
                    rv = format( getCargoCapacity( target ) );
                }
                else if( rowIndex == rows + 1 )
                {
                    rv = getPoints( columnIndex );
                }
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        boolean rv;
        rv = ( columnIndex > 0  && columnIndex <= configData.getMyPlanets( !isAsteroid, isAsteroid ).size() && rowIndex < getRowCount() - 2 );
        return rv;
    }


    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        List<Planet> data = configData.getMyPlanets( !isAsteroid, isAsteroid );
        Planet target = data.get( columnIndex - 1 );

        Ship ship = Ship.ships( configData.getVersion() )[rowIndex];

        Long count = Long.valueOf( value.toString() );
        target.getFleet( isAsteroid ).setCount( ship, count );
    }

    public void editingCanceled( ChangeEvent e )
    {
    }

    public void editingStopped( ChangeEvent e )
    {
        // update the status rows
        fireTableRowsUpdated( getRowCount() - 2, getRowCount() - 1 );
    }

    @Override
    public long getTotalPoints() {
        long rv = 0;
        // if the account holds the total fleet points impoorted from JSON use these
        // else add all planets as before
        Account account = configData.getActiveAccount();
        if( account != null && account.hasTotalShips() ) {
            // only do this once b/c planet and asteroid points are added
            if( !isAsteroid ) {
                Cost total = fleetPoints(account.getTotalShips());
                rv = (long)total.points();
            }
            else {
                rv = 0;
            }
        }
        else {
            rv = super.getTotalPoints();
        }
        return rv;
    }

    @Override
    protected Cost totalCost( Planet planet ) {
        Fleet fleet = planet.getFleet( isAsteroid );
        Cost total = fleetPoints(fleet);
        return total;
    }

    private Cost fleetPoints(Fleet fleet) {
        Cost total = new Cost( 0, 0, 0, 0 );
        for( Ship ship : Ship.values() )
        {
            long count = fleet.getCount( ship );
            if( count > 0 )
            {
                Cost cost = new Cost( ship.getCost(), count );
                total.add( cost );
            }
        }
        return total;
    }

    protected long getCargoCapacity( Planet target )
    {
        long cargo = 0;
        int capacityBonus = configData.getSpecialization().getCapacitybonus();
        for( Ship ship : Ship.values() )
        {
            if( ship == Ship.none || ship == Ship.solarCell )
            {
                continue;
            }
            long count = target.getFleet( isAsteroid ).getCount( ship );
            long capacity = (100 + capacityBonus) * ship.getLoad() / 100;
            cargo +=  count * capacity;
        }

        return cargo;
    }
}
