/****************************************
 *  COPYRIGHT (C) 2008-2017
 *  Holger Graf
 ****************************************/
package sitool.table;

import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.AbstractTableModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Fleet;
import sitool.components.Ship;
import sitool.components.TotalPointDisplay;
import sitool.fleet.TacticalCalculator;

/**
 * @author graf
 *
 */
public class TacticalFleetStrengthTableModel extends AbstractTableModel implements CellEditorListener
{
    private static final long serialVersionUID = -4539721329053509477L;
    private static final Ship[] ships = TacticalCalculator.ships.toArray(new Ship[TacticalCalculator.ships.size()]);
    public static final int rows = ships.length;

    private final ConfigData configData;
    private Fleet fleet = new Fleet();
    private Fleet myShips;

    public TacticalFleetStrengthTableModel( MainFrame mainFrame )
    {
        this.configData = mainFrame.getConfigData();
        myShips = (configData.getActiveAccount() != null) ? configData.getActiveAccount().getTotalShips() : new Fleet();
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        int rv = 5;
        return rv;
    }

    /**
     * @see javax.swing.table.AbstractTableModel#getColumnName(int)
     */
    @Override
    public String getColumnName( int column )
    {
        String rv = "";
        switch( column )
        {
            case 0:
                rv = "";
                break;
            case 1:
                rv = configData.getI18nText( "Tactical.available" );
                break;
            case 2:
                rv = configData.getI18nText( "Tactical.number" );
                break;
            case 3:
                rv = configData.getI18nText( "Tactical.base" );
                break;
            case 4:
                rv = configData.getI18nText( "Tactical.bonus" );
                break;

        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return rows + 1;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        if( rowIndex < rows )
        {
            Ship ship = ships[rowIndex];
            long value;
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( ship.name() );
                    break;
                case 1:
                    value = myShips.getCount(ship);
                    rv = (value != 0) ? TotalPointDisplay.format( value ) : "";
                    break;
                case 2:
                    rv = Long.valueOf( fleet.getCount(ship) );
                    break;
                case 3:
                    value = TacticalCalculator.baseStrength(ship, fleet);
                    rv = (value != 0) ? TotalPointDisplay.format( value ) : "";
                    break;
                case 4:
                    value = TacticalCalculator.bonusStrength(ship, fleet);
                    rv = (value != 0) ? TotalPointDisplay.format( value ) : "";
                    break;
            }
        }
        else if ( rowIndex == rows )
        {
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( "total" );
                    break;
                case 1:
                    rv = TotalPointDisplay.format( TacticalCalculator.tacticalStrength(myShips) );
                    break;
                case 2:
                    rv = TotalPointDisplay.format( TacticalCalculator.tacticalStrength(fleet) );
                    break;
                case 3:
                    rv = TotalPointDisplay.format( TacticalCalculator.baseStrength(fleet) );
                    break;
                case 4:
                    rv = TotalPointDisplay.format( TacticalCalculator.bonusStrength(fleet) );
                    break;
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return ( columnIndex == 2 && rowIndex < rows );
    }

    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        Ship ship = ships[rowIndex];
        switch( columnIndex )
        {
            case 2:
                fleet.setCount(ship, (Long)value);
                fireTableDataChanged();
                break;
        }
    }

    public void fillValues() {
        fleet.setShips(myShips.getShips());
        fireTableDataChanged();
    }

    public void resetValues() {
        fleet.clearShips();
        fireTableDataChanged();
    }

    public void update() {
        myShips = (configData.getActiveAccount() != null) ? configData.getActiveAccount().getTotalShips() : new Fleet();
        fireTableDataChanged();
    }

    @Override
    public void editingStopped(ChangeEvent e) {
        fireTableDataChanged();
    }

    @Override
    public void editingCanceled(ChangeEvent e) {
    }

}
