/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.table;

import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.Research;
import sitool.components.Technology;
import sitool.components.TotalPointDisplay;
import sitool.components.TotalPointSource;
import sitool.technology.TechCalculator;

/**
 * @author graf
 *
 */
public class TechnologyTableModel extends AbstractTableModel  implements CellEditorListener, TotalPointSource
{
    private static final long serialVersionUID = -516892705671164626L;

    private MainFrame mainFrame;
    private ConfigData configData;
    public static final int summaryRow = Research.values().length;

    public TechnologyTableModel( MainFrame mainFrame )
    {
        this.mainFrame = mainFrame;
        configData = mainFrame.getConfigData();
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return 4;
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return summaryRow + 1;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = null;
        if( rowIndex == summaryRow  && columnIndex < 3 )
        {
            rv = "";
        }
        else
        {
            switch( columnIndex )
            {
                case 0:
                    rv = configData.getI18nText( Research.values()[rowIndex].name() );
                    break;
                case 1:
                    rv = getTechValue( rowIndex );
                    break;
                case 2:
                    rv = getHint( rowIndex );
                    break;
                case 3:
                    rv = getPoints( rowIndex );
                    break;
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return ( columnIndex == 1 );
    }


    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex)
    {
        switch( columnIndex )
        {
            case 1:
                setTechValue( value, rowIndex );
                break;
        }
    }

    private Integer getTechValue( int rowIndex )
    {
        int techValue = 0;
        Technology tech = configData.getMyTech();

        switch( rowIndex )
        {
            case 0:
                techValue = tech.getMining();
                break;
            case 1:
                techValue = tech.getEnergy();
                break;
            case 2:
                techValue = tech.getDrive();
                break;
            case 3:
                techValue = tech.getRocket();
                break;
            case 4:
                techValue = tech.getDiffusion();
                break;
            case 5:
                techValue = tech.getWarp();
                break;
            case 6:
                techValue = tech.getEspionage();
                break;
            case 7:
                techValue = tech.getWeapons();
                break;
            case 8:
                techValue = tech.getShields();
                break;
            case 9:
                techValue = tech.getStructure();
                break;
            case 10:
                techValue = tech.getTarget();
                break;
            case 11:
                techValue = tech.getParticle();
                break;
            case 12:
                techValue = tech.getGraviton();
                break;
            case 13:
                techValue = tech.getTeleportation();
                break;
            case 14:
                techValue = tech.getBlackhole();
                break;
        }
        return Integer.valueOf( techValue );
    }

    private Object setTechValue( Object value, int rowIndex )
    {
        int techValue = Integer.parseInt( value.toString() );
        Technology tech = mainFrame.getConfigData().getMyTech();

        switch( rowIndex )
        {
            case 0:
                tech.setMining( techValue );
                break;
            case 1:
                tech.setEnergy( techValue );
                break;
            case 2:
                tech.setDrive( techValue );
                break;
            case 3:
                tech.setRocket( techValue );
                break;
            case 4:
                tech.setDiffusion( techValue );
                break;
            case 5:
                tech.setWarp( techValue );
                break;
            case 6:
                tech.setEspionage( techValue );
                break;
            case 7:
                tech.setWeapons( techValue );
                break;
            case 8:
                tech.setShields( techValue );
                break;
            case 9:
                tech.setStructure( techValue );
                break;
            case 10:
                tech.setTarget( techValue );
                break;
            case 11:
                tech.setParticle( techValue );
                break;
            case 12:
                tech.setGraviton( techValue );
                break;
            case 13:
                tech.setTeleportation( techValue );
                break;
            case 14:
                tech.setBlackhole( techValue );
                break;
        }
        return Integer.valueOf( techValue );
    }

    private Object getHint( int rowIndex )
    {
        Technology tech = mainFrame.getConfigData().getMyTech();
        String rv = " ";

        switch( rowIndex )
        {
            case 0:
                rv = configData.getI18nText( "Bonus.mine.production", Integer.valueOf( 2 * tech.getMining() ) );
                break;
            case 1:
                rv = configData.getI18nText( "Bonus.energy.production", Integer.valueOf( tech.getEnergy() ) );
                break;
            case 3:
                rv = configData.getI18nText( "Bonus.rocket", Integer.valueOf( 10 * tech.getRocket() ) );
                break;
            case 4:
                rv = configData.getI18nText( "Bonus.diffusion", Integer.valueOf( 10 * tech.getDiffusion() ) );
                break;
            case 5:
                rv = configData.getI18nText( "Bonus.warp", Integer.valueOf( 10 * tech.getWarp() ) );
                break;
            case 7:
                rv = configData.getI18nText( "Bonus.firepower", Integer.valueOf( 10 * tech.getWeapons() ) );
                break;
            case 8:
                rv = configData.getI18nText( "Bonus.shields", Integer.valueOf( 10 * tech.getShields() ) );
                break;
            case 9:
                rv = configData.getI18nText( "Bonus.structure", Integer.valueOf( 10 * tech.getStructure() ) );
                break;
        }

        return rv;
    }

    private String getPoints( int rowIndex )
    {
        long total;
        if( rowIndex == summaryRow )
        {
            total = getTotalPoints();
        }
        else
        {
            total = (long)getCost( rowIndex ).points();
        }

        return TotalPointDisplay.format( total );
    }

    private Cost getCost( int rowIndex )
    {
        Cost base = Research.values()[rowIndex].getCost();
        int level = getTechValue( rowIndex ).intValue();
        Cost total = TechCalculator.totalCost( level, base );
        return total;
    }

    public void editingCanceled( ChangeEvent e )
    {
    }

    public void editingStopped( ChangeEvent e )
    {
        // update the value column
        fireTableChanged( new TableModelEvent(this, 0, summaryRow, getColumnCount() - 1, TableModelEvent.UPDATE) );
    }

    public long getTotalPoints()
    {
        Cost total = new Cost( 0, 0, 0, 0 );
        for( int row = 0; row < summaryRow; row++ )
        {
            total.add( getCost( row) );
        }
        return (long)total.points();
    }

    public void resetRows( int[] rows ) {
        for(int row : rows) {
            setTechValue("0", row);
        }
        fireTableRowsUpdated(0, getRowCount() - 1);
    }
}
