/****************************************
 *  COPYRIGHT (C) 2012
 *  Holger Graf
 ****************************************/
package sitool.table;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map.Entry;

import javax.swing.table.AbstractTableModel;

import sitool.ConfigData;
import sitool.components.Planet;
import sitool.components.Position;
import sitool.components.TotalPointDisplay;
import sitool.savecalc.Calculator;

/**
 * @author graf
 *
 */
public class TeleporterTableModel extends AbstractTableModel
{
    private static final long serialVersionUID = -3254428293432587724L;

    private ConfigData configData;
    private Calculator calculator;
    private List<Entry<Planet, Planet>> data;
    
    
    public TeleporterTableModel( ConfigData configData )
    {
        this.configData = configData;
        this.calculator = new Calculator( configData );
        this.data = new ArrayList<Entry<Planet, Planet>>();
    }

    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    @Override
    public int getRowCount()
    {
        return data.size();
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    @Override
    public int getColumnCount()
    {
        return 3;
    }

    
    @Override
    public Class<?> getColumnClass( int columnIndex )
    {
        if( columnIndex < 2 )
        {
            return Planet.class;
        }
        return String.class;
    }

    @Override
    public String getColumnName( int columnIndex )
    {
        String rv ="";
        switch( columnIndex )
        {
            case 0:
                rv = configData.getI18nText( "Savecalc.table.start" );
                break;
            case 1:
                rv = configData.getI18nText( "Savecalc.table.end" );
                break;
            case 2:
                rv = configData.getI18nText( "Energy" );
                break;
        }
        return rv;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    @Override
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        Entry<Planet, Planet> entry = data.get( rowIndex );
        long particles = 0;
        switch( columnIndex )
        {
            case 0:
                rv = entry.getKey();
                break;
            case 1:
                rv = entry.getValue();
                break;
            case 2:
                Position source = entry.getKey().getPosition(); 
                Position target = entry.getValue().getPosition(); 
                particles = calculator.getTeleporterPartikel( source, target );
                rv = TotalPointDisplay.format( particles );
                break;
        }
        return rv;
    }

    public void addData( Entry<Planet, Planet> data)
    {
        if( data != null )
        {
            this.data.add( data );
            fireTableDataChanged();
        }
    }
    
    public void removeData( int [] rows )
    {
        if( rows != null && rows.length > 0 )
        {
            Arrays.sort( rows );
            for( int i = rows.length - 1; i >=0; i-- )
            {
                if( i < data.size() )
                    data.remove( rows[i] );
            }
            fireTableDataChanged();
        }
    }
    
    public void removeAll()
    {
        data.clear();
        fireTableDataChanged();
    }
    
}
