/****************************************
 *  COPYRIGHT (C) 2021
 *  Holger Graf
 ****************************************/
package sitool.table;

import javax.swing.table.AbstractTableModel;

import sitool.Account;
import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Ship;
import sitool.components.TotalPointDisplay;
import sitool.components.TotalPointSource;
import sitool.fleet.TacticalCalculator;

/**
 * @author graf
 *
 */
public class TotalShipTableModel extends AbstractTableModel implements TotalPointSource
{
    private static final long serialVersionUID = 9045672684082762031L;

    private final MainFrame mainFrame;
    private final int columnCount = 5;

    public TotalShipTableModel( MainFrame mainFrame )
    {
        this.mainFrame = mainFrame;
    }

    /**
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return columnCount;
    }


    /**
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        int lines = (int)Math.ceil(Ship.ships(mainFrame.getConfigData().getVersion()).length / columnCount);
        return 2 * lines;
    }

    /**
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt( int rowIndex, int columnIndex )
    {
        Object rv = "";
        ConfigData configData = mainFrame.getConfigData();
        int offset = (rowIndex / 2) * columnCount + columnIndex;
        Ship ship = Ship.ships(configData.getVersion())[offset];
        if(rowIndex % 2 == 0) {
            rv = configData.getI18nText(ship.name());
        }
        else {
            Account account = configData.getActiveAccount();
            if( account != null && account.hasTotalShips()) {
                rv = TotalPointDisplay.format( account.getTotalShips().getCount(ship) );
            }
        }
        return rv;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return false;
    }

    @Override
    public long getTotalPoints() {
        long rv = 0;
        Account account = mainFrame.getConfigData().getActiveAccount();
        if( account != null && account.hasTotalShips()) {
            rv = TacticalCalculator.tacticalStrength(account.getTotalShips());
        }
        return rv;
    }

}
