/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.technology;

import sitool.components.Cost;
import sitool.components.Research;

/**
 * @author graf
 *
 */
public class TechCalculator
{
    /**
     * Returns research time in ms
     * @param cost
     * @return
     */
    public static double researchTime( Cost cost )
    {
        double rv = 0;

        rv += 7500.0 * cost.getIron();
        rv += 7500.0 * cost.getMetal();
        rv += 10000.0 * cost.getKryptonite();
        // spice doesn't count
        rv /= 6.0;
        rv = Math.floor( rv );
        return rv;
    }


    public static Cost currentCost( int level, Cost initialCost )
    {
        double base = 2.0;

        double d;
        boolean overflow = false;
        d = Math.pow( base, (level - 1) ) * initialCost.getIron();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long iron = (long)d;
        d = Math.pow( base, (level - 1) ) * initialCost.getMetal();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long metal = (long)d;
        d = Math.pow( base, (level - 1) ) * initialCost.getKryptonite();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long kryptonite = (long)d;
        d = Math.pow( base, (level - 1) ) * initialCost.getSpice();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long spice = (long)d;

        if( overflow )
        {
        	iron = -1;
        	metal = -1;
        	kryptonite = -1;
        	spice = -1;
        }

        Cost rv = new Cost( iron, metal, kryptonite, spice );
        return rv;
    }

    public static Cost totalCost( int level, Cost initialCost )
    {
        double base = 2.0;

        double d;
        boolean overflow = false;
        d = ( Math.pow( base, level ) - 1 ) * initialCost.getIron();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long iron = (long)d;
        d = ( Math.pow( base, level ) - 1 ) * initialCost.getMetal();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long metal = (long)d;
        d = ( Math.pow( base, level ) - 1 ) * initialCost.getKryptonite();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long kryptonite = (long)d;
        d = ( Math.pow( base, level ) - 1 ) * initialCost.getSpice();
        if( d > Long.MAX_VALUE )
        	overflow = true;
        long spice = (long)d;

        if( overflow )
        {
        	iron = -1;
        	metal = -1;
        	kryptonite = -1;
        	spice = -1;
        }

        Cost rv = new Cost( iron, metal, kryptonite, spice );
        return rv;
    }

    public static Cost calculateCost( Research item, int level ) {
        return currentCost(level, item.getCost());
    }
}
