/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.technology;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;
import sitool.components.TotalPointSource;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.SpinnerCellEditor;
import sitool.table.TechnologyTableModel;
import sitool.table.TooltipTable;

/**
 *
 * @author graf
 *
 */
public class TechPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private MainFrame mainFrame;
    private ConfigData configData;
    private JPanel center;
    private TechnologyTableModel model;
    private JTable table;
    private JButton button = new JButton();

    public TechPane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        //center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    protected void actionResetTechs() {
        int[] rows = table.getSelectedRows();
        model.resetRows( rows );
        mainFrame.getDisplay().update();
    }

    public void update()
    {
        if( mainFrame.getConfigData().getActiveAccount() != null )
        {
            center.setVisible( true );
            model.fireTableDataChanged();
        }
        else
        {
            center.setVisible( false );
        }
        button.setText( configData.getI18nText( "TechBox.button" ) );
        updateTable();
    }

    protected JPanel createCenterPanel()
    {
        center = new JPanel( new BorderLayout() );

        model = new TechnologyTableModel( mainFrame );
        mainFrame.registerTotalPointSource( TotalPointSource.Type.Research, model );
        final BlindTableCellRenderer rowRenderer = new BlindTableCellRenderer();
        final TableCellRenderer col1Renderer = new NumericSpinnerTableCellRenderer( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        final TableCellEditor editor = new SpinnerCellEditor( new SpinnerNumberModel( 0, 0, Integer.MAX_VALUE, 1 ) );
        editor.addCellEditorListener( model );
        table = new TooltipTable( model )
        {
            private static final long serialVersionUID = 7149497989459592130L;
            public TableCellRenderer getCellRenderer(int row, int column)
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 0 || modelColumn == 3 || row == TechnologyTableModel.summaryRow )
                {
                    rv = rowRenderer;
                    if( modelColumn == 0 )
                    {
                        rowRenderer.setHorizontalAlignment( SwingConstants.LEFT );
                    }
                    else
                    {
                        rowRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
                    }
                }
                else if( modelColumn == 1 )
                {
                    rv = col1Renderer;
                }
                else
                {
                    rv = super.getCellRenderer( row, column );
                }
                return rv;
            }

            public TableCellEditor getCellEditor( int row, int column )
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                if( modelColumn == 1 )
                {
                    rv = editor;
                }
                else
                {
                    rv = super.getCellEditor( row, column );
                }
                return rv;
            }

        };
        table.setDefaultRenderer( Object.class, new AlternateLineTableCellRenderer() );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        table.getInputMap(WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "resetTechs");
        table.getActionMap().put("resetTechs", new AbstractAction() {
            private static final long serialVersionUID = 5999113634471333070L;

            @Override
            public void actionPerformed( ActionEvent e ) {
                actionResetTechs();
            }
        });

        JPanel tablePanel = new JPanel( new BorderLayout() );
        tablePanel.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        tablePanel.add( table, BorderLayout.NORTH );
        center.add( tablePanel, BorderLayout.CENTER );
        JPanel north = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        north.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        button.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionRead();
            }
        });
        north.add( button );
        center.add( north, BorderLayout.NORTH );

        return center;
    }

    private void updateTable()
    {
        model.fireTableStructureChanged();
        int rowHeight = new JComboBox<>().getPreferredSize().height - 3;
        table.setRowHeight( rowHeight );
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

        TableColumn col = table.getColumnModel().getColumn(0);
        int width = 200;
        col.setPreferredWidth( width );
        col.setMinWidth( width );
        col.setMaxWidth( width );

        col = table.getColumnModel().getColumn(1);
        width = 80;
        col.setPreferredWidth( width );
        col.setMinWidth( width );
        col.setMaxWidth( width );

        width = 90;
        col = table.getColumnModel().getColumn(3);
        col.setPreferredWidth( width );
        col.setMinWidth( width );
        col.setMaxWidth( width );

    }

    protected void actionRead()
    {
        JDialog dlg = new ResearchBox( mainFrame );
        mainFrame.showModalBox( dlg );
    }

}
