/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.technology;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ModulePane;

/**
 * @author graf
 *
 */
public class TechTreePane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 2758582246555940481L;
    private MainFrame mainFrame = null;
    private ConfigData configData;
    private JTabbedPane pane;
    
    protected static final Color enabled  = new Color( 0x44108810, true);
    protected static final Color disabled = new Color( 0x44881010, true);

    private Map<String, Integer> titles = new HashMap<String, Integer>();
    
    public TechTreePane( MainFrame parent )
    {
        mainFrame = parent;
        configData = mainFrame.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( null );
        
        add( createCenterPanel(), BorderLayout.CENTER );
    }

    public void update()
    {
        // propagate update
        Component c = pane.getSelectedComponent();
        if( c instanceof ModulePane )
        {
            ((ModulePane)c).update();
            //System.out.println( "update pane " + c.getClass().getSimpleName() );
        }
        for( Map.Entry<String, Integer> entry : titles.entrySet() )
        {
            int index = entry.getValue().intValue();
            pane.setTitleAt( index, configData.getI18nText( entry.getKey() ) );
        }
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );
        
        String key;
        int index = 0;
        
        pane = new JTabbedPane( JTabbedPane.TOP );
        BuildingTechTreePane buildings = new BuildingTechTreePane( mainFrame );
        key = "statistics.buildings";
        pane.add( buildings, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        ResearchTechTreePane research = new ResearchTechTreePane( mainFrame );
        key = "statistics.research";
        pane.add( research, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        ShipTechTreePane ships = new ShipTechTreePane( mainFrame );
        key = "statistics.ships";
        pane.add( ships, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        DefenseTechTreePane defense = new DefenseTechTreePane( mainFrame );
        key = "statistics.defense";
        pane.add( defense, configData.getI18nText( key ) );
        titles.put( key, Integer.valueOf( index ) );
        index++;

        pane.addChangeListener( new ChangeListener() 
        {
            public void stateChanged( ChangeEvent e )
            {
                update();
            }
        });
        pane.setSelectedIndex( 0 );
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

}
