/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.technology;

import java.util.ArrayList;
import java.util.List;

import sitool.ConfigData;
import sitool.components.Building;
import sitool.components.Defense;
import sitool.components.ListItem;
import sitool.components.Planet;
import sitool.components.Research;
import sitool.components.Ship;
import sitool.components.Stage;
import sitool.components.Technology;

/**
 * @author graf
 *
 */
public class TechtreeDependency
{
    private ConfigData configData;
    private final String disabled = "#8B0000;";
    private final String enabled = "#009400;";

    private List<Planet> planets = null;
    private Technology technology = null;

    public TechtreeDependency( ConfigData configData )
    {
        this.configData = configData;
    }

    public List<Planet> getPlanets()
    {
        return this.planets;
    }

    public void setPlanets( List<Planet> planets )
    {
        this.planets = planets;
    }

    public void updateTechnology(List<ListItem<Research>> tech) {
        try {
            this.technology = configData.getMyTech().clone();
        }
        catch(CloneNotSupportedException e) {}
        for(ListItem<Research> item : tech) {
            this.technology.setTech(item.getItem(), item.getLevel());
        }
    }

    public boolean isEnabled( Building building )
    {
        boolean rv = false;
        switch( building )
        {
            case ironMine:
            case furnace:
            case kryptoniteMine:
            case spiceMine:
            case fusionPlant:
            case developmentCenter:
            case fleetBase:
            case ironStorage:
            case metalStorage:
            case kryptoniteStorage:
            case spiceStorage:
                rv = true;
                break;
            case researchLab:
                rv = hasBuilding( Building.developmentCenter, 2 );
                break;
            case thermosolarPlant:
                rv = hasResearch( Research.energy, 4 ) && hasBuilding( Building.spiceMine, 6 );
                break;
            case weaponsFactory:
                rv = hasResearch( Research.energy, 1 ) && hasBuilding( Building.developmentCenter, 3 );
                break;
            case particleCannon:
                rv = hasResearch( Research.particle, 5 ) && hasBuilding( Building.weaponsFactory, 6 );
                break;
            case particleShield:
                rv = hasResearch( Research.particle, 2 ) && hasBuilding( Building.spiceMine, 6 );
                break;
            case msa:
                rv = hasResearch( Research.energy, 10 ) && hasBuilding( Building.developmentCenter, 12 );
                break;
            case teleporter:
                rv = hasResearch( Research.teleportation, 6 ) && hasBuilding( Building.msa, 2 );
                break;
            case scanner:
                rv = hasAsteroid() && hasBuilding( Building.developmentCenter, 8, true );
                break;
            case tunnel:
                rv = hasBuilding( Building.weaponsFactory, 5 );
                break;
            case superComputer:
                rv = hasBuilding(Building.researchLab, 15);
                break;
        }
        return rv;
    }

    public boolean isEnabled( Research research )
    {
        boolean rv = false;
        switch( research )
        {
            case mining:
                rv = hasResearch( Research.energy, 4 ) && hasBuilding( Building.researchLab, 2 );
                break;
            case energy:
            case drive:
                rv = hasBuilding( Building.researchLab, 1 );
                break;
            case rocket:
                rv = hasResearch( Research.drive, 2 );
                break;
            case diffusion:
                rv = hasResearch( Research.drive, 6 );
                break;
            case warp:
                rv = hasResearch( Research.drive, 8 ) && hasBuilding( Building.researchLab, 8 );
                break;
            case espionage:
            case weapons:
                rv = hasBuilding( Building.researchLab, 3 );
                break;
            case shields:
                rv = hasResearch( Research.energy, 4 ) && hasBuilding( Building.researchLab, 4 );
                break;
            case structure:
                rv = hasBuilding( Building.researchLab, 5 );
                break;
            case target:
                rv = hasResearch( Research.energy, 6 ) && hasBuilding( Building.researchLab, 5 );
                break;
            case particle:
                rv = hasResearch( Research.shields, 5 ) && hasBuilding( Building.researchLab, 6 );
                break;
            case graviton:
                rv = hasResearch( Research.energy, 2 ) && hasBuilding( Building.researchLab, 2 );
                break;
            case teleportation:
                rv = hasBuilding( Building.researchLab, 10 );
                break;
            case blackhole:
                rv = hasResearch( Research.warp, 10 ) && hasResearch( Research.graviton, 10 );
                break;

        }
        return rv;
    }

    public boolean isEnabled( Ship ship )
    {
        boolean rv = false;
        switch( ship )
        {
            case smallTransport:
                rv = hasResearch( Research.rocket, 4 ) && hasBuilding( Building.weaponsFactory, 2 );
                break;
            case largeTransport:
                rv = hasResearch( Research.rocket, 6 ) && hasBuilding( Building.weaponsFactory, 6 );
                break;
            case transmitter:
                rv = hasResearch( Research.diffusion, 8 ) && hasBuilding( Building.weaponsFactory, 8 );
                break;
            case smallFighter:
                rv = hasResearch( Research.rocket, 1 ) && hasBuilding( Building.weaponsFactory, 1 );
                break;
            case manOWar:
                rv = hasResearch( Research.rocket, 4 ) && hasResearch( Research.weapons, 2 ) && hasBuilding( Building.weaponsFactory, 4 );
                break;
            case starCruiser:
                rv = hasResearch( Research.diffusion, 5 ) && hasResearch( Research.shields, 4 ) && hasBuilding( Building.weaponsFactory, 5 );
                break;
            case phoenix:
                rv = hasResearch( Research.diffusion, 8 ) && hasResearch( Research.structure, 5 ) && hasBuilding( Building.weaponsFactory, 7 );
                break;
            case colonyShip:
                rv = hasResearch( Research.rocket, 8 ) && hasBuilding( Building.weaponsFactory, 4 );
                break;
            case solarCell:
                rv = hasBuilding( Building.weaponsFactory, 1 );
                break;
            case probe:
                rv = hasResearch( Research.rocket, 3 ) && hasResearch( Research.espionage, 2 ) && hasBuilding( Building.weaponsFactory, 1 );
                break;
            case recycler:
                rv = hasResearch( Research.diffusion, 4 ) && hasBuilding( Building.weaponsFactory, 4 );
                break;
            case battleship:
                rv = hasResearch( Research.warp, 4 ) && hasBuilding( Building.weaponsFactory, 8 );
                break;
            case stealthBomber:
                rv = hasResearch( Research.warp, 6 ) && hasResearch( Research.espionage, 8 ) && hasBuilding( Building.weaponsFactory, 10 );
                break;
            case destroyer:
                rv = hasResearch( Research.warp, 8 ) && hasBuilding( Building.weaponsFactory, 10 );
                break;
            case imperialStarbase:
                rv = hasResearch( Research.blackhole, 2 ) && hasBuilding( Building.weaponsFactory, 13 );
                break;
            case jammer:
                rv = hasResearch( Research.rocket, 3 ) && hasResearch( Research.espionage, 2 ) && hasBuilding( Building.weaponsFactory, 1 );
                break;
            case empbomb:
                rv = hasResearch( Research.rocket, 6 ) && hasResearch( Research.espionage, 4 ) && hasBuilding( Building.weaponsFactory, 3 );
                break;
            case scanship:
                rv = hasBuilding( Building.weaponsFactory, 1 );
                break;
            default:
                break;
        }
        return rv;
    }

    public boolean isEnabled( Defense defense )
    {
        boolean rv = false;
        switch( defense )
        {
            case mortar:
                rv = hasBuilding( Building.weaponsFactory, 1 );
                break;
            case turboFlak:
                rv = hasResearch( Research.energy, 6 ) && hasBuilding( Building.weaponsFactory, 6 );
                break;
            case gravitonCannon:
                rv = hasResearch( Research.graviton, 2 ) && hasResearch( Research.energy, 3 ) && hasBuilding( Building.weaponsFactory, 3 );
                break;
            case heavyGravitonCannon:
                rv = hasResearch( Research.graviton, 6 ) && hasResearch( Research.target, 5 ) && hasBuilding( Building.weaponsFactory, 5 );
                break;
            case ionGun:
                rv = hasResearch( Research.target, 4 ) && hasBuilding( Building.weaponsFactory, 5 );
                break;
            case positronLighter:
                rv = hasResearch( Research.target, 8 ) && hasResearch( Research.structure, 6 ) && hasBuilding( Building.weaponsFactory, 6 );
                break;
            case intervalCannon:
                rv = hasResearch( Research.particle, 4 ) && hasResearch( Research.target, 10 ) && hasBuilding( Building.weaponsFactory, 7 );
                break;
            case laserBattery:
                rv = hasResearch( Research.target, 10 ) && hasResearch( Research.energy, 8 ) && hasBuilding( Building.weaponsFactory, 8 );
                break;
            case solarGun:
                rv = hasResearch( Research.target, 10 ) && hasResearch( Research.energy, 10 ) && hasBuilding( Building.weaponsFactory, 10 );
                break;
            case smallShieldDome:
                rv = hasResearch( Research.energy, 4 ) && hasResearch( Research.shields, 2 ) && hasBuilding( Building.weaponsFactory, 4 );
                break;
            case largeShieldDome:
                rv = hasResearch( Research.energy, 8 ) && hasResearch( Research.shields, 4 ) && hasBuilding( Building.weaponsFactory, 6 );
                break;
            case solarCell:
                rv = hasBuilding( Building.weaponsFactory, 1 );
                break;
        }
        return rv;
    }

    public String getDependencies( Building building )
    {
        String color;
        StringBuilder sb = new StringBuilder();
        int level;
        Building depBuilding;
        Research depResearch;
        ArrayList<String> rv = new ArrayList<String>();
        switch( building )
        {
            case ironMine:
            case furnace:
            case kryptoniteMine:
            case spiceMine:
            case fusionPlant:
            case developmentCenter:
            case fleetBase:
            case ironStorage:
            case metalStorage:
            case kryptoniteStorage:
            case spiceStorage:
                rv.add( color( " - ", enabled ) );
                break;
            case researchLab:
                level = 2;
                depBuilding = Building.developmentCenter;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case thermosolarPlant:
                level = 4;
                depResearch = Research.energy;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 6;
                depBuilding = Building.spiceMine;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case weaponsFactory:
                level = 1;
                depResearch = Research.energy;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 3;
                depBuilding = Building.developmentCenter;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case particleCannon:
                level = 5;
                depResearch = Research.particle;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 6;
                depBuilding = Building.weaponsFactory;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case particleShield:
                level = 2;
                depResearch = Research.particle;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 6;
                depBuilding = Building.spiceMine;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case msa:
                level = 10;
                depResearch = Research.energy;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 12;
                depBuilding = Building.developmentCenter;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case teleporter:
                level = 6;
                depResearch = Research.teleportation;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 2;
                depBuilding = Building.msa;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case scanner:
                color = ( hasAsteroid() ) ? enabled : disabled;
                rv.add( color( configData.getI18nText( "Asteroid" ), color ) );
                level = 8;
                depBuilding = Building.developmentCenter;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level, true ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case tunnel:
                level = 5;
                depBuilding = Building.weaponsFactory;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case superComputer:
                level = 15;
                depBuilding = Building.researchLab;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
        }
        return format( rv.toArray(new String[rv.size()]) );
    }

    public String getDependencies( Research research )
    {
        String color;
        StringBuilder sb = new StringBuilder();
        int level;
        Building depBuilding;
        Research depResearch;
        ArrayList<String> rv = new ArrayList<String>();
        switch( research )
        {
            case mining:
                level = 4;
                depResearch = Research.energy;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                level = 2;
                depBuilding = Building.researchLab;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case energy:
            case drive:
                level = 1;
                depBuilding = Building.researchLab;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case rocket:
                level = 2;
                depResearch = Research.drive;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case diffusion:
                level = 6;
                depResearch = Research.drive;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case warp:
                depResearch = Research.drive;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.researchLab;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case espionage:
            case weapons:
                depBuilding = Building.researchLab;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case shields:
                depResearch = Research.energy;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.researchLab;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case structure:
                depBuilding = Building.researchLab;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case target:
                depResearch = Research.energy;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.researchLab;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case particle:
                depResearch = Research.shields;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.researchLab;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case graviton:
                depResearch = Research.energy;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.researchLab;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case teleportation:
                depBuilding = Building.researchLab;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case blackhole:
                depResearch = Research.warp;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.graviton;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
        }
        return format( rv.toArray(new String[rv.size()]) );
    }

    public String getDependencies( Ship ship )
    {
        String color;
        StringBuilder sb = new StringBuilder();
        int level;
        Building depBuilding;
        Research depResearch;
        ArrayList<String> rv = new ArrayList<String>();
        switch( ship )
        {
            case smallTransport:
                depResearch = Research.rocket;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case largeTransport:
                depResearch = Research.rocket;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case transmitter:
                depResearch = Research.diffusion;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case smallFighter:
                depResearch = Research.rocket;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case manOWar:
                depResearch = Research.rocket;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.weapons;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case starCruiser:
                depResearch = Research.diffusion;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.shields;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case phoenix:
                depResearch = Research.diffusion;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.structure;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 7;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case colonyShip:
                depResearch = Research.rocket;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case solarCell:
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case probe:
                depResearch = Research.rocket;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.espionage;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case recycler:
                depResearch = Research.diffusion;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case battleship:
                depResearch = Research.warp;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case stealthBomber:
                depResearch = Research.warp;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.espionage;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case destroyer:
                depResearch = Research.warp;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case imperialStarbase:
                depResearch = Research.blackhole;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 13;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case jammer:
                depResearch = Research.rocket;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.espionage;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case empbomb:
                depResearch = Research.rocket;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.espionage;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case scanship:
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            default:
                break;
        }
        return format( rv.toArray(new String[rv.size()]) );
    }

    public String getDependencies( Defense defense )
    {
        String color;
        StringBuilder sb = new StringBuilder();
        int level;
        Building depBuilding;
        Research depResearch;
        ArrayList<String> rv = new ArrayList<String>();
        switch( defense )
        {
            case mortar:
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case turboFlak:
                depResearch = Research.energy;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case gravitonCannon:
                depResearch = Research.energy;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.graviton;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 3;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case heavyGravitonCannon:
                depResearch = Research.target;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.graviton;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case ionGun:
                depResearch = Research.target;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 5;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case positronLighter:
                depResearch = Research.target;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.structure;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case intervalCannon:
                depResearch = Research.particle;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.target;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 7;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case laserBattery:
                depResearch = Research.energy;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.target;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case solarGun:
                depResearch = Research.energy;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.target;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 10;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case smallShieldDome:
                depResearch = Research.energy;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.shields;
                level = 2;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case largeShieldDome:
                depResearch = Research.energy;
                level = 8;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depResearch = Research.shields;
                level = 4;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depResearch.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasResearch( depResearch, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                depBuilding = Building.weaponsFactory;
                level = 6;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
            case solarCell:
                depBuilding = Building.weaponsFactory;
                level = 1;
                sb.setLength( 0 );
                sb.append( configData.getI18nText( depBuilding.name() ) );
                sb.append( " [" );
                sb.append( configData.getI18nText( "Level" ) );
                sb.append( ' ' );
                sb.append( level );
                sb.append( ']' );
                color = ( hasBuilding( depBuilding, level ) ) ? enabled : disabled;
                rv.add( color( sb.toString(), color ) );
                break;
        }
        return format( rv.toArray(new String[rv.size()]) );
    }

    protected String color( String value, String color )
    {
        StringBuilder sb = new StringBuilder( "<span style=\"color:" );
        sb.append( color );
        sb.append( "\">" );
        sb.append( value );
        sb.append( "</span>" );
        return sb.toString();

    }

    protected String format( String... lines )
    {
        StringBuilder sb = new StringBuilder();
        sb.append( "<html>" );
        sb.append( "<p align=\"left\">" );
        for( int i = 0; i < lines.length || i < 1; i++ )
        {
            if( i > 0 )
            {
                sb.append( "<br>" );
            }
            if( i >= lines.length || lines[i].trim().length() == 0 )
            {
                sb.append( "&nbsp;" );
            }
            else
            {
                sb.append( lines[i] );
            }
        }
        sb.append( "</p>" );
        sb.append( "<html>" );
        return sb.toString();
    }

    private boolean hasAsteroid()
    {
        boolean rv = false;
        for( Planet planet : configData.getMyPlanets() )
        {
            if( planet.getAstroSize() > 0 )
            {
                rv = true;
                break;
            }
        }
        return rv;
    }

    private boolean hasBuilding( Building building, int level )
    {
        return hasBuilding( building, level, false );
    }

    private boolean hasBuilding( Building building, int level, boolean astro )
    {
        boolean rv = false;
        List<Planet> myPlanets = ( this.planets != null ) ? this.planets : configData.getMyPlanets();

        for( Planet planet : myPlanets )
        {
            Stage stage = planet.getStage( building, astro );
            if( stage != null &&  stage.getLevel() >= level )
            {
                rv = true;
                break;
            }
        }
        return rv;
    }

    private boolean hasResearch( Research research, int level )
    {
        boolean rv = false;
        Technology tech = (this.technology != null) ? this.technology : configData.getMyTech();
        if( tech.getTech( research ) >= level )
        {
            rv = true;
        }
        return rv;
    }
}
