/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.trade;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.Cost;
import sitool.components.ModulePane;
import sitool.components.Resource;
import sitool.components.TextFieldFocusListener;
import sitool.table.AlternateLineTableCellRenderer;
import sitool.table.BlindTableCellRenderer;
import sitool.table.CellCopyHandler;
import sitool.table.CheckBoxTableCellRenderer;
import sitool.table.NumericSpinnerTableCellRenderer;
import sitool.table.SpinnerCellEditor;
import sitool.table.TooltipTable;
import sitool.table.TradeTableModel;

/**
 * @author graf
 */
public class TradePane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = 7285176454851334424L;

    private JPanel center;
    private ConfigData configData;
    private MainFrame mainFrame;

    private List<JTextField> inputList = new ArrayList<JTextField>();
    private List<JLabel> labelList = new ArrayList<JLabel>();
    private List<JLabel> exchangeList = new ArrayList<JLabel>();
    private TradeTableModel model;
    private JTable table;
    protected SpinnerCellEditor col1editor;


    private JButton calculate = new JButton();
    private JLabel margin = new JLabel();
    private JLabel calculateby = new JLabel();
    private JLabel exchange = new JLabel();
    private JLabel ware = new JLabel();
    private JRadioButton value = new JRadioButton();
    private JRadioButton volume = new JRadioButton();
    
    private JLabel[] exchangeRates = new JLabel[Resource.values().length];

    public TradePane( MainFrame parent )
    {
        mainFrame = parent;
        configData = parent.getConfigData();
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );

        JPanel center = createCenterPanel();
        // center.setPreferredSize( new Dimension( 640, 520 ) );

        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        updateTable();
        updateLabel();
    }

    private void updateLabel()
    {
        margin.setText( configData.getI18nText( "Margin" ) );
        calculateby.setText( configData.getI18nText( "CalculateBy" ) );
        calculate.setText( configData.getI18nText( "Calculate" ) );
        value.setText( configData.getI18nText( "Value" ) );
        volume.setText( configData.getI18nText( "Volume" ) );
        exchange.setText( configData.getI18nText( "ExchangeRate" ) );
        exchange.setHorizontalAlignment( SwingConstants.CENTER );
        ware.setText( configData.getI18nText( "Ware" ) );
        ware.setHorizontalAlignment( SwingConstants.CENTER );
        ware.setBorder( BorderFactory.createEmptyBorder( 2, 5, 10, 10 ) );

        int index = 0;
        for( Resource resource : Resource.values() )
        {
            String text = configData.getI18nText( resource.name() );
            exchangeList.get( index ).setText( text );
            labelList.get( index ).setText( text );
            exchangeRates[index].setText( Integer.toString( resource.getWeight() ) );
            index++;
        }
    }

    private void updateTable()
    {
        boolean isValue = value.isSelected();
        Resource[] resources = Resource.values();
        Cost values = new Cost( 0, 0, 0, 0 );
        for( int i = 0; i < resources.length; i++ )
        {
            JTextField field = inputList.get( i );
            values.setResource( resources[i], getValue( field.getText() ) );
        }
        model.update( isValue, values );
        model.fireTableStructureChanged();
        int rowHeight = new JComboBox().getPreferredSize().height - 3;
        int width;
        table.setRowHeight( rowHeight );
        TableColumnModel columnModel = table.getColumnModel();
        TableColumn column;
        column = columnModel.getColumn( 1 );
        width = 50;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( 2 );
        width = 25;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( 4 );
        width = 45;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( 6 );
        width = 45;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
        column = columnModel.getColumn( 8 );
        width = 45;
        column.setMinWidth( width );
        column.setMaxWidth( width );
        column.setPreferredWidth( width );
    }

    protected JPanel createCenterPanel()
    {
        center = new JPanel( new BorderLayout() );

        JPanel upper = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        center.add( upper, BorderLayout.CENTER );

        int total = Resource.values().length;
        JPanel box = new JPanel();
        box.setLayout( new BoxLayout( box, BoxLayout.Y_AXIS ) );
        upper.add( box );
        box.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        box.add( getInputPanel( total ) );
        box.add( Box.createRigidArea( new Dimension( 100, 50 ) ) );

        JPanel outPanel = new JPanel( new BorderLayout() );
        margin.setHorizontalAlignment( SwingConstants.CENTER );
        outPanel.add( margin, BorderLayout.NORTH );
        JPanel optionPanel = new JPanel( new GridLayout( 3, 1 ) );
        outPanel.add( optionPanel, BorderLayout.WEST );
        optionPanel.add( calculateby );
        ButtonGroup group = new ButtonGroup();
        value.setSelected( true );
        value.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                col1editor = new SpinnerCellEditor( new SpinnerNumberModel( 100, 0, 120, 1 ) );
                update();
            }

        } );
        optionPanel.add( value );
        group.add( value );
        volume.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                col1editor = new SpinnerCellEditor( new SpinnerNumberModel( 100, 0, 100, 1 ) );
                update();
            }

        } );
        optionPanel.add( volume );
        group.add( volume );

        final TableCellRenderer rowHeader = new BlindTableCellRenderer();
        final TableCellRenderer col1renderer = new NumericSpinnerTableCellRenderer(
                        new SpinnerNumberModel( 100, 0, 120, 1 ) );
        Icon lock = ConfigData.getIcon( "/lock.gif" );
        Icon unlock = ConfigData.getIcon( "/unlock.gif" );
        final CheckBoxTableCellRenderer col2renderer = new CheckBoxTableCellRenderer( lock, unlock );
        col2renderer.setHorizontalAlignment( SwingConstants.CENTER );
        col1editor = new SpinnerCellEditor( new SpinnerNumberModel( 100, 0, 120, 1 ) );
        JCheckBox checkBox = new JCheckBox( unlock );
        checkBox.setHorizontalAlignment( SwingConstants.CENTER );
        final TableCellEditor col2editor = new DefaultCellEditor( checkBox );
        model = new TradeTableModel( mainFrame );
        table = new TooltipTable( model ) {
            private static final long serialVersionUID = -8499738896262825180L;

            @Override
            public TableCellEditor getCellEditor( int row, int column )
            {
                TableCellEditor rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );

                switch( modelColumn )
                {
                    case 1:
                        if( row == getRowCount() - 1 )
                        {
                            rv = super.getCellEditor( row, column );
                        }
                        else
                        {
                            rv = col1editor;
                        }
                        break;
                    case 2:
                        if( row == getRowCount() - 1 )
                        {
                            rv = super.getCellEditor( row, column );
                        }
                        else
                        {
                            rv = col2editor;
                        }
                        break;
                    default:
                        rv = super.getCellEditor( row, column );
                }
                return rv;
            }

            @Override
            public TableCellRenderer getCellRenderer( int row, int column )
            {
                TableCellRenderer rv;
                // convert to model
                int modelColumn = convertColumnIndexToModel( column );
                switch( modelColumn )
                {
                    case 0:
                        rv = rowHeader;
                        break;
                    case 1:
                        if( row == getRowCount() - 1 )
                        {
                            rv = super.getCellRenderer( row, column );
                        }
                        else
                        {
                            rv = col1renderer;
                        }
                        break;
                    case 2:
                        if( row == getRowCount() - 1 )
                        {
                            rv = super.getCellRenderer( row, column );
                        }
                        else
                        {
                            rv = col2renderer;
                        }
                        break;
                    default:
                        rv = super.getCellRenderer( row, column );
                }
                return rv;
            }

            public Dimension getPreferredScrollableViewportSize()
            {
                Dimension size = getPreferredSize();
                size.width += 150;
                return size;
            }
        };
        table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        table.addMouseListener( new CellCopyHandler() );
        AlternateLineTableCellRenderer defaultRenderer = new AlternateLineTableCellRenderer();
        defaultRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
        table.setDefaultRenderer( Object.class, defaultRenderer );
        JScrollPane scrollPane = new JScrollPane( table, JScrollPane.VERTICAL_SCROLLBAR_NEVER,
                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        outPanel.add( scrollPane, BorderLayout.CENTER );
        scrollPane.setBorder( BorderFactory.createEmptyBorder( 6, 6, 6, 6 ) );

        box.add( outPanel );
        box.add( Box.createVerticalGlue() );

        JPanel lower = new JPanel( new FlowLayout( FlowLayout.CENTER ) );
        center.add( lower, BorderLayout.SOUTH );

        GridBagConstraints labelConstraints = new GridBagConstraints();
        labelConstraints.gridheight = 1;
        labelConstraints.gridwidth = 1;
        labelConstraints.weightx = 0.0;
        labelConstraints.weighty = 0.0;

        JPanel exchangeRate = new JPanel( new GridBagLayout() );
        lower.add( exchangeRate );

        labelConstraints.gridx = 0;
        labelConstraints.gridy = 0;
        labelConstraints.weightx = 1.0;
        labelConstraints.gridwidth = 2 * total - 1;
        exchangeRate.add( exchange, labelConstraints );
        int x = 0;
        int i = 0;
        labelConstraints.gridwidth = 1;
        JLabel label;
        for( Resource resource : Resource.values() )
        {
            labelConstraints.gridx = x;
            labelConstraints.weightx = 1.0;
            labelConstraints.gridy = 1;
            label = new JLabel();
            exchangeList.add( label );
            exchangeRate.add( label, labelConstraints );
            labelConstraints.gridx = x;
            labelConstraints.gridy = 2;
            exchangeRates[i] = new JLabel( Integer.toString( resource.getWeight() ) ); 
            exchangeRate.add( exchangeRates[i], labelConstraints );
            i++;
            x++;
            if( resource.ordinal() < total - 1 )
            {
                labelConstraints.weightx = 0.0;
                labelConstraints.gridx = x;
                labelConstraints.gridy = 1;
                exchangeRate.add( new JLabel( "  :  " ), labelConstraints );
                labelConstraints.gridx = x;
                labelConstraints.gridy = 2;
                exchangeRate.add( new JLabel( "  :  " ), labelConstraints );
                x++;
            }
        }

        return center;
    }

    JPanel getInputPanel( int total )
    {
        JPanel rv = new JPanel( new BorderLayout() );

        rv.add( ware, BorderLayout.NORTH );
        
        JPanel resourcePanel = new JPanel( new GridLayout( total, 3, 4, 6 ) );
        resourcePanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        rv.add( resourcePanel, BorderLayout.CENTER );
        JTextField textField = new JTextField();;
        Dimension size = textField.getPreferredSize();
        size.width += 160;
        
        ActionListener l = new ActionListener() 
        {
			public void actionPerformed(ActionEvent e)
			{
				update();
			}
        };
        FocusListener f = new FocusAdapter()
        {
            public void focusLost( FocusEvent e )
            {
                update();
            }
            
        };
        FocusListener f2 = new TextFieldFocusListener();
        
        JLabel label;
        for( Resource resource : Resource.values() )
        {
            label = new JLabel();
            label.setHorizontalAlignment( SwingConstants.RIGHT );
            label.setBorder( BorderFactory.createEmptyBorder( 1, 5, 1, 10 ) );
            labelList.add( label );
            resourcePanel.add( label );
            textField.setPreferredSize( size );
            textField.addActionListener( l );
            textField.addFocusListener( f );
            textField.addFocusListener( f2 );
            resourcePanel.add( textField );
            inputList.add( textField );
            if( resource.ordinal() < total - 1 )
            {
                resourcePanel.add( new JLabel() );
            }
            else
            {
                calculate.addActionListener( l );
                JPanel p = new JPanel( new FlowLayout( FlowLayout.LEFT, 5, 0 ) );
                p.add( calculate );
                resourcePanel.add( p );
            }
            textField = new JTextField();
        }
        return rv;
    }

    private long getValue( String in )
    {
        long value;
        StringBuffer sb = new StringBuffer();
        sb.append( in );
        for( int i = sb.length() - 1; i >= 0; i-- )
        {
            char ch = sb.charAt( i );
            if( !Character.isDigit( ch ) )
            {
                sb.deleteCharAt( i );
            }
        }
        // if nothing is left, use '0'
        if( sb.length() == 0 )
        {
            sb.append( '0' );
        }
        value = Long.valueOf( sb.toString() );
        return value;
    }
}
