/****************************************
 *  COPYRIGHT (C) 2008
 *  Holger Graf
 ****************************************/
package sitool.trade;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sitool.ConfigData;
import sitool.MainFrame;
import sitool.components.ButtonTabbedPane;
import sitool.components.ModulePane;

/**
 * @author graf
 *
 */
public class TradeSummaryPane extends JPanel implements ModulePane
{
    private static final long serialVersionUID = -4027491488562178409L;
    protected static final String extension = ".stz";
    private MainFrame mainFrame = null;
    private JTabbedPane pane = new ButtonTabbedPane(ConfigData.getIcon("/close.png"));
    private JPanel center;

    private JButton button = new JButton();
    private int index = 0;


    public TradeSummaryPane( MainFrame parent )
    {
        mainFrame = parent;
        setLayout( new BorderLayout() );
        setBorder( null );
        center = createCenterPanel();
        add( center, BorderLayout.CENTER );
    }

    public void update()
    {
        // propagate update
        Component c = pane.getSelectedComponent();
        if( c instanceof ModulePane )
        {
            ((ModulePane)c).update();
            //System.out.println( "update pane " + c.getClass().getSimpleName() );
        }
        button.setText( mainFrame.getConfigData().getI18nText( "New" ) );
    }

    protected JPanel createCenterPanel()
    {
        JPanel panel = new JPanel( new BorderLayout() );

        JPanel upper = new JPanel( new BorderLayout() );
        upper.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
        JPanel left = new JPanel( new GridLayout( 1, 3, 12, 4 ) );
        left.setBorder( new EmptyBorder( 8, 4, 8, 4) );
        left.add( button );
        button.addActionListener( new ActionListener()
        {
            public void actionPerformed( ActionEvent e )
            {
                actionNew();
            }

        });
        left.add( button );
        upper.add( left, BorderLayout.WEST );
        panel.add( upper, BorderLayout.NORTH );

        actionNew();

        pane.addChangeListener( new ChangeListener()
        {
            public void stateChanged( ChangeEvent e )
            {
                update();
            }
        });
        panel.add( pane, BorderLayout.CENTER );
        return panel;
    }

    protected void actionNew()
    {
        TradePane tradePane = new TradePane( mainFrame );
        pane.addTab( getTitle(), tradePane );
        update();
    }

    public String getTitle()
    {
        if(pane.getTabCount() == 0) {
            index = 0;
        }
        StringBuilder sb = new StringBuilder();
        sb.append( '(' )
          .append(  ++index )
          .append( ')' );
        return sb.toString();
    }
}
